/*$Header: /afs/ece/usr/lowpower/CMUDSP/cvsroot/cmudsp/misc/c-code/coff_dumper/mem_list.c,v 1.1 1998/11/03 18:21:20 inacio Exp $*/
/*****************************************************************************
 *
 *mem_list.c
 *
 *
 *
 *Chris Inacio
 *Carnegie Mellon University
 *January 1997
 *
 *
 *****************************************************************************/
/*
 *$Log: mem_list.c,v $
 *Revision 1.1  1998/11/03 18:21:20  inacio
 *files to build the coff dumper and some test files to run before "releasing"
 *it.  no formal testing for the tool though
 *
 * Revision 1.4  1998/08/27  04:57:11  inacio
 * added the opcode scramble function
 *
 * Revision 1.3  1997/07/25  06:18:20  inacio
 * added some more error checking/handling and I put more information into
 * the print out of the memory files.
 *
 * Revision 1.2  1997/04/29  18:39:32  inacio
 * Changed the way things are added into the list so it works.
 * Took care of the case of adding to the top of the list.
 * cleaned up some random bugs and changed the way it prints.
 *
 * This is the first working version!!
 *
 * Revision 1.1  1997/04/28  16:06:08  inacio
 * Initial revision
 *
 *
 */
 
#include <stdio.h>
#include <stdlib.h>
#include "mem_list.h"


/*
 *file scope variables
 *
 */
static Tmem_list *p_mem = NULL;
static Tmem_list *x_mem = NULL;
static Tmem_list *y_mem = NULL;


/*
 *prototypes
 */
static void dump_a_ram (FILE *ofile, Tmem_list *which_ram);
void die_here (char *file_name, int line_no);


 
/*****************************************************************************
 *add_to_mem
 *
 *this maintains a sorted list for the different memories possible on the
 *56K and holds them sparsely
 *
 *****************************************************************************/
void
add_to_mem (unsigned int address, unsigned int value, Tmem_space mem)
{
	Tmem_list *this_ram = NULL;
	int new_flag = 0;
	Tmem_list *new_cell = NULL, *last_cell = NULL;
	
	/*
	 *determine which RAM it is and if the ram list has been initialized,
	 *point this_ram to the correct ram list.  set flag=1 if it is the first
	 *element in the list
	 */
	switch (mem)
	{
		case pram:
			if (p_mem == NULL)
			{
				p_mem = (Tmem_list *) malloc (sizeof (Tmem_list));
				new_flag = 1;
			}
			if (p_mem == NULL)
			{
				fprintf (stderr, "\nCouldn't get enough memory\n");
				die_here (__FILE__,__LINE__);
			}
			this_ram = p_mem;
			break;
		case xram:
			if (x_mem == NULL)
			{
				x_mem = (Tmem_list *) malloc (sizeof (Tmem_list));
				new_flag = 1;
			}
			if (x_mem == NULL)
			{
				fprintf (stderr, "\nCouldn't get enough memory\n");
				die_here (__FILE__,__LINE__);
			}
			this_ram = x_mem;
			break;
		case yram:
			if (y_mem == NULL)
			{
				y_mem = (Tmem_list *) malloc (sizeof (Tmem_list));
				new_flag = 1;
			}
			if (y_mem == NULL)
			{
				fprintf (stderr, "\nCouldn't get enough memory\n");
				die_here (__FILE__,__LINE__);
			}
			this_ram = y_mem;
			break;
		default:
			this_ram = NULL;
	}

	/*
	 *return if I don't have access to a memory list
	 */
	if (this_ram == NULL && new_flag == 0)
	{
		fprintf (stderr, "I don't have access to a memory list\n");
		fprintf (stderr, "\tfile = \"%s\", line = \"%d\"\n",__FILE__,__LINE__);
		return;
	}

	/*
	 *if it's a new list, we have to handle it specially
	 */
	if (new_flag)
	{
		this_ram->address = address;
		this_ram->value = value;
		this_ram->next = NULL;
		return;
	}


	/*
	 *see if this address already exists....
	 */
	last_cell = this_ram;

	do 
	{
   	if (last_cell->address == address)
   	{
   		/*
   		 *we're replacing a value already in memory
   		 */
   		last_cell->value = value;
   		return;
   	}
	} while ((last_cell = last_cell->next));


	/*
	 *make a new cell to add to the list, because this doesn't exist
	 *already
	 */
	new_cell = (Tmem_list *) malloc (sizeof (Tmem_list));
	if (new_cell == NULL)
	{
		fprintf (stderr, "\n\aCouldn't allocate memory for mem list\n");
		die_here (__FILE__,__LINE__);
	}
	new_cell->address = address;
	new_cell->value = value;


	/*
	 *find the place in the list, because we want to keep it sorted
	 */
	last_cell = this_ram;

	while (last_cell->next)
	{
		if ((address < last_cell->next->address) && (address > last_cell->address))
			break;
		last_cell = last_cell->next;
	}


	/*
	 *make sure it shouldn't become the first thing in the list...
	 *
	 *this sucks, because I have to change a _mem pointer!!
	 *
	 */
	if (address < this_ram->address)
	{
		switch (mem)		
		{
			case pram:
				p_mem = new_cell;
				new_cell->next = this_ram;
				break;
			case xram:
				x_mem = new_cell;
				new_cell->next = this_ram;
				break;
			case yram:
				y_mem = new_cell;
				new_cell->next = this_ram;
				break;
		}

		/*
		 *let's not add in the same cell twice!
		 */
		return;
	}


	/*
	 *add the cell into the list in the right location
	 */
	if (last_cell->next)
	{
		new_cell->next = last_cell->next;
	}
	else
	{
		new_cell->next = NULL;
	}

	last_cell->next = new_cell;


}


/*****************************************************************************
 *dump_lists
 *
 *
 *
 *****************************************************************************/
void
dump_lists ()
{
	FILE *out_file = NULL;
	
	out_file = fopen ("pmem_file","w");
	dump_a_ram (out_file, p_mem);
	fclose (out_file);
	
	out_file = fopen ("xmem_file","w");
	dump_a_ram (out_file, x_mem);
	fclose (out_file);

	out_file = fopen ("ymem_file","w");
	dump_a_ram (out_file, y_mem);
	fclose (out_file);
	
}

/*****************************************************************************
 *dump_a_ram
 *
 *
 *
 *****************************************************************************/
static
void
dump_a_ram (FILE *ofile, Tmem_list *which_ram)
{
	Tmem_list *location = NULL;
	int loop;
	int ms_nibble;
	int sms_nibble;
	int dword;
	int scrambled_data;
	int datum_out;

	location = which_ram;

	/*
	 *check to see we used that RAM
	 */
	if (location == NULL)
		return;


	/*
	 *dump out the start of RAM if we didn't already get it initialized
	 */
	if (location->address > 0)
	{
		for (loop=0; loop < location->address; loop++)
			fprintf (ofile, "%x // 0x%x\n", 0, 0);
	}

	/*
	 *loop through the list of ram locations
	 */
	while (1)
	{
		/*
		 *dump the current location
		 */

      if (which_ram == p_mem)
      {
   		ms_nibble = location->value & 0x0f;
   		ms_nibble = ms_nibble << 20;
   		sms_nibble = location->value & 0xf0;
   		sms_nibble = sms_nibble << 12;
   		dword = location->value & (~0xff) & (0x00ffffff);
   		dword = dword >> 8;
   		scrambled_data = ms_nibble | sms_nibble | dword;
   		datum_out = scrambled_data;
      }
      else
      {
         datum_out = location->value;
      }
      
		fprintf (ofile, "%x // address -> 0x%x value -> 0x%x\n", datum_out, location->address, datum_out);

		/*
		 *if we're at the end of defined RAM then stop trying to go further
		 */
		if (location->next == NULL)
			break;

		/*
		 *fill in the blank spots between the list nodes....
		 */
		if (location->next->address != location->address+1)
		{
			for (loop=location->address+1; loop<location->next->address; loop++)
				fprintf (ofile, "%x // 0x%x\n", 0, 0);
		}
		
		/*
		 *advance to the next location and continue
		 */
		location = location->next;

	}

}

/*****************************************************************************
 *****************************************************************************/
 
