/*$Header: /afs/ece/usr/lowpower/CMUDSP/cvsroot/cmudsp/misc/c-code/coff_dumper/test_main.c,v 1.1 1998/11/03 18:21:22 inacio Exp $ */
/*****************************************************************************
 *
 *
 *
 *
 *
 *Chris Inacio
 *Carnegie Mellon University
 *January 1997
 *
 *
 *****************************************************************************/
/*
 *$Log: test_main.c,v $
 *Revision 1.1  1998/11/03 18:21:22  inacio
 *files to build the coff dumper and some test files to run before "releasing"
 *it.  no formal testing for the tool though
 *
 * Revision 1.3  1997/07/25  15:56:00  inacio
 * added a time date stamp and small banner to the program, to know what version
 * you are using.
 *
 * Revision 1.2  1997/07/25  06:17:39  inacio
 * added hack to get rid of certain undesirable sections
 *
 * Revision 1.1  1997/04/29  18:41:12  inacio
 * Initial revision
 *
 *
 *
 *
 *Log
 * Revision 1.4  1997/01/21  19:07:29  inacio
 * this works, converts the 32-bit stuff read in from the coff file into
 * 24-bit stuff held in the structure for the target_word which is the size
 * of the target word.
 *
 * Revision 1.3  1997/01/21  18:58:04  inacio
 * doesn't copy data from the 32-bit word into the 24-bit structure correctly,
 * but using a memcpy into the point should work....
 *
 * Revision 1.2  1997/01/20  21:50:20  inacio
 * this version reads in coff files including the data chunks and spits them
 * out to the screen, the functions are just waiting for integration into the
 * pli to read coff into varilog
 *
 * Revision 1.1  1997/01/16  17:09:45  inacio
 * Initial revision
 *
 *
 */
 
 
/*
 *
 *includes
 *
 */ 
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <limits.h>
#include "filehdr.h"
#include "aouthdr.h"
#include "scnhdr.h"
#include "dspext.h"
#include "mem_list.h"

/*
 *Macro stuff
 *
 */
#ifndef DEBUG
#define DEBUG 0
#endif

/*
 *this is the default size for the Mot 56002
 */
#ifndef TARGET_WORD_SIZE
#define TARGET_WORD_SIZE 24
#endif

#define BYTE_MASK 0x00FF

/*
 *type definitions
 *
 */
typedef struct target_word_tag
{
	char raw_word[TARGET_WORD_SIZE / CHAR_BIT];
} target_word;


/*
 *
 *prototypes
 *
 */ 
int main (int argc, char *argv[]);
void die_here (char *file_name, int line_no);
void read_header (void);
void read_sections (void);
void read_data (void);
static Tmem_space translate_section_to_mem_space(XCNHDR *section);


#if DEBUG != 0
static void print_section_stats (XCNHDR *s_hdr, int iter);
static void print_raw_data (target_word *raw_data, int data_size);
#endif


/*
 *
 *file globals
 *
 */
static FILE *in_file = NULL;
static unsigned long num_sections = 0;
static unsigned int section_head_offset = 0;
static XCNHDR *section_header = NULL;
static unsigned int code_size = 0; /* in words */
static unsigned int data_size = 0; /* in words */
/*static unsigned int bss_size = 0; /* in words  -- not used!! */
/*static unsigned int start_point = 0; /* program start point -- weird value */

 
/*****************************************************************************
 *main
 *
 *
 *****************************************************************************/
int
main (int argc, char *argv[])
{
	
	fprintf (stdout, "Coff_Text_Dump\n");
	fprintf (stdout, "Chris Inacio <inacio@ece.cmu.edu>\n");
	fprintf (stdout, "version: %s %s\n\n",__DATE__,__TIME__);

	if (argc < 2)
	{
		fprintf (stderr, "\n\aI need a filename argument of a COFF file!\n");
		exit (1);
	}

	if ((in_file = fopen (argv[1], "rb")) == NULL)
	{
		fprintf (stderr, "Could not open file %s\n", argv[1]);
		die_here (__FILE__, __LINE__);
	}


	read_header ();
	read_sections ();
	read_data ();

	dump_lists();

	return 0;
}





/*****************************************************************************
 *read_header
 *
 *This is designed to read in the necessary information for the COFF for
 *further reading, plus make sure its for 5600{1,2} and already linked...
 *I won't handle relocation records!!
 *****************************************************************************/
void
read_header ()
{
	FILHDR	fileheader;
	AOUTHDR  optional_header;
	
	/*
	 *read in the file header
	 *
	 */
	fread (&fileheader, sizeof(fileheader), 1, in_file);
	
	if (ferror(in_file))
	{
		die_here (__FILE__, __LINE__);
	}

	/*
	 *first lets make sure I have the right type of file
	 *
	 *we make sure that it is COFF for Motorola 56000
	 *
	 */
	if (fileheader.f_magic !=	 M56KMAGIC)
	{
		fprintf (stderr, "\a\a\nThis is not a Motorola COFF file for the 56K!\n");
		die_here (__FILE__, __LINE__);
	}


	/*
	 *file is not linked!!!
	 */
	if (!(fileheader.f_flags & F_RELFLG && fileheader.f_flags & F_EXEC))
	{
		fprintf (stderr, "\a\nI can only read linked object files!\n");
		die_here (__FILE__, __LINE__);
	}
	
	/*
	 *
	 *read the optional header
	 *
	 */
	if (fileheader.f_opthdr > 0)
		fread (&optional_header, sizeof (optional_header), 1, in_file);
	
	if (ferror(in_file))
	{
		die_here (__FILE__, __LINE__);
	}
	
	/*
	 *dig out information which we need from the headers...
	 *
	 *since we don't know about debuggers we don't need lots of stuff
	 *like symbols or strings...
	 *
	 */
	num_sections = fileheader.f_nscns;
	section_head_offset = fileheader.f_opthdr + sizeof(fileheader);


	/*
	 *for debug purposes
	 */
#if DEBUG != 0
	fprintf (stdout, "num sections = \"%lu\" section head offset = \"%u\"\n",num_sections, section_head_offset);	
#endif

}


/*****************************************************************************
 *read_sections
 *
 *read in the sections and load them into the right memory locations
 *
 *****************************************************************************/
void
read_sections ()
{
	int rc, loop;
	
	section_header = (XCNHDR *) malloc (sizeof (XCNHDR) * num_sections);
	
	
	/*
	 *position the file pointer to the beginning of the section headers
	 */
	rc = fseek (in_file, section_head_offset, SEEK_SET);
	if (rc)
	{
		fprintf (stderr, "\nCouldn't seek to section headers...\n");
		die_here (__FILE__,__LINE__);
	}


	/*
	 *Read in the section headers...
	 */
	for (loop = 0; loop < num_sections; loop++)
	{
		fread ((section_header+loop),sizeof (XCNHDR), 1, in_file);
		if (ferror(in_file))
		{
			fprintf (stderr, "\nCouldn't read in section header number %d.\n",(loop+1));
			die_here(__FILE__,__LINE__);
		}
	}

	
	/*
	 *debug code, print out the section headers....
	 */
#if DEBUG != 0
	for (loop=0; loop < num_sections; loop++)
	{
		print_section_stats ((section_header+loop), loop);
	}
#endif


}






/*****************************************************************************
 *read_data
 *
 *reads the data sections in
 *
 *Motorola normalizes all of their raw data sections so that each word is
 *normalized to 4 bytes (32-bits) no matter how big the target word size
 *is, but the number things to read is in words!
 *****************************************************************************/
void
read_data ()
{
	int loop, convert_loop, loopi;
	target_word *raw_data = NULL;
	unsigned long *dummy_data = NULL;
	
	
	
	/*
	 *go through the list of sections
	 */
	for (loop=0; loop < num_sections; loop++)
	{
		/*
		 *only do sections with data...
		 */
		if ((section_header+loop)->_s.s_size > 0)
		{
			if ((section_header+loop)->_s.s_flags & STYP_BLOCK)
			{
				fprintf (stderr, "\n\n\aWarning: Block data not yet supported!!\n");
				(section_header+loop)->_s.s_size = 0;
			}
			else
			{
				/*
				 *storage space for our new found data
				 */
         	raw_data = (target_word *) malloc (sizeof (target_word) * (section_header+loop)->_s.s_size );
         	dummy_data = (unsigned long *) malloc (sizeof (unsigned long) *  (section_header+loop)->_s.s_size );

				/*
				 *get the file to the right place to read this stuff in
				 */
         	if (fseek (in_file, (section_header+loop)->_s.s_scnptr+1, SEEK_SET))
         	{
   				fprintf (stderr, "\nCould not seek on input file to read data\n");
   				die_here (__FILE__,__LINE__);
         	}
				
				/*
				 *Motorola normalizes the data by storing any host size in 4 bytes
				 *so we have to read it in that size
				 */
				fread (dummy_data, sizeof (unsigned long), (section_header+loop)->_s.s_size, in_file);
         	if (ferror(in_file))
         	{
   				fprintf (stderr, "\nError reading data section %d.\n",(loop+1));
   				die_here (__FILE__,__LINE__);
         	}

				/*
				 *now let's convert it into host word size stuff!
				 */
				for (convert_loop=0; convert_loop < (section_header+loop)->_s.s_size; convert_loop++)
				{
					for (loopi=0; loopi<TARGET_WORD_SIZE / CHAR_BIT; loopi++)
					{
						char *val;
						
					  	val = (char *) (dummy_data+convert_loop);
					  	(raw_data+convert_loop)->raw_word[loopi] = *(val+loopi);
					  	
					}
				}

			}


		/*
		 *should be storing the data here!
		 */
		/*
		 *added hack, because I don't completely understand the MOT section
		 *flags, and I'm reading crap from sections I shouldn't I guess
		 *
		 *replace the hack with the one from Motorola's own code...
		 *
		 */
		if ((section_header+loop)->_s.s_scnptr && (section_header+loop)->_s.s_size)
		{
			Tmem_space memory_space;
			
			memory_space = translate_section_to_mem_space(section_header+loop);
			for (loopi=0; loopi < (section_header+loop)->_s.s_size; loopi++)
			{
				add_to_mem((CORE_ADDR_ADDRU((section_header+loop)->_s.s_paddr)+loopi),(dummy_data[loopi]>>8),memory_space);
			}

		}



#if DEBUG != 0
		/*
		 *display the data to see if I'm reading anything normal....
		 */
		fprintf (stdout, "\n\nRaw data for section %d:",(loop+1));
		print_raw_data (raw_data, (section_header+loop)->_s.s_size);
#endif
      	
		}

	if (raw_data)
		free (raw_data);
	if (dummy_data)
		free (dummy_data);

	}



}



/*****************************************************************************
 *
 *Debug code and utility routines below here
 *
 *****************************************************************************/




/*****************************************************************************
 *translate_section_to_mem_space
 *
 *take a section header and determine which memory space it is in and return
 *the value for the enumeration
 *
 *****************************************************************************/
static
Tmem_space
translate_section_to_mem_space(XCNHDR *section)
{
	switch (CORE_ADDR_MAP(section->_s.s_paddr))
	{
		case memory_map_p:
		case memory_map_pi:
		case memory_map_pe:
		case memory_map_pr:
			return pram;
			break;
		case memory_map_x:
		case memory_map_xi:
		case memory_map_xe:
		case memory_map_xr:
			return xram;
			break;
		case memory_map_y:
		case memory_map_yi:
		case memory_map_ye:
		case memory_map_yr:
			return yram;
			break;
		case memory_map_l:
		case memory_map_li:
		case memory_map_le:
			fprintf (stderr, "\a\a");
			fprintf (stdout, "\t\tMemory space is \"l\"\n");
			break;
		case memory_map_none:
			fprintf (stderr, "\a\a");
			fprintf (stdout, "\t\tMemory space is \"none\"\n");
			break;
		default:
			fprintf (stderr, "\a\a");
			fprintf (stdout, "\t\t\aMemory space is something weird\n");
	}

	return yram;

}



/*****************************************************************************
 *print_section_stats
 *
 *
 *
 *****************************************************************************/
#if DEBUG != 0
static
void
print_section_stats (XCNHDR *s_hdr, int iter)
{
	fprintf (stdout, "\n\n");
	fprintf (stdout, "Section (%d) name is \"%s\"\n",(iter+1),s_hdr->_s.s_name);
	fprintf (stdout, "\tPhysical Address is \"0x%lx\"\n",CORE_ADDR_ADDRU(s_hdr->_s.s_paddr));
	fprintf (stdout, "\t\tMemory space is \"%d\"\n", CORE_ADDR_MAP(s_hdr->_s.s_paddr));
	switch (CORE_ADDR_MAP(s_hdr->_s.s_paddr))
	{
		case memory_map_p:
		case memory_map_pi:
		case memory_map_pe:
		case memory_map_pr:
			fprintf (stdout, "\t\tMemory space is \"p\"\n");
			break;
		case memory_map_x:
		case memory_map_xi:
		case memory_map_xe:
		case memory_map_xr:
			fprintf (stdout, "\t\tMemory space is \"x\"\n");
			break;
		case memory_map_y:
		case memory_map_yi:
		case memory_map_ye:
		case memory_map_yr:
			fprintf (stdout, "\t\tMemory space is \"y\"\n");
			break;
		case memory_map_l:
		case memory_map_li:
		case memory_map_le:
			fprintf (stdout, "\t\tMemory space is \"l\"\n");
			break;
		case memory_map_none:
			fprintf (stdout, "\t\tMemory space is \"none\"\n");
			break;
		default:
			fprintf (stdout, "\t\t\aMemory space is something weird\n");
	}
	fprintf (stdout, "\tVirtual Address is \"0x%lx\"\n",CORE_ADDR_ADDRU(s_hdr->_s.s_vaddr));
	fprintf (stdout, "\t\tMemory space is \"%d\"\n", CORE_ADDR_MAP(s_hdr->_s.s_vaddr));
	switch (CORE_ADDR_MAP(s_hdr->_s.s_vaddr))
	{
		case memory_map_p:
		case memory_map_pi:
		case memory_map_pe:
		case memory_map_pr:
			fprintf (stdout, "\t\tMemory space is \"p\"\n");
			break;
		case memory_map_x:
		case memory_map_xi:
		case memory_map_xe:
		case memory_map_xr:
			fprintf (stdout, "\t\tMemory space is \"x\"\n");
			break;
		case memory_map_y:
		case memory_map_yi:
		case memory_map_ye:
		case memory_map_yr:
			fprintf (stdout, "\t\tMemory space is \"y\"\n");
			break;
		case memory_map_l:
		case memory_map_li:
		case memory_map_le:
			fprintf (stdout, "\t\tMemory space is \"l\"\n");
			break;
		case memory_map_none:
			fprintf (stdout, "\t\tMemory space is \"none\"\n");
			break;
		default:
			fprintf (stdout, "\t\t\aMemory space is something weird\n");
	}
	fprintf (stdout, "\tSize is \"0x%lx\" words\n",s_hdr->_s.s_size);
	fprintf (stdout, "\tRaw data pointer (in file offset) \"0x%lx\"\n",s_hdr->_s.s_scnptr);
	fprintf (stdout, "\tSection flags \"0x%lx\"\n",s_hdr->_s.s_flags);
	if (s_hdr->_s.s_flags & STYP_REG)
		fprintf (stdout, "\t\tRegular section\n");
	if (s_hdr->_s.s_flags & STYP_DSECT)
		fprintf (stdout, "\t\tDummy section\n");
	if (s_hdr->_s.s_flags & STYP_NOLOAD)
		fprintf (stdout, "\t\tNoload section\n");
	if (s_hdr->_s.s_flags & STYP_GROUP)
		fprintf (stdout, "\t\tGrouped section\n");
	if (s_hdr->_s.s_flags & STYP_PAD)
		fprintf (stdout, "\t\tPadding section\n");
	if (s_hdr->_s.s_flags & STYP_COPY)
		fprintf (stdout, "\t\tCopy section\n");
	if (s_hdr->_s.s_flags & STYP_TEXT)
		fprintf (stdout, "\t\tText section (code)\n");
	if (s_hdr->_s.s_flags & STYP_DATA)
		fprintf (stdout, "\t\tData section\n");
	if (s_hdr->_s.s_flags & STYP_BSS)
		fprintf (stdout, "\t\tBSS section\n");
	if (s_hdr->_s.s_flags & STYP_BLOCK)
		fprintf (stdout, "\t\tBlock section\n");
	if (s_hdr->_s.s_flags & STYP_OVERLAY)
		fprintf (stdout, "\t\tMacro section\n");
	if (s_hdr->_s.s_flags & STYP_BW)
		fprintf (stdout, "\t\tI don't know\n");
	if (s_hdr->_s.s_flags & STYP_OVERLAYP)
		fprintf (stdout, "\t\tOverlay section\n");
	

}
#endif




/*****************************************************************************
 *print_raw_data
 *
 *dumps out the information of the raw data sections
 *****************************************************************************/
#if DEBUG != 0
static 
void 
print_raw_data (target_word *raw_data, int data_size)
{
	int loopo, loopi, loop;
	
#define ROW_SIZE  ((int)(80 / ((TARGET_WORD_SIZE / 4) + 4)) - 1)

	for (loopo = 0; (loopo*ROW_SIZE) < data_size; loopo++)
	{
		fprintf (stdout, "\n");
		for (loopi = 0; loopi < ROW_SIZE; loopi++)
		{
			if ((ROW_SIZE * loopo)+loopi >= data_size)
				break;
			fprintf (stdout, "0x");
			for (loop = 0; loop < TARGET_WORD_SIZE/CHAR_BIT; loop ++)
				fprintf (stdout, "%02x", (raw_data+(loopo*ROW_SIZE)+loopi)->raw_word[loop]);
			fprintf (stdout, "  ");
		}
	}

	fprintf (stdout, "\n");

}
#endif


/*****************************************************************************
 *die_here
 *
 *quick and easy death function
 *
 *****************************************************************************/
void
die_here (char *file_name, int line_no)
{
	char *error_string;
	
	error_string = strerror (errno);
	
	fprintf (stderr, "\n\n");

	perror (error_string);
	
	free (error_string);

	fprintf (stderr, "\a\n\nBombing out here, see ya\n");
	fprintf (stderr, "File: \"%s\", line: \"%d\"\n\n", file_name, line_no);
	
	exit (999);
}



/*****************************************************************************
 *****************************************************************************/
