/*$Header: /afs/ece/usr/lowpower/CMUDSP/cvsroot/cmudsp/misc/c-code/qdvpp/qdhash.c,v 1.3 1999/06/03 12:32:17 inacio Exp $*/
/*****************************************************************************
 *qdhash.c
 *
 * quick and dirty hash to support the quick and dirty Verilog preprocessor
 *
 *
 *Chris Inacio
 *Carnegie Mellon University
 *Copyright (C) 1998
 *
 * $Author: inacio $
 * $Revision: 1.3 $
 * $Date: 1999/06/03 12:32:17 $
 *
 *****************************************************************************/
/*
 *$Log: qdhash.c,v $
 *Revision 1.3  1999/06/03 12:32:17  inacio
 *fixed a bug in the hash_func so that it can no longer generate negative
 *values.  It works with unsigned values now, and the mod function at the end
 *keeps it in range.
 *
 *Revision 1.2  1999/06/02 18:44:39  inacio
 *moved the defines to be in the correct location and added limits.h to the
 *includes for char_bit
 *
 *Revision 1.1  1998/11/03 18:09:14  inacio
 *This is the quick-n-dirty Verilog PreProcessor
 *
 * Revision 1.1  1998/07/29  03:02:52  inacio
 * Initial revision
 *
 *
 */
 

 
/*
 * includes
 */
#include "verilog.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>

 



/*
 * Defines
 */
#define HASH_TABLE_SIZE 4096
#define HASH_DIV 2039
#define HASH_BMAP_SIZE (HASH_TABLE_SIZE / (sizeof(int) * CHAR_BIT))

/*
 * File local variables
 */
static Tdefined_val *hash_table[HASH_TABLE_SIZE];
static int hash_bmap[HASH_BMAP_SIZE];

/*
 * local types
 */
typedef struct Tchained_def_val_tag
{
   Tdefined_val *datum;
   struct Tchained_def_val_tag *next;
} Tchained_def_val;


/*
 * local functions
 */
static int hash_func (char *name);
static void set_hash_bmap (int key);
static int check_hash_bmap (int key);



/*****************************************************************************
 *hash_make_table
 *
 *initializes the hash table
 *****************************************************************************/
void
hash_make_table ()
{
   int loop;
   
   for (loop=0; loop < HASH_TABLE_SIZE; loop++)
   {
      hash_table[loop] = NULL;
   }
   
   for (loop=0; loop < HASH_BMAP_SIZE; loop++)
   {
      hash_bmap[loop] = 0;
   }
   
   return;
}


/*****************************************************************************
 *hash_add
 *
 * adds an item to the hash table
 *****************************************************************************/
void
hash_add (Tdefined_val *in_hash)
{
   int key;
   
   /*
    * find the key
    */
   key = hash_func (in_hash->name);

   /*
    * simple insertion into table
    */
   if (hash_table[key]==NULL)
   {
      hash_table[key] = in_hash;
   }
   else
   {
      /*
       * do chaining!!
       */
       if (check_hash_bmap(key))
       {
         /*
          * already chained
          */
         Tchained_def_val *old_val;
         Tchained_def_val *new_val;
         
         old_val = (Tchained_def_val *) hash_table[key];
         while (old_val->next)
         {
            old_val = old_val->next;
         }
         
         new_val = (Tchained_def_val *) malloc (sizeof(Tchained_def_val));
         new_val->datum = in_hash;
         new_val->next = NULL;
         old_val->next = new_val;
       }
       else
       {
         /*
          * start doing chaining
          */
         Tchained_def_val *old_val;
         Tchained_def_val *new_val;
         
         new_val = (Tchained_def_val *) malloc (sizeof(Tchained_def_val));
         new_val->next = NULL;
         new_val->datum = in_hash;
         old_val = (Tchained_def_val *) malloc (sizeof(Tchained_def_val));
         old_val->datum = hash_table[key];
         old_val->next = new_val;
         hash_table[key] = (Tdefined_val *) old_val;

         /*
          * remember to set the chaining bit on
          */
         set_hash_bmap(key);
       }
   }
}



/*****************************************************************************
 *hash_lookup
 *
 * returns whether or not an item is in the hash table
 *****************************************************************************/
int
hash_lookup (char *check_string)
{
   int key;
   int chained = 0;
   
   key = hash_func (check_string);

   /*
    * value isn't in the hash table
    */
   if (hash_table[key]==NULL)
   {
       return 0;
   }
   
   chained = check_hash_bmap(key);
   
   if (!chained)
   {
      return (!strcmp (hash_table[key]->name,check_string));
   }
   else
   {
      Tdefined_val *item;
      Tchained_def_val *list;
      
      list = (Tchained_def_val *) hash_table[key];
      
      do
      {
         item = list->datum;
         if (!strcmp(item->name, check_string))
         {
            return 1;
         }
         list = list->next;
      } while (list);
      
      return 0;
   }
   
   /*
    * should never get here
    */
   fprintf (stderr, "Shouldn't get to the end of hash_lookup function!\n");
   return 0;
}


/*****************************************************************************
 *hash_func
 *
 * generates the key from the string input
 *****************************************************************************/
static int
hash_func (char *name)
{
   unsigned int char_sum = 0;
   unsigned int char_mac = 1;
   int loop;
   
   for (loop=0; *(name+loop); loop++)
   {
      char_sum += *(name+loop);
      char_mac += char_mac * (*(name+loop));
   }
   char_mac--;
   
   return ((char_mac + char_sum) % HASH_TABLE_SIZE);

}

/*****************************************************************************
 *set_hash_bmap
 *
 * sets a bit in the hash_bmap to tell that chaining is turned on
 *****************************************************************************/
static void
set_hash_bmap (int key)
{
   int which_word;
   int word_bit;
   int mask = 0;
   
   which_word = key / (sizeof(int) * CHAR_BIT);
   word_bit = key % (sizeof(int) * CHAR_BIT);
   mask = 0x01 << word_bit;
   hash_bmap[which_word] |=  mask;
   
}

/*****************************************************************************
 *check_hash_bmap
 *
 * checks a bit in the hash_bmap to determine if chaining is on
 *****************************************************************************/
static int
check_hash_bmap (int key)
{
   int which_word;
   int word_bit;
   int mask = 0;
   
   which_word = key / (sizeof(int) * CHAR_BIT);
   word_bit = key % (sizeof(int) * CHAR_BIT);
   mask = 0x01 << word_bit;
   return (hash_bmap[which_word] & mask);
}


/*****************************************************************************
 *****************************************************************************/
