/*$Header: /afs/ece/usr/lowpower/CMUDSP/cvsroot/cmudsp/misc/c-code/reformat/refor.c,v 1.1 1998/11/03 18:15:09 inacio Exp $*/
/*****************************************************************************
 *refor.c
 *
 *reformats verilog output files so they can diff against Motorola's
 *
 *
 *Chris Inacio
 *Carnegie Mellon University
 *August 1997
 *
 *
 *****************************************************************************/
/*
 *$Log: refor.c,v $
 *Revision 1.1  1998/11/03 18:15:09  inacio
 *This is the reformat program used in comparing the output of the verilog
 *simulations to the output of the Motorola simulator
 *
 * Revision 1.3  1997/08/26  22:58:38  inacio
 * added the ability to get input from stdin and have output go to stdout
 *
 * Revision 1.2  1997/08/21  06:39:08  inacio
 * added time date stamp and some comments
 *
 * Revision 1.1  1997/08/21  06:32:49  inacio
 * Initial revision
 *
 *
 */
 
/*
 *Program constants
 */
#define BUF_SIZE 1024

/*
 *Includes
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
 *Typedefs
 */
typedef struct Tfblock_tag
{
	FILE *infile;
	FILE *outfile;
} Tfblock;


/*
 *prototypes
 */
void main (int argc, char *argv[]);
Tfblock *process_arg (int argc, char *argv[]);


/*****************************************************************************
 *main
 *
 *
 *
 *****************************************************************************/
void
main (int argc, char *argv[])
{
	Tfblock *fblock = NULL;
	char temp_str[BUF_SIZE];
	
	
	/*
	 *process the file name arguments and make sure I get good results
	 */
	fblock = process_arg (argc, argv);
	if (!fblock)
	{
		fprintf (stderr, "\aFile error!\n");
		exit (1);
	}


	/*
	 *loop through the file, processing the lines...
	 */
	do
	{
		fgets (temp_str,BUF_SIZE,fblock->infile);

		/*
		 *strip out any blank lines...
		 */
		if (!strcmp(temp_str,"") || !strcmp(temp_str,"\r") || !strcmp(temp_str,"\n") )
			continue;

		/*
		 *good line to process
		 */
		if (!strncmp(temp_str,"x:$0000",strlen("x:$0000")) || !strncmp(temp_str,"y:$0000",strlen("y:$0000")) )
		{
			char *beg_str = temp_str + strlen("x:$");
			char *mid_str = temp_str + strlen("x:$0000");
			strcpy (beg_str,mid_str);
		}
		fprintf (fblock->outfile,"%s",temp_str);
		
	} while (!feof(fblock->infile));


	fclose (fblock->infile);
	fclose (fblock->outfile);
	
}



/*****************************************************************************
 *process_arg
 *
 *
 *****************************************************************************/
Tfblock *
process_arg (int argc, char *argv[])
{
	Tfblock *fblock = NULL;
	FILE *in_file;
	FILE *out_file;


	/*
	 *print out the version/program banner
	 */
	fprintf (stderr, "%s: version date: %s %s\n",argv[0],__DATE__,__TIME__);
	fprintf (stderr, "Chris Inacio <inacio@ece.cmu.edu>\n");
	fprintf (stderr, "Carnegie Mellon University\n");

	/*
	 *make sure I have enough arguments, print usage if not
	 */
	if (argc!=3)
	{
		fprintf (stderr, "Usage: %s <input file name> <output file name>\n",argv[0]);
		fprintf (stderr, "\tUse \"-\" (without quotes) for stdin or stdout respectively\n");
		return NULL;
	}

	/*
	 *open the input file
	 */
	if (!strcmp(argv[1],"-"))
	{
		in_file = stdin;
	}
	else
	{
		in_file = fopen (argv[1],"r");
   	if (!in_file)
   	{
   		fprintf (stderr, "Error: File \"%s\" does not exist or cannot be opened\n",argv[1]);
   		fprintf (stderr, "\ta valid input file must be provided.\n");
   		return NULL;
   	}
	}

	/*
	 *open the output file
	 */	
	if (!strcmp(argv[2],"-"))
	{
		out_file = stdout;
	}
	else
	{
		out_file = fopen (argv[2],"w");
	}
	if (!out_file)
	{
		fprintf (stderr, "Error: File \"%s\" cannot be created or opened for writing\n",argv[2]);
		fprintf (stderr, "\ta valid output file must be provided.\n");
		fclose (in_file);
		return NULL;
	}

	/*
	 *create the storage pass for the parameter block and store the
	 *file pointers
	 */
	fblock = (Tfblock *) malloc (sizeof (fblock));
	fblock->infile = in_file;
	fblock->outfile = out_file;
	

	return fblock;
}

 
/*****************************************************************************
 *****************************************************************************/
 
