/* $Id: pads_spice.v,v 1.13 1998/08/31 21:51:22 ar39 Exp $ */
/*****************************************************************************
 *
 *
 *
 *
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: ar39 $
 *Date: $Date: 1998/08/31 21:51:22 $
 *Revision: $Revision: 1.13 $
 *
 *
 *Log: $Log: pads_spice.v,v $
 *Log: Revision 1.13  1998/08/31 21:51:22  ar39
 *Log: Makes vec file with correct timing.  :)
 *Log:
 *Log: Revision 1.12  1998/05/08 18:45:58  benk
 *Log: Added more reads from memory to figure out what is going on with extracted spice
 *Log:
# Revision 1.11  1998/04/07  18:12:18  benk
# Back to version with two scan chains
#
# Revision 1.8  1998/03/25  20:04:38  inacio
# added a task that dumps the contents of memory into a file for testing, it's
# a lot easier to use then what was there.
#
# Revision 1.7  1998/03/24  19:52:58  benk
# Changed timing of e_enable going high at end of dsp mode to falling clock edge
#
 *Log: Revision 1.2  1998/03/05 01:00:46  benk
 *Log: Made changes to match new pads hierarchy and other minor changes
 *Log:
# Revision 1.1  1998/02/25  00:17:49  benk
# New files for generating spice test bench
#
# Revision 1.12  1997/12/15  23:03:04  benk
# Verilog files and other stuff to generate test vectors and run ADM simulation
#
# Revision 1.11  1997/12/09  22:07:29  inacio
# got rid of extra precision warnings
#
# Revision 1.10  1997/12/09  21:31:36  inacio
# added synch stuff at the start of mem loading
#
# Revision 1.9  1997/12/09  21:29:07  inacio
# fixed zeroing out the ram
#
# Revision 1.8  1997/12/09  05:32:08  inacio
# removed some display messages, wasn't using them anymore
#
# Revision 1.7  1997/12/09  00:45:24  inacio
# added lots more stuff, like mem dumping to the file, plus the ability to
# easily change the number of cycles for program execution, etc.
#
# Revision 1.6  1997/12/07  04:39:55  inacio
# added some code to initialize the fake memories to 0x0 to be able to make
# sure the writes are working correctly
#
# Revision 1.5  1997/12/05  22:02:42  inacio
# corrected some the parameter typos
#
# Revision 1.4  1997/12/05  19:06:58  inacio
# cleaned up timing to load the chip from the external world, added the new
# extra power and gnd pins to the package
#
# Revision 1.3  1997/11/18  22:48:03  inacio
# fixed when I read the data from the fake memories
#
# Revision 1.2  1997/11/18  22:25:50  inacio
# Added better timing to the reset line; I turn off the write enable line more
# often now.
#
# Revision 1.1  1997/11/18  18:08:01  inacio
# test file for the whole chip behavioral model
#
 *****************************************************************************/


`timescale 1ns / 1ps
`define CLOCK_CYCLE 50

/*
 *
 * set PROG_CYCLES to how many cycles you want to simulate for
 *
 *
 * FIR - 2062
 * FFT - 7200
 * LMS - 51990
 */
`define PROG_CYCLES 25


module thing;


/*
 * variables to do the writing and bookkeeping
 */
   reg [32:0]	clock_count;
   reg [23:0]	dummy_word;
   reg [15:0]	dummy_addr;
   reg [23:0]	dummy_data;
   reg		dummy_ram_enable;
   reg		dummy_ram_write;
   reg [2:0]	vil;
   integer	j, mem_dump, vec_file;
   integer     dumpfile;


   /*
    * some variables and stuff for doing memory dumps
    *
    */
`ifdef DUMP_MEMORY

   /*
    * fake memories to load up the real Cascade memories and store results
    */
   reg [23:0]  temp_P_ram[1023:0];
   reg [23:0]  temp_X_ram[1023:0];
   reg [23:0]  temp_Y_ram[1023:0];

   reg   save_e_enable;
   reg   save_e_write;
   reg   save_e_outen;
   reg   save_edb_input;
   reg   save_eab_input;
   reg   save_e_access;
   


`endif





/*
 * Connections to hook up to the chip, uncomment the reg version if you need
 * to make an assignment to it.
 */
   reg		Clk;
   reg		reset;
   reg		e_enable;
   reg [1:0]	mem_select;
   reg		e_outen;
   reg		e_write;
   reg		test;
   reg 		scanin;

/*
reg	[15:0]eab;
reg	[16:0]edb1;
reg	[23:17]edb2;
reg	scanout;
reg	estrobe;
reg	e_access;
*/



/*
 * Connections to hook up to the chip, uncomment the reg version if you need
 * to make an assignment to it.  Comment out the wire version if you need to
 * make an assignment to it
 */
/*
wire	Clk;
wire 	reset;
wire	e_enable;
wire	[1:0]mem_select;
wire	e_outen;
wire	e_write;
wire	test;
*/
   wire [15:0]	eab;
   wire [23:0]	edb;
   wire [15:0]	eab_in;
   wire [23:0]	edb_in;
   wire		scanout;
   wire		estrobe;
   wire		e_access;

   reg		edb_input, eab_input;   // control for "input" to edb, eab bus
   

/*****************************************************************************
 * function for input line of vec file
 *
 *****************************************************************************/
task vec_input;
   begin
      $fstrobe(vec_file, $time, 
	       " %b %b %b x x %b %b %b %b %b %b %b x ; input = %b %b ",
	       Clk, reset, mem_select, e_enable, 
	       eab_in, edb_in, e_outen, e_write,
	       test, scanin,
	       eab_input, edb_input);
//      $display($time, "writing to vec file");
      
   end
endtask   

/*****************************************************************************
 * write one memory location
 *
 *****************************************************************************/
task write_mem ;
   input [15:0] address;
   input [24:0] data;
   begin
//	$display( $time, "Writing to %b memory: addr: %x  data: %x", mem_select, address, data);
      dummy_addr = address;
      dummy_data = data[23:0];
//      vec_input;
      #10;
      e_write = 1'b0;
      vec_input;
      #(`CLOCK_CYCLE-20);
      e_write = 1'b1;
      vec_input;
      #10;
   end
endtask // write_mem

/*****************************************************************************
 * read one memory location
 *
 *****************************************************************************/
task read_mem ;
   input [15:0] address;
   begin
      // #1;
      dummy_addr = address;
      // vec_input;
      #(`CLOCK_CYCLE) ;
   end
endtask // write_mem



/*****************************************************************************
 *dump_memory_out
 *
 * description: dumps out all of the memories (X, Y, and P) into a file
 *
 * limitations: this assumes that the external ports are in a relatively
 * stable state.  that the control signals are not changing, including
 * e_access
 *
 * parameters: takes one parameter, a string for the file name to dump to
 *
 * side effects: doesn't stop the clock, (like a function would,) it uses
 * timing, this makes the vectors file a lot longer
 *
 *****************************************************************************/
`ifdef DUMP_MEMORY
task dump_mem;
   input [8*25:1] fname;
   
   begin

   /*
    * a couple of diagnostic messages...
    */
   $display ("Reading out the memory . . .\n");
   $display ("Dumping the memory to the %s file . . .\n",fname);


   /*
    * store the state of the chip
    */
   save_e_enable <= e_enable;
   save_e_write <= e_write;
   save_e_outen <= e_outen;
   save_e_access <= e_access;
   save_edb_input <= edb_input;
   save_eab_input <= eab_input;
   

   e_enable <= 1'b1;
   e_write <= 1'b1;
   e_outen <= 1'b0;
   #(`CLOCK_CYCLE * 2);
   edb_input = 0;
   eab_input = 1;
   #(`CLOCK_CYCLE * 2);
   wait (e_access == 1'b0);

   for (j = 0; j < 1024; j = j + 1)
      begin
         for (vil = 0; vil < 3'b100; vil = vil + 1)
            begin
               case (vil)
               2'b00:
                  begin
                     mem_select = vil;
                     #(`CLOCK_CYCLE);
                     dummy_addr = j;
                     #(`CLOCK_CYCLE)
                     // $display ("X RAM setting, vil is %x, eab is %x, dummy_addr is %x, and edb is %x\n",vil, eab, dummy_addr, edb);
                     // $display ("\tmem_select is %x edb_in is %x dummy_data is %x\n",mem_select,edb_in,dummy_data);
			            temp_X_ram[j] = edb;
			            #(`CLOCK_CYCLE);
                  end
               2'b01:
                  begin
                     mem_select = vil;
                     #(`CLOCK_CYCLE * 1/4);
                     dummy_addr = j;
                     #(`CLOCK_CYCLE * 3/4)
                     // $display ("Y RAM setting, vil is %x, eab is %x, dummy_addr is %x, and edb is %x\n",vil, eab, dummy_addr, edb);
			            temp_Y_ram[j] = edb;
                  end
               2'b11:
                  begin
                     mem_select = vil;
                     #(`CLOCK_CYCLE * 1/4);
                     dummy_addr = j;
                     #(`CLOCK_CYCLE * 3/4)
                     // $display ("P RAM setting, vil is %x, eab is %x, dummy_addr is %x, and edb is %x\n",vil, eab, dummy_addr, edb);
			            temp_P_ram[j] = edb;
                  end
		         default:
                     // $display ("NO RAM setting, vil is %x, eab is %x, dummy_addr is %x, and edb is %x\n",vil, eab, dummy_addr, edb);
		               ;
		         endcase
	         end // for (vil = 0; vil < 3'b100; vil = vil + 1)
	      end // for (j = 0; ...
               
   /*
    * Dump the memories to a file
    */
   $display ("Dumping results of memory read...\n");
   mem_dump = $fopen (fname);

   for (j=0; j <= 1024; j = j + 4)
	   $fdisplay (mem_dump, "p:$%x=\t$%x\t$%x\t$%x\t$%x",j,temp_P_ram[j],
	      temp_P_ram[j+1],temp_P_ram[j+2],temp_P_ram[j+3]);

   for (j=0; j <= 1024; j = j + 4)
	   $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x",j,temp_X_ram[j],
	      temp_X_ram[j+1],temp_X_ram[j+2],temp_X_ram[j+3]);
      
   for (j=0; j <= 1024; j = j + 4)
	   $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x",j,temp_Y_ram[j],
	      temp_Y_ram[j+1],temp_Y_ram[j+2],temp_Y_ram[j+3]);
      
   $fclose (mem_dump);



   /*
    * reset the chip operating mode to what it was before we checked out the
    * memory state
    */
   e_enable <= save_e_enable;
   e_write <= save_e_write;
   e_outen <= save_e_outen;
   edb_input <= save_edb_input;
   eab_input <= save_eab_input;
   wait (e_access == save_e_access);


   end
endtask
`endif



/*****************************************************************************
 *instance the chip
 *
 *****************************************************************************/

pads fullchip (Clk, reset, mem_select, estrobe, e_access, e_enable, eab,
	       edb, e_outen, e_write, test, scanin, scanout,
	       pwr0, pwr1, pwr2, pwr3, pwr4, pwr5, pwr6, pwr7, 
	       pwr8, pwr9, gnd0, gnd1, gnd2, gnd3, gnd4, gnd5, gnd6, gnd7, 
	       gnd8, gnd9);

/*****************************************************************************
 *build the assign statements to drive the busses
 *
 *****************************************************************************/
assign
	eab = (eab_input) ? dummy_addr : 16'bz,
	edb = (edb_input) ? dummy_data : 24'bz,
        eab_in = (eab_input) ? dummy_addr : 16'bz,
	edb_in = (edb_input) ? dummy_data : 24'bz;

   
/*****************************************************************************
 *reset the chip and get it ready to load the memories
 *
 *****************************************************************************/
initial
   begin


      /*
       *
       * Probe the chip and build wave forms to view
       *
       */
      $shm_open("wavesCR.shm");
      $shm_probe(thing, "A", fullchip, "AS");
      
      /*
       * open text file for dumping results 
       */
      dumpfile = $fopen("verilogSim.dump");

      /*
       * Set up dump file to hold input vectors for ADM
       */
      vec_file = $fopen("pads.vec.vout");
      /* Vector pattern definition section for ADM vector file */
      $fdisplay(vec_file,
		"radix 1 1 11 1 1 1 1111111111111111 \n",
		"+11111111111111111 1111111 1 1 1 1 \n",
		"+1");
      $fdisplay(vec_file,
		"io    i i ii o o i bbbbbbbbbbbbbbbb \n",
		"+bbbbbbbbbbbbbbbbb bbbbbbb i i i i \n",
		"+o");
      $fdisplay(vec_file,
		"vname CLOCK_PAD  RESET_PAD  MEM_SLCT_PAD[1:0] \n",
		"+ESTROBE_PAD  EACCESS_PAD  EENABLE_PAD  EAB_PAD[15:0] \n",
		"+EDB_PAD[23:0]  E_OUTEN_PAD  E_WRITE_PAD \n",
		"+TEST_PAD  SCAN_IN_PAD  SCAN_OUT_PAD");
      $fdisplay(vec_file,
		"option cbc  ; context based control of bidir pins");
      $fdisplay(vec_file,
		"tunit ns");
      $fdisplay(vec_file,
		"tdelay 5 0 1 11 0 0 1 0000000000000000 \n",
		"+00000000000000000 0000000 \n",
		"+1 1 1 1 0  ; delay data inputs by 3 ns");
      $fdisplay(vec_file,
		"idelay 5 0 0 00 0 0 0 1111111111111111 \n",
		"+11111111111111111 1111111 \n",
		"+0 0 0 0 0  ; delay data inputs by 3 ns");
      $fdisplay(vec_file,
		"slope 0.2");
      $fdisplay(vec_file,
		"vih 3.3");
      $fdisplay(vec_file,
		"vil 0");
      $fdisplay(vec_file,
		"vth 1.6  ;Threshold voltage is a guess");
      $fdisplay(vec_file,
		"; Start vectors");
      
      /*
       * reset the clock counter
       */
      clock_count <= 0;
      
      

      /*
       * signals that I want to have already set when I get ready
       * to run after reset....  Initializes the E bus to load the internal
       * RAMs
       */
      e_enable = 0;
      mem_select = 2'b11;
      dummy_addr = 16'b0;
      dummy_data = 23'b0;
      e_write = 1'b1;
      e_outen = 1'b1;
      test = 1'b0;
      vil = 3'b10;
      dummy_word = 24'hffffff;
      edb_input = 0;
      eab_input = 0;
      reset = 0;
      scanin = 0;
      
      /*
       * reset the chip
       */
      #(`CLOCK_CYCLE) ;
      reset <= 1;
      #(`CLOCK_CYCLE ) ;
      #(`CLOCK_CYCLE / 2) ;
      reset <= 0;


      /*
       * write the values into the memory
       *
       */
      $display ("Loading the memory . . .",$time,"\n");
	   /*
       * Enter memory mode, wait two cycles, then drive busses
       */
      e_enable <= 1;
      #(`CLOCK_CYCLE * 2) ;
      edb_input = 1;
      eab_input = 1;


      /* load program memory */
      mem_select = 2'b11;
      write_mem(16'h0,24'h0af080);
      write_mem(16'h1,24'h000050);
      write_mem(16'h50,24'h300000);
      write_mem(16'h51,24'h0503a0);
/*      
      write_mem(16'h52,24'h340000);
      write_mem(16'h53,24'h0503a4);
      write_mem(16'h54,24'h61f400);
      write_mem(16'h55,24'h000100);
      write_mem(16'h56,24'h65f400);
      write_mem(16'h57,24'h000100);
      write_mem(16'h58,24'h44d900);
      write_mem(16'h59,24'h060081);
      write_mem(16'h5a,24'h00005f);
      write_mem(16'h5b,24'hf01813);
      write_mem(16'h5c,24'h0603a0);
      write_mem(16'h5d,24'hf098d2);
      write_mem(16'h5e,24'h2050d3);
      write_mem(16'h5f,24'hb2b900);
*/
      
      /* load X data memory */
      mem_select = 2'b00;
      write_mem(16'h0100,24'hd81b19);
      write_mem(16'h0001,24'h000000);
      write_mem(16'h0002,24'h000000);
      write_mem(16'h0003,24'h000000);
      write_mem(16'h0101,24'h0be985);

      /* load Y data memory */
      mem_select = 2'b01;
      write_mem(16'h0000,24'h0ccccd);
      write_mem(16'h0001,24'h266666);
      write_mem(16'h0002,24'hf33333);
      write_mem(16'h0003,24'h19999a);
      write_mem(16'h0101,24'h0be985);





      /* SANITY CHECK */
      mem_select = 2'b11;
      edb_input = 0;
      eab_input = 1;
      e_outen = 1'b0;
      read_mem(16'h0);
      read_mem(16'h50);
      mem_select = 2'b00;
      read_mem(16'h0100);
      read_mem(16'h0003);
      mem_select = 2'b01;
      read_mem(16'h0001);
      read_mem(16'h0101);
      
      e_outen = 1'b1;
      mem_select = 2'b11;
      edb_input = 1;
      eab_input = 1;
      write_mem(16'h52,24'h340000);
      write_mem(16'h53,24'h0503a4);
      write_mem(16'h54,24'h61f400);
      write_mem(16'h55,24'h000100);
      write_mem(16'h56,24'h65f400);
      write_mem(16'h57,24'h000100);
      write_mem(16'h58,24'h44d900);
      write_mem(16'h59,24'h060081);
      write_mem(16'h5a,24'h00005f);
      write_mem(16'h5b,24'hf01813);
      write_mem(16'h5c,24'h0603a0);
      write_mem(16'h5d,24'hf098d2);
      write_mem(16'h5e,24'h2050d3);
      write_mem(16'h5f,24'hb2b900);




      

      /*
       * Leave test mode
       */
      e_write = 1'b1;
      e_enable = 0;
      edb_input = 0;
      eab_input = 0;
      /*
       * Snoop XDB bus
       */
      mem_select = 2'b00;

      /*
       * reset the chip and wait for e_access to go high
       */		
      reset = 1;
      #(`CLOCK_CYCLE * 2);
      #(`CLOCK_CYCLE / 2);
      reset = 0;
      #(`CLOCK_CYCLE / 2);

`ifdef DUMP_MEMORY
   dump_mem ("verilog_input_dump");
`endif

       /*
       * Simulate the program
       *
       */
      $display ("Running the program . . . \n");
      #(`CLOCK_CYCLE * `PROG_CYCLES);
      
	       
      /*
       *
       * read out the physical memories
       *
       */
      $display ("Dumping the memory . . .\n");
      #(`CLOCK_CYCLE / 2);
      e_enable <= 1'b1;
      #(`CLOCK_CYCLE / 2);
      #(`CLOCK_CYCLE * 2);
      e_write <= 1'b1;
      e_outen <= 1'b0;
      edb_input = 0;
      eab_input = 1;

      /* read X data memory */
      mem_select = 2'b00;
      read_mem(16'h0000); // Data X(1)
      read_mem(16'h0003); // Data X(0)

      /* read Y data memory */
      mem_select = 2'b01;
      read_mem(16'h0100);  // data from first iteration of FIR
      read_mem(16'h0101);  // data I wrote in before
      read_mem(16'h0000);  // 1st coefficient
      read_mem(16'h0001);  // 2nd coefficient
      read_mem(16'h0002);  // 3rd coefficient
      read_mem(16'h0003);  // 4th coefficient
      #(`CLOCK_CYCLE * 2);

      $shm_close;           /* close the dump file */

      $fclose(vec_file);    /* close the vector file */
            
      $fclose(dumpfile);    /* close the dump file */
            
      $finish;

   end


/*****************************************************************************
 *build the clock signal
 *
 *****************************************************************************/

/*
always
   begin
      #5 ;
      // Check ESTROBE output only
      $fstrobe(vec_file, $time, 
	       " %b %b %b %b x %b %b %b %b %b %b %b x ; input = %b %b ",
	       Clk, reset,
               mem_select, estrobe, e_enable, 
	       eab_in, edb_in, e_outen, e_write,
	       test, scanin,
	       eab_input, edb_input);

      #(`CLOCK_CYCLE/2 - 10) if (~Clk)
      // Outputs to vec file (except ESTROBE)
      $fstrobe(vec_file, $time, 
	       " %b %b %b x %b %b %b %b %b %b %b %b %b ; input = %b %b ",
	       Clk, reset, 
	       mem_select , e_access, e_enable, 
	       eab, edb, e_outen, e_write,
	       test, 
	       scanin, scanout,
	       eab_input, edb_input);
      
      #5 Clk <= 1;
      vec_input;

      #5 ;
      // Check ESTROBE output only
      $fstrobe(vec_file, $time, 
	       " %b %b %b %b x %b %b %b %b %b %b %b x ; input = %b %b ",
	       Clk, reset, mem_select, estrobe, e_enable, 
	       eab_in, edb_in, e_outen, e_write,
	       test, scanin,
	       eab_input, edb_input);

      #(`CLOCK_CYCLE/2 - 5) Clk <= 0;
      clock_count = clock_count + 1;
      vec_input;
      // $display ("Clock is now: %x\n",clock_count);
*/

always
  begin
      #1;
      $fstrobe(vec_file, $time, 
	       " %b %b %b %b %b %b %b %b %b %b %b %b %b ; input = %b %b #2",
	       Clk, reset, 
	       mem_select, estrobe, e_access, e_enable, 
	       eab, edb, e_outen, e_write,
	       test, scanin, scanout,
	       eab_input, edb_input);
  end
      

always
   begin
      // Check ESTROBE output only
/*
      $fstrobe(vec_file, $time, 
	       " %b %b %b %b x %b %b %b %b %b %b %b x ; input = %b %b #1",
	       Clk, reset,
               mem_select, estrobe, e_enable, 
	       eab_in, edb_in, e_outen, e_write,
	       test, scanin,
	       eab_input, edb_input);
*/
      #(`CLOCK_CYCLE/2);
      Clk <= 1;
      // Outputs to vec file (except ESTROBE)
/*
      $fstrobe(vec_file, $time, 
	       " %b %b %b %b %b %b %b %b %b %b %b %b %b ; input = %b %b #2",
	       Clk, reset, 
	       mem_select, estrobe, e_access, e_enable, 
	       eab, edb, e_outen, e_write,
	       test, scanin, scanout,
	       eab_input, edb_input);
*/
      #(`CLOCK_CYCLE/2)
      Clk <= 0;
      clock_count = clock_count + 1;
      // $display ("Clock is now: %x\n",clock_count);

`ifdef CYCLE_DUMP
      /* dump contents of architected registers */
      /* to send only to a file, use dumpfile instead of */
      /* dumpfile_and_stdout */
      $display(
		"\nCore signals: \n\t PAB = %h \t XAB = %h \t YAB = %h \n",
		fullchip.pads_core.processor_C_PAB, 
		fullchip.pads_core.processor_C_XAB, 
		fullchip.pads_core.processor_C_YAB, 
		"\t PDB = %h \t XDB = %h \t YDB = %h \n",
		fullchip.pads_core.processor_C_PDB, 
		fullchip.pads_core.processor_C_XDB, 
		fullchip.pads_core.processor_C_YDB, 

		"PCU Registers: \n",
		"\t PC = %h \n",
		fullchip.pads_core.processor_dsp_PCU_PC, 	
		
		"ALU Registers: \n",
		"\t x = %h  \ty = %h\n",
		{fullchip.pads_core.processor_dsp_ALU.x1, 
		 fullchip.pads_core.processor_dsp_ALU.x0},
		{fullchip.pads_core.processor_dsp_ALU.y1, 
		 fullchip.pads_core.processor_dsp_ALU.y0},
		"\t A = %h \t B = %h \n",
		{fullchip.pads_core.processor_dsp_ALU.a2, 
		 fullchip.pads_core.processor_dsp_ALU.a1, 
		 fullchip.pads_core.processor_dsp_ALU.a0},
		{fullchip.pads_core.processor_dsp_ALU.b2, 
		 fullchip.pads_core.processor_dsp_ALU.b1, 
		 fullchip.pads_core.processor_dsp_ALU.b0},

		"next",$time,
		"\n\n"
		);
`endif // ifdef CYCLE_DUMP
      
   end
			


endmodule	



/*****************************************************************************
 *****************************************************************************/
