/* $Id: agu_control_wrapper.v,v 1.10 1998/08/19 20:03:33 yftong Exp $ */
/*****************************************************************************
 *agu_control_wrapper
 *
 * moves the agu_control from stage2 to stage1.  To do this, all the outputs
 * have to be latched at the end of stage1 to be passed on to stage 2.
 *
 * To keep the variables distinct the naming convention for this file is:
 * acw_ in front of a variable name means agu_control_wrapper
 * ac_ in front of a variable name means agu_control
 *
 * There are 72 scanable elements in this file, including the 1 scanable DFF
 * in the immediate file, but not including the scanable elements in the
 * agu_control block.
 *
 *
 *Copyright (C) 1998 Carnegie Mellon University
 *
 *Author: $Author: yftong $
 *Date: $Date: 1998/08/19 20:03:33 $
 *Revision: $Revision: 1.10 $
 *
 *
 *Log: $Log: agu_control_wrapper.v,v $
 *Log: Revision 1.10  1998/08/19 20:03:33  yftong
 *Log: added in DesignWare components into the structural descriptions
 *Log:
# Revision 1.9  1998/06/16  20:19:53  inacio
# added comments about the scan chain, be careful to keep them up to date.
# fixed some formatting too.
#
# Revision 1.8  1998/06/15  22:48:08  benk
# Fixed another bug, this time a clock not hooked up properly
#
# Revision 1.7  1998/06/15  22:36:55  benk
# Fixed CVS related bug
#
# Revision 1.6  1998/06/15  22:23:54  inacio
# I added in the scan chain to the flip flops in the wrapper unit.  I merged
# in Andrew's extra clocks after adding the scan chain, and I added Ben's
# immediate flip flop to be scannable too.  I have not tested this version,
# but I did test it before the merge.  The scan chain is known to have
# problems, but that shouldn't effect normal operation.
#
# Revision 1.5  1998/06/15  22:06:18  ar39
# Fixed a bad clock on the last line.
#
 *Log: Revision 1.4  1998/06/15 21:52:07  benk
 *Log: Added stage 2 immediate coming out of AGU for use by other units.
 *Log: Not used by others yet.
 *Log:
# Revision 1.3  1998/06/15  20:04:48  ar39
# Removed spurious net.
#
 *Log: Revision 1.2  1998/06/15 17:02:10  ar39
 *Log: Split more AGU clocks--whoo-hoo!
 *Log:
 *Log: Revision 1.1  1998/06/09 19:50:00  inacio
 *Log: This file wraps the agu_control file, the control logic to move it from
 *Log: stage2 to stage1 to speed up the agu alu.
 *Log:
 *****************************************************************************/



module agu_control_wrapper (Clk_0, Clk_1, reset, pdb2, rep, alt_pdb,
   scan_in, test, scan_out, xab_select, yab_select, PAB_2_select, xwrite,
   xread, ywrite, yread, PABwrite_2, gdb_out_select, gdb_write,
   addr_alu_H_N_or_1, addr_alu_H_add_sub, addr_alu_L_N_or_1,
   addr_alu_L_add_sub, Rin1_H_select, Rraddr1_H_select, Rwaddr1_H_select,
   Rwrite1_H_2, Rin1_L_select, Rraddr1_L_select, Rwaddr1_L_select,
   Rwrite1_L_2, Nraddr1_H_select, Nraddr1_L_select, Mraddr1_H_select,
   Mraddr1_L_select, Rraddr2_L_select, Rraddr2_H_select, Nraddr2_L_select,
   Nraddr2_H_select, Mraddr2_L_select, Mraddr2_H_select, Rin2_H_select,
   Rwaddr2_H_select, Rwrite2_H_3, Rin2_L_select, Rwaddr2_L_select,
   Rwrite2_L_3, Nin2_H_select, Nwaddr2_H_select, Nwrite2_H_3, Nin2_L_select,
   Nwaddr2_L_select, Nwrite2_L_3, Min2_H_select, Mwaddr2_H_select,
   Mwrite2_H_3, Min2_L_select, Mwaddr2_L_select, Mwrite2_L_3,
   wrap_once_L_ena, wrap_once_H_ena, addr_aluL_ena, addr_aluH_ena,
   immediate3);


/*
 * ---------------
 * pin directions
 * --------------
 */
input Clk_0, Clk_1;
input reset;
input [23:0] pdb2;
input rep;
input [23:0] alt_pdb;


input scan_in;
input test;
output scan_out;

output [2:0] xab_select;
output [2:0] yab_select;
output [1:0] PAB_2_select;
output xwrite;
output xread;
output ywrite;
output yread;
output PABwrite_2;
output [2:0] gdb_out_select;
output gdb_write;
output addr_alu_H_N_or_1;
output addr_alu_H_add_sub;
output addr_alu_L_N_or_1;
output addr_alu_L_add_sub;
output [1:0] Rin1_H_select;
output Rraddr1_H_select;
output [1:0] Rwaddr1_H_select;
output Rwrite1_H_2;
output [1:0] Rin1_L_select;
output Rraddr1_L_select;
output [1:0] Rwaddr1_L_select;
output Rwrite1_L_2;
output Nraddr1_H_select;
output Nraddr1_L_select;
output Mraddr1_H_select;
output Mraddr1_L_select;
output [1:0] Rraddr2_L_select;
output [1:0] Rraddr2_H_select;
output [1:0] Nraddr2_L_select;
output [1:0] Nraddr2_H_select;
output [1:0] Mraddr2_L_select;
output [1:0] Mraddr2_H_select;

output Rin2_H_select;
output Rwaddr2_H_select;
output Rwrite2_H_3;
output Rin2_L_select;
output Rwaddr2_L_select;
output Rwrite2_L_3;
output Nin2_H_select;
output Nwaddr2_H_select;
output Nwrite2_H_3;
output Nin2_L_select;
output Nwaddr2_L_select;
output Nwrite2_L_3;
output Min2_H_select;
output Mwaddr2_H_select;
output Mwrite2_H_3;
output Min2_L_select;
output Mwaddr2_L_select;
output Mwrite2_L_3;
   

/*
 * guard latches controls
 */
output addr_aluL_ena;
output addr_aluH_ena;
output wrap_once_L_ena;
output wrap_once_H_ena;

/*
 * Stage 3 immediate control signal for other units
 */
output   immediate3;


/*
 * ----------------------------------
 * all the wires in the world, Yuck!
 * ----------------------------------
 */
wire inv_reset;
wire immediate;
wire imm_scan_aguctrl;

/*
 * all the wires for hooking up the AGU control logic
 */
wire [23:0] ac_pdb2;

wire [2:0] ac_xab_select;
wire [2:0] ac_yab_select;
wire [1:0] ac_PAB_2_select;
wire ac_xwrite;
wire ac_xread;
wire ac_ywrite;
wire ac_yread;
wire ac_PABwrite_2;
wire [2:0] ac_gdb_out_select;
wire ac_gdb_write;
wire ac_addr_alu_H_N_or_1;
wire ac_addr_alu_H_add_sub;
wire ac_addr_alu_L_N_or_1;
wire ac_addr_alu_L_add_sub;
wire [1:0] ac_Rin1_H_select;
wire ac_Rraddr1_H_select;
wire [1:0] ac_Rwaddr1_H_select;
wire ac_Rwrite1_H_2;
wire [1:0] ac_Rin1_L_select;
wire ac_Rraddr1_L_select;
wire [1:0] ac_Rwaddr1_L_select;
wire ac_Rwrite1_L_2;
wire ac_Nraddr1_H_select;
wire ac_Nraddr1_L_select;
wire ac_Mraddr1_H_select;
wire ac_Mraddr1_L_select;
wire [1:0] ac_Rraddr2_L_select;
wire [1:0] ac_Rraddr2_H_select;
wire [1:0] ac_Nraddr2_L_select;
wire [1:0] ac_Nraddr2_H_select;
wire [1:0] ac_Mraddr2_L_select;
wire [1:0] ac_Mraddr2_H_select;

wire ac_Rin2_H_select;
wire ac_Rwaddr2_H_select;
wire ac_Rwrite2_H_3;
wire ac_Rin2_L_select;
wire ac_Rwaddr2_L_select;
wire ac_Rwrite2_L_3;
wire ac_Nin2_H_select;
wire ac_Nwaddr2_H_select;
wire ac_Nwrite2_H_3;
wire ac_Nin2_L_select;
wire ac_Nwaddr2_L_select;
wire ac_Nwrite2_L_3;
wire ac_Min2_H_select;
wire ac_Mwaddr2_H_select;
wire ac_Mwrite2_H_3;
wire ac_Min2_L_select;
wire ac_Mwaddr2_L_select;
wire ac_Mwrite2_L_3;


/*
 * guard latches controls
 */
wire ac_addr_aluL_ena;
wire ac_addr_aluH_ena;
wire ac_wrap_once_L_ena;
wire ac_wrap_once_H_ena;

/*
 * Stage 3 immediate control signal for other units
 */
wire   immediate3;


/*
 * make the agu_control unit inside here
 *
 * Notes about the parameters:
 *
 * Generally they are prefixed with "ac_" which means they are the original
 *    stage2 control logic versions.  The "ac_" prfixed ones should go into
 *    agu_control.  (Read the notes at the top of the file for further info.
 *
 * The 8'b1111_1111 replaced addr_modifier_out.  This signal was used to
 *    control a guard latch in front of the second adder in the AGU ALU, but
 *    replacing the logic and flip flops to do this is impossible and move
 *    the logic into stage 1; so we set the input to all 1's making all the
 *    control points turn on the extra adder.  (We burn a little power and
 *    get a lot of time.)
 *
 * The scan chain input to the agu_control logic is immediate3, the last
 *    scanable DFF at the bottom of this file.
 *
 */
agu_control agu_control (Clk_0, reset, ac_pdb2, immediate, immediate3,
   test, scan_out, ac_xab_select, ac_yab_select, ac_PAB_2_select, ac_xwrite,
   ac_xread, ac_ywrite, ac_yread, ac_PABwrite_2, ac_gdb_out_select,
   ac_gdb_write, ac_addr_alu_H_N_or_1, ac_addr_alu_H_add_sub,
   ac_addr_alu_L_N_or_1, ac_addr_alu_L_add_sub, ac_Rin1_H_select,
   ac_Rraddr1_H_select, ac_Rwaddr1_H_select, ac_Rwrite1_H_2,
   ac_Rin1_L_select, ac_Rraddr1_L_select, ac_Rwaddr1_L_select,
   ac_Rwrite1_L_2, ac_Nraddr1_H_select, ac_Nraddr1_L_select,
   ac_Mraddr1_H_select, ac_Mraddr1_L_select, ac_Rraddr2_L_select,
   ac_Rraddr2_H_select, ac_Nraddr2_L_select, ac_Nraddr2_H_select,
   ac_Mraddr2_L_select, ac_Mraddr2_H_select, ac_Rin2_H_select,
   ac_Rwaddr2_H_select, ac_Rwrite2_H_3, ac_Rin2_L_select,
   ac_Rwaddr2_L_select, ac_Rwrite2_L_3, ac_Nin2_H_select,
   ac_Nwaddr2_H_select, ac_Nwrite2_H_3, ac_Nin2_L_select,
   ac_Nwaddr2_L_select, ac_Nwrite2_L_3, ac_Min2_H_select,
   ac_Mwaddr2_H_select, ac_Mwrite2_H_3, ac_Min2_L_select,
   ac_Mwaddr2_L_select, ac_Mwrite2_L_3, 8'b1111_1111,
   ac_wrap_once_L_ena, ac_wrap_once_H_ena, ac_addr_aluL_ena,
   ac_addr_aluH_ena);


/*
 * generate an immediate signal for the AGU
 */
check_imm immediate_checker (
   .Clk (Clk_0),
   .reset (reset),
   .pdb (ac_pdb2),
   .immediate (immediate),
   .scan_in (scan_in),
   .test (test),
   .scan_out (imm_scan_aguctrl)
   );




/*
 * Some extra glue logic to connect things together
 *
 */
`ifdef DW
   assign inv_reset = ~reset;
   DW01_mux_any #(48, 1, 24) select_output
   ({alt_pdb, pdb2}, rep, ac_pdb2);
`else   
   stdinv reset_inverter (reset, inv_reset);
   mux2 #(24, 0) pdb_selector (pdb2, alt_pdb, rep, ac_pdb2);
`endif


/*
 * D Flip Flop Instantiations
 *
 * dff_c (Clock, Clear, D, Q)
 *
 * options:
 *    N -> number of bits
 *    DPFLAG -> datapath flag (0 - not datapath, 1 - datapath, 2 - ??)
 *    GROUP -> group name e.g. "dpath"
 *    BUFFER_SIZE -> buffer size (1-6) e.g. "3"
 *    OPTIMIZE -> what to optimize for (speed, standard, lowpower) e.g. "speed"
 *
 * dffs_c (Clock, Clear, D, Scanin, Test, Q)
 *
 */

/*
 *Make an extra set of flip-flops for all the outputs
 */
`ifdef DW
   DW03_reg_s_pl #(3, 0) acw_xab_select_ltch (ac_xab_select, Clk_0, inv_reset, 1'b1, xab_select);
   DW03_reg_s_pl #(3, 0) acw_yab_select_ltch (ac_yab_select, Clk_0, inv_reset, 1'b1, yab_select);
   DW03_reg_s_pl #(2, 0) acw_PAB_2_select_ltch (ac_PAB_2_select, Clk_0, inv_reset, 1'b1, PAB_2_select);
   DW03_reg_s_pl #(1, 0) acw_xwrite_ltch (ac_xwrite, Clk_0, inv_reset, 1'b1, xwrite);
   DW03_reg_s_pl #(1, 0) acw_xread_ltch (ac_xread, Clk_0, inv_reset, 1'b1, xread);
   DW03_reg_s_pl #(1, 0) acw_ywrite_ltch (ac_ywrite, Clk_0, inv_reset, 1'b1, ywrite);
   DW03_reg_s_pl #(1, 0) acw_yread_ltch (ac_yread, Clk_0, inv_reset, 1'b1, yread);
   DW03_reg_s_pl #(1, 0) acw_PABwrite_2_ltch (ac_PABwrite_2, Clk_0, inv_reset, 1'b1, PABwrite_2);
   DW03_reg_s_pl #(3, 0) acw_gdb_out_select_ltch (ac_gdb_out_select, Clk_0, inv_reset, 1'b1, gdb_out_select);
   DW03_reg_s_pl #(1, 0) acw_gdb_write (ac_gdb_write, Clk_0, inv_reset, 1'b1, gdb_write);
   DW03_reg_s_pl #(1, 0) acw_addr_alu_H_N_or_1 (ac_addr_alu_H_N_or_1, Clk_0, inv_reset, 1'b1, addr_alu_H_N_or_1);
   DW03_reg_s_pl #(1, 0) acw_addr_alu_H_add_sub (ac_addr_alu_H_add_sub, Clk_0, inv_reset, 1'b1, addr_alu_H_add_sub);
   DW03_reg_s_pl #(1, 0) acw_addr_alu_L_N_or_1 (ac_addr_alu_L_N_or_1, Clk_0, inv_reset, 1'b1, addr_alu_L_N_or_1);
   DW03_reg_s_pl #(1, 0) acw_addr_alu_L_add_sub (ac_addr_alu_L_add_sub, Clk_1, inv_reset, 1'b1, addr_alu_L_add_sub);
   DW03_reg_s_pl #(2, 0) acw_Rin1_H_select (ac_Rin1_H_select, Clk_1, inv_reset, 1'b1, Rin1_H_select);
   DW03_reg_s_pl #(1, 0) acw_Rraddr1_H_select (ac_Rraddr1_H_select, Clk_1, inv_reset, 1'b1, Rraddr1_H_select);
   DW03_reg_s_pl #(2, 0) acw_Rwaddr1_H_select (ac_Rwaddr1_H_select, Clk_1, inv_reset, 1'b1, Rwaddr1_H_select);
   DW03_reg_s_pl #(1, 0) acw_Rwrite1_H_2 (ac_Rwrite1_H_2, Clk_1, inv_reset, 1'b1, Rwrite1_H_2);
   DW03_reg_s_pl #(2, 0) acw_Rin1_L_select (ac_Rin1_L_select, Clk_1, inv_reset, 1'b1, Rin1_L_select);
   DW03_reg_s_pl #(1, 0) acw_Rraddr1_L_select (ac_Rraddr1_L_select, Clk_1, inv_reset, 1'b1, Rraddr1_L_select);
   DW03_reg_s_pl #(2, 0) acw_Rwaddr1_L_select (ac_Rwaddr1_L_select, Clk_1, inv_reset, 1'b1, Rwaddr1_L_select);
   DW03_reg_s_pl #(1, 0) acw_Rwrite1_L_2 (ac_Rwrite1_L_2,Clk_1, inv_reset, 1'b1, Rwrite1_L_2);
   DW03_reg_s_pl #(1, 0) acw_Nraddr1_H_select (ac_Nraddr1_H_select, Clk_1, inv_reset, 1'b1, Nraddr1_H_select);
   DW03_reg_s_pl #(1, 0) acw_Nraddr1_L_select (ac_Nraddr1_L_select, Clk_1, inv_reset, 1'b1, Nraddr1_L_select);
   DW03_reg_s_pl #(1, 0) acw_Mraddr1_H_select (ac_Mraddr1_H_select, Clk_1, inv_reset, 1'b1, Mraddr1_H_select);
   DW03_reg_s_pl #(1, 0) acw_Mraddr1_L_select (ac_Mraddr1_L_select, Clk_1, inv_reset, 1'b1, Mraddr1_L_select);
   DW03_reg_s_pl #(2, 0) acw_Rraddr2_L_select (ac_Rraddr2_L_select, Clk_1, inv_reset, 1'b1, Rraddr2_L_select);
   DW03_reg_s_pl #(2, 0) acw_Rraddr2_H_select (ac_Rraddr2_H_select, Clk_1, inv_reset, 1'b1, Rraddr2_H_select);
   DW03_reg_s_pl #(2, 0) acw_Nraddr2_L_select (ac_Nraddr2_L_select, Clk_1, inv_reset, 1'b1, Nraddr2_L_select);
   DW03_reg_s_pl #(2, 0) acw_Nraddr2_H_select (ac_Nraddr2_H_select, Clk_1, inv_reset, 1'b1, Nraddr2_H_select);
   DW03_reg_s_pl #(2, 0) acw_Mraddr2_L_select (ac_Mraddr2_L_select, Clk_1, inv_reset, 1'b1, Mraddr2_L_select);
   DW03_reg_s_pl #(2, 0) acw_Mraddr2_H_select (ac_Mraddr2_H_select, Clk_1, inv_reset, 1'b1, Mraddr2_H_select);
   DW03_reg_s_pl #(1, 0) acw_Rin2_H_select (ac_Rin2_H_select, Clk_1, inv_reset, 1'b1, Rin2_H_select);
   DW03_reg_s_pl #(1, 0) acw_Rwaddr2_H_select (ac_Rwaddr2_H_select, Clk_1, inv_reset, 1'b1, Rwaddr2_H_select);
   DW03_reg_s_pl #(1, 0) acw_Rwrite2_H_3 (ac_Rwrite2_H_3, Clk_1, inv_reset, 1'b1, Rwrite2_H_3);
   DW03_reg_s_pl #(1, 0) acw_Rin2_L_select (ac_Rin2_L_select, Clk_1, inv_reset, 1'b1, Rin2_L_select);
   DW03_reg_s_pl #(1, 0) acw_Rwaddr2_L_select (ac_Rwaddr2_L_select, Clk_1, inv_reset, 1'b1, Rwaddr2_L_select);
   DW03_reg_s_pl #(1, 0) acw_Rwrite2_L_3 (ac_Rwrite2_L_3, Clk_1, inv_reset, 1'b1, Rwrite2_L_3);
   DW03_reg_s_pl #(1, 0) acw_Nin2_H_select (ac_Nin2_H_select, Clk_1, inv_reset, 1'b1, Nin2_H_select);
   DW03_reg_s_pl #(1, 0) acw_Nwaddr2_H_select (ac_Nwaddr2_H_select, Clk_1, inv_reset, 1'b1, Nwaddr2_H_select);
   DW03_reg_s_pl #(1, 0) acw_Nwrite2_H_3 (ac_Nwrite2_H_3, Clk_1, inv_reset, 1'b1, Nwrite2_H_3);
   DW03_reg_s_pl #(1, 0) acw_Nin2_L_select (ac_Nin2_L_select, Clk_1, inv_reset, 1'b1, Nin2_L_select);
   DW03_reg_s_pl #(1, 0) acw_Nwaddr2_L_select (ac_Nwaddr2_L_select, Clk_1, inv_reset, 1'b1, Nwaddr2_L_select);
   DW03_reg_s_pl #(1, 0) acw_Nwrite2_L_3 (ac_Nwrite2_L_3, Clk_1, inv_reset, 1'b1, Nwrite2_L_3);
   DW03_reg_s_pl #(1, 0) acw_Min2_H_select (ac_Min2_H_select, Clk_1, inv_reset, 1'b1, Min2_H_select);
   DW03_reg_s_pl #(1, 0) acw_Mwaddr2_H_select (ac_Mwaddr2_H_select, Clk_1, inv_reset, 1'b1, Mwaddr2_H_select);
   DW03_reg_s_pl #(1, 0) acw_Mwrite2_H_3 (ac_Mwrite2_H_3, Clk_1, inv_reset, 1'b1, Mwrite2_H_3);
   DW03_reg_s_pl #(1, 0) acw_Min2_L_select (ac_Min2_L_select, Clk_1, inv_reset, 1'b1, Min2_L_select);
   DW03_reg_s_pl #(1, 0) acw_Mwaddr2_L_select (ac_Mwaddr2_L_select, Clk_1, inv_reset, 1'b1, Mwaddr2_L_select);
   DW03_reg_s_pl #(1, 0) acw_Mwrite2_L_3 (ac_Mwrite2_L_3, Clk_1, inv_reset, 1'b1, Mwrite2_L_3);
   DW03_reg_s_pl #(1, 0) acw_wrap_once_L_ena (ac_wrap_once_L_ena, Clk_1, inv_reset, 1'b1, wrap_once_L_ena);
   DW03_reg_s_pl #(1, 0) acw_wrap_once_H_ena (ac_wrap_once_H_ena, Clk_1, inv_reset, 1'b1, wrap_once_H_ena);
   DW03_reg_s_pl #(1, 0) acw_addr_aluL_ena (ac_addr_aluL_ena, Clk_1, inv_reset, 1'b1, addr_aluL_ena);
   DW03_reg_s_pl #(1, 0) acw_addr_aluH_ena (ac_addr_aluH_ena, Clk_1, inv_reset, 1'b1, addr_aluH_ena);
   DW03_reg_s_pl #(1, 0) immediate_ltch (immediate, Clk_1, inv_reset, 1'b1, immediate3);
   
`else   
   
   dffs_c #(3, 0) acw_xab_select_ltch (Clk_0, inv_reset, ac_xab_select, imm_scan_aguctrl, test, xab_select);
   dffs_c #(3, 0) acw_yab_select_ltch (Clk_0, inv_reset, ac_yab_select, xab_select[2], test, yab_select);
   dffs_c #(2, 0) acw_PAB_2_select_ltch (Clk_0, inv_reset, ac_PAB_2_select, yab_select[2], test, PAB_2_select);
   dffs_c #(1, 0) acw_xwrite_ltch (Clk_0, inv_reset, ac_xwrite, PAB_2_select[1], test, xwrite);
   dffs_c #(1, 0) acw_xread_ltch (Clk_0, inv_reset, ac_xread, xwrite, test, xread);
   dffs_c #(1, 0) acw_ywrite_ltch (Clk_0, inv_reset, ac_ywrite, xread, test, ywrite);
   dffs_c #(1, 0) acw_yread_ltch (Clk_0, inv_reset, ac_yread, ywrite, test, yread);
   dffs_c #(1, 0) acw_PABwrite_2_ltch (Clk_0, inv_reset, ac_PABwrite_2, yread, test, PABwrite_2);
   dffs_c #(3, 0) acw_gdb_out_select_ltch (Clk_0, inv_reset, ac_gdb_out_select, PABwrite_2, test, gdb_out_select);
   dffs_c #(1, 0) acw_gdb_write (Clk_0, inv_reset, ac_gdb_write, gdb_out_select[2], test, gdb_write);
   dffs_c #(1, 0) acw_addr_alu_H_N_or_1 (Clk_0, inv_reset, ac_addr_alu_H_N_or_1, gdb_write, test, addr_alu_H_N_or_1);
   dffs_c #(1, 0) acw_addr_alu_H_add_sub (Clk_0, inv_reset, ac_addr_alu_H_add_sub, addr_alu_H_N_or_1, test, addr_alu_H_add_sub);
   dffs_c #(1, 0) acw_addr_alu_L_N_or_1 (Clk_0, inv_reset, ac_addr_alu_L_N_or_1, addr_alu_H_add_sub, test, addr_alu_L_N_or_1);
   dffs_c #(1, 0) acw_addr_alu_L_add_sub (Clk_1, inv_reset, ac_addr_alu_L_add_sub, addr_alu_L_N_or_1, test, addr_alu_L_add_sub);
   dffs_c #(2, 0) acw_Rin1_H_select (Clk_1, inv_reset, ac_Rin1_H_select, addr_alu_L_add_sub, test, Rin1_H_select);
   dffs_c #(1, 0) acw_Rraddr1_H_select (Clk_1, inv_reset, ac_Rraddr1_H_select, Rin1_H_select[1], test, Rraddr1_H_select);
   dffs_c #(2, 0) acw_Rwaddr1_H_select (Clk_1, inv_reset, ac_Rwaddr1_H_select, Rraddr1_H_select, test, Rwaddr1_H_select);
   dffs_c #(1, 0) acw_Rwrite1_H_2 (Clk_1, inv_reset, ac_Rwrite1_H_2, Rwaddr1_H_select[1], test, Rwrite1_H_2);
   dffs_c #(2, 0) acw_Rin1_L_select (Clk_1, inv_reset, ac_Rin1_L_select, Rwrite1_H_2, test, Rin1_L_select);
   dffs_c #(1, 0) acw_Rraddr1_L_select (Clk_1, inv_reset, ac_Rraddr1_L_select, Rin1_L_select[1], test, Rraddr1_L_select);
   dffs_c #(2, 0) acw_Rwaddr1_L_select (Clk_1, inv_reset, ac_Rwaddr1_L_select, Rraddr1_L_select, test, Rwaddr1_L_select);
   dffs_c #(1, 0) acw_Rwrite1_L_2 (Clk_1, inv_reset, ac_Rwrite1_L_2, Rwaddr1_L_select[1], test, Rwrite1_L_2);
   dffs_c #(1, 0) acw_Nraddr1_H_select (Clk_1, inv_reset, ac_Nraddr1_H_select, Rwrite1_L_2, test, Nraddr1_H_select);
   dffs_c #(1, 0) acw_Nraddr1_L_select (Clk_1, inv_reset, ac_Nraddr1_L_select, Nraddr1_H_select, test, Nraddr1_L_select);
   dffs_c #(1, 0) acw_Mraddr1_H_select (Clk_1, inv_reset, ac_Mraddr1_H_select, Nraddr1_L_select, test, Mraddr1_H_select);
   dffs_c #(1, 0) acw_Mraddr1_L_select (Clk_1, inv_reset, ac_Mraddr1_L_select, Mraddr1_H_select, test, Mraddr1_L_select);
   dffs_c #(2, 0) acw_Rraddr2_L_select (Clk_1, inv_reset, ac_Rraddr2_L_select, Mraddr1_L_select, test, Rraddr2_L_select);
   dffs_c #(2, 0) acw_Rraddr2_H_select (Clk_1, inv_reset, ac_Rraddr2_H_select, Rraddr2_L_select[1], test, Rraddr2_H_select);
   dffs_c #(2, 0) acw_Nraddr2_L_select (Clk_1, inv_reset, ac_Nraddr2_L_select, Rraddr2_H_select[1], test, Nraddr2_L_select);
   dffs_c #(2, 0) acw_Nraddr2_H_select (Clk_1, inv_reset, ac_Nraddr2_H_select, Nraddr2_L_select[1], test, Nraddr2_H_select);
   dffs_c #(2, 0) acw_Mraddr2_L_select (Clk_1, inv_reset, ac_Mraddr2_L_select, Nraddr2_H_select[1], test, Mraddr2_L_select);
   dffs_c #(2, 0) acw_Mraddr2_H_select (Clk_1, inv_reset, ac_Mraddr2_H_select, Mraddr2_L_select[1], test, Mraddr2_H_select);
   dffs_c #(1, 0) acw_Rin2_H_select (Clk_1, inv_reset, ac_Rin2_H_select, Mraddr2_H_select[1], test, Rin2_H_select);
   dffs_c #(1, 0) acw_Rwaddr2_H_select (Clk_1, inv_reset, ac_Rwaddr2_H_select, Rin2_H_select, test, Rwaddr2_H_select);
   dffs_c #(1, 0) acw_Rwrite2_H_3 (Clk_1, inv_reset, ac_Rwrite2_H_3, Rwaddr2_H_select, test, Rwrite2_H_3);
   dffs_c #(1, 0) acw_Rin2_L_select (Clk_1, inv_reset, ac_Rin2_L_select, Rwrite2_H_3, test, Rin2_L_select);
   dffs_c #(1, 0) acw_Rwaddr2_L_select (Clk_1, inv_reset, ac_Rwaddr2_L_select, Rin2_L_select, test, Rwaddr2_L_select);
   dffs_c #(1, 0) acw_Rwrite2_L_3 (Clk_1, inv_reset, ac_Rwrite2_L_3, Rwaddr2_L_select, test, Rwrite2_L_3);
   dffs_c #(1, 0) acw_Nin2_H_select (Clk_1, inv_reset, ac_Nin2_H_select, Rwrite2_L_3, test, Nin2_H_select);
   dffs_c #(1, 0) acw_Nwaddr2_H_select (Clk_1, inv_reset, ac_Nwaddr2_H_select, Nin2_H_select, test, Nwaddr2_H_select);
   dffs_c #(1, 0) acw_Nwrite2_H_3 (Clk_1, inv_reset, ac_Nwrite2_H_3, Nwaddr2_H_select, test, Nwrite2_H_3);
   dffs_c #(1, 0) acw_Nin2_L_select (Clk_1, inv_reset, ac_Nin2_L_select, Nwrite2_H_3, test, Nin2_L_select);
   dffs_c #(1, 0) acw_Nwaddr2_L_select (Clk_1, inv_reset, ac_Nwaddr2_L_select, Nin2_L_select, test, Nwaddr2_L_select);
   dffs_c #(1, 0) acw_Nwrite2_L_3 (Clk_1, inv_reset, ac_Nwrite2_L_3, Nwaddr2_L_select, test, Nwrite2_L_3);
   dffs_c #(1, 0) acw_Min2_H_select (Clk_1, inv_reset, ac_Min2_H_select, Nwrite2_L_3, test, Min2_H_select);
   dffs_c #(1, 0) acw_Mwaddr2_H_select (Clk_1, inv_reset, ac_Mwaddr2_H_select, Min2_H_select, test, Mwaddr2_H_select);
   dffs_c #(1, 0) acw_Mwrite2_H_3 (Clk_1, inv_reset, ac_Mwrite2_H_3, Mwaddr2_H_select, test, Mwrite2_H_3);
   dffs_c #(1, 0) acw_Min2_L_select (Clk_1, inv_reset, ac_Min2_L_select, Mwrite2_H_3, test, Min2_L_select);
   dffs_c #(1, 0) acw_Mwaddr2_L_select (Clk_1, inv_reset, ac_Mwaddr2_L_select, Min2_L_select, test, Mwaddr2_L_select);
   dffs_c #(1, 0) acw_Mwrite2_L_3 (Clk_1, inv_reset, ac_Mwrite2_L_3, Mwaddr2_L_select, test, Mwrite2_L_3);
   dffs_c #(1, 0) acw_wrap_once_L_ena (Clk_1, inv_reset, ac_wrap_once_L_ena, Mwrite2_L_3, test, wrap_once_L_ena);
   dffs_c #(1, 0) acw_wrap_once_H_ena (Clk_1, inv_reset, ac_wrap_once_H_ena, wrap_once_L_ena, test, wrap_once_H_ena);
   dffs_c #(1, 0) acw_addr_aluL_ena (Clk_1, inv_reset, ac_addr_aluL_ena, wrap_once_H_ena, test, addr_aluL_ena);
   dffs_c #(1, 0) acw_addr_aluH_ena (Clk_1, inv_reset, ac_addr_aluH_ena, addr_aluL_ena, test, addr_aluH_ena);
   dffs_c #(1, 0) immediate_ltch (Clk_1, inv_reset, immediate, addr_aluH_ena, test, immediate3);

`endif

endmodule
