/*	File:	address_mode.v	  					*/

/*	module name: address_mode					*/

/*	Description: Generates the control signals to choose the correct addressing mode out of the address-alu. 	*/
/*				 The three addressing modes are: linear, wrap-around-once (modulo) and multiple-wrap-around (modulo) addressing. */
/* added the bit reverse addressing mode -- Jonathan Tong 6/23 */

/* Author: Nitzan Weinberg , Jonathan Tong  */


/*==============================================================================*/


/********************************************************************************/
/*																				*/
/*	module:	address_mode														*/
/*																				*/
/********************************************************************************/

module address_mode (
	Mn,
	address_mode_select
	);


/*==============================================================================*/
/*============	I/O direction	================================================*/
/*==============================================================================*/

input [15:0] Mn;

output [1:0] address_mode_select;


/*==============================================================================*/
/*============		I/O type	================================================*/
/*==============================================================================*/

wire [15:0] Mn;

reg [1:0] address_mode_select;
/* (address_mode_select = 2'b00) => (linear)				
   (address_mode_select = 2'b01) => (wrap_once_addr)		
   (address_mode_select = 2'b10) => (multi_wrap_addr)		
   (address_mode_select = 2'b11) => (bit_reverse)    */


/*==============================================================================*/
/*===========	Internal Nets	================================================*/
/*==============================================================================*/



/*==============================================================================*/
/*	Processes																	*/
/*==============================================================================*/

always @(	Mn or
			address_mode_select
		)

	begin	
		if ( Mn == 16'h0000)
			/*----------------------*/
			/* bit reverse          */
			/*----------------------*/
			address_mode_select = 2'b11;
		else if ( Mn[15] == 1'b0)
			/*------------------*/
			/* wrap around ONCE */
			/*------------------*/
			address_mode_select = 2'b01;

		else if ( (Mn[15:12] >= 4'h8) && (Mn[15:12] <= 4'hb) )
			/*----------------------*/
			/* multiple wrap around */
			/*----------------------*/
			address_mode_select = 2'b10;
			
		else	
			/*--------*/
			/* linear */
			/*--------*/
			address_mode_select = 2'b00;

	end		/* always */


endmodule		/* end module address_mode */
