/*	File:	XYregImm_c.v	  					*/

/*	module name: XYregImm						*/

/*	Description: XYregImm */

/*	 register for X, Y registers with input mux for immediate data	*/
/*	Instantiates Cascade modules	*/


/*======================================================================*/


`ifdef SIM
   `include "../cmudsp/structural/globals.v"
`else
   `include "./verilog/globals.v"
`endif
   



/************************************************************************/
/*									*/
/*	module:								*/
/*									*/
/************************************************************************/

module XYregImm_c (
	Clk, reset,
	DB,
	imm,
	pdb2,
	in_ctl,
	write,
	q
	);


/*============	I/O direction	========================================*/

input			Clk, reset;
input [`databus]	DB;		/* from XDB or YDB */
input [`databus]	pdb2;		/* immediate data from pdb2 */
input [7:0]		imm;		/* short immediate data from pdb3 */
input [1:0]	 	in_ctl;
input 		 	write;
output [`databus]	q;

/*=======================================================================*/


/*============		parameters	=================================*/

parameter dbgroup = "xyreg";

/*============		I/O type	=================================*/

wire [`databus]	 q;

/*================ internal signals =====================================*/

wire [`databus] 	mux_out;
wire [15:0]		imm_wide;


/*======================================================================*/
/*	processes							*/
/*======================================================================*/

/* mux to select inputs, sign extend immediate data */

assign imm_wide = {16{imm[7]}};

`ifdef DW
   DW01_mux_any #(96, 2, 24) MUX ({pdb2, pdb2, {imm, 16'b0}, DB }, in_ctl, mux_out);
`else   
   mux3 #(24,1, dbgroup) MUX (
	DB, {imm, 16'b0}, pdb2, in_ctl[0], in_ctl[1], mux_out
	);
`endif

/* enable register.  Reset, enable are active low */
`ifdef DW   
   DW03_reg_s_pl #(24, 0) reg2 (mux_out, Clk, reset, ~write, q);   
`else 
   dffh_c #(24,1,dbgroup) DFF (
	Clk, reset, mux_out, write, q
	);
`endif

endmodule	/* end module */
