/*	File:	logic_unit.v	  						*/

/*	module name: logic_unit						*/

/*	Description: logic_unit */

/*		Perform 24 bit logical operations	*/


/*==============================================================================*/


/* `include "/afs/ece.cmu.edu/usr/lowpower/dsp/alu/globals.v" */

`ifdef SIM
   `include "../cmudsp/structural/globals.v"
`else
   `include "./verilog/globals.v"
`endif


/********************************************************************************/
/*																				*/
/*	module:																		*/
/*																				*/
/********************************************************************************/

module logic_c (
	c,		/* control signals */
	in1, in2,
	out
	);


/*============	I/O direction	================================================*/

input [2:0]			c;	
input [`databus] 	in1, in2;
output [`databus]	out;

/*==============================================================================*/



/*============		I/O type	================================================*/

wire [`databus]	 out;

/*===============	Internal Nets	=============================================*/

wire [`databus] l_and, l_or, l_eor, l_not;
wire [`databus] lsl, rol, lsr, ror;
wire [`databus] logic, shift;		/* intermediate mux outputs */
	

/*==============================================================================*/
/*	processes																	*/
/*==============================================================================*/

/* create signals for each possible function */

`ifdef DW
   assign l_and = in1 & in2;
   assign l_or = in1 | in2;
   assign l_eor = in1 ^ in2;
   assign l_not = ~in1;
`else
   and2 #(24, 1, "dp24") AND2 (
	in1, in2, l_and);

   or2 #(24, 1, "dp24") OR2 (
	in1, in2, l_or);

   xor2 #(24, 1, "dp24") XOR2 (
	in1, in2, l_eor);

   inv #(24, 1, "dp24") INV (
	in1, l_not);
`endif
	
assign lsr = {1'b0,in1[23:1]};	
				
assign ror = {in1[0],in1[23:1]};	

assign lsl = {in1[22:0], 1'b0};	

assign rol = {in1[22:0],in1[23]};	

/* first level muxes: choose among logic and shift functions */
`ifdef DW
   DW01_mux_any #(96, 2, 24) LOGIC_MUX (
	{l_not, l_and, l_eor, l_or}, {c[1], c[0]}, logic);
`else
   mux4 #(24, 1, "dp24") LOGIC_MUX (
	l_or, l_eor, l_and, l_not, c[0], c[1], logic);
`endif

`ifdef DW
   DW01_mux_any #(96, 2, 24) SHIFT_MUX (
        {rol, lsl, ror, lsr}, {c[1], c[0]}, shift);
`else    
   mux4 #(24, 1, "dp24") SHIFT_MUX (
	lsr, ror, lsl, rol, c[0], c[1], shift);
`endif	

/* second level mux: choose logic or shift function */
`ifdef DW
   DW01_mux_any #(48, 1, 24) FINAL_MUX ({shift, logic}, c[2], out);
`else 
   mux2 #(24, 1, "dp24") FINAL_MUX (
	logic, shift, c[2], out);
`endif	


endmodule	/* end module */
