/* $Id: addr_bus_control.v,v 1.7 1998/02/12 20:53:52 inacio Exp $ */
/*****************************************************************************
 *addr_bus_control.v
 *
 *controls the address bus lines, muxes them between the core and rs_ab
 *
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: inacio $
 *Date: $Date: 1998/02/12 20:53:52 $
 *Revision: $Revision: 1.7 $
 *
 *
 *Log: $Log: addr_bus_control.v,v $
 *Log: Revision 1.7  1998/02/12 20:53:52  inacio
 *Log: changed formatting
 *Log:
# Revision 1.6  1998/01/13  02:34:16  inacio
# took out a large hunk of commented code
#
# Revision 1.5  1997/12/14  23:21:08  benk
# Rebuilt control for thingy.  All logic in thingy_control.
# Added X_READ, Y_READ to enable signals.
# addr_bus_control simplified to avoid 'xxxx' values in rtl simulation.
#
# Revision 1.4  1997/11/15  07:07:39  inacio
# fixed error, cpu_stop is active low, and it was written as if it was active
# high
#
# Revision 1.3  1997/11/09  08:51:45  inacio
# I changed the tabbing in the files to be spaces, now it looks the same in
# less as it does in my editor
#
# Revision 1.2  1997/10/15  22:04:37  inacio
# added the comments back in
#
 *****************************************************************************/

module addr_bus_control (cpu_stop, mem_slct, C_PAB, C_XAB, C_YAB, rs_ab, 
      E_PAB, E_XAB, E_YAB);

//paramter abuswidth = 16;
`define abuswidth 16

input  cpu_stop;
input  [1:0]mem_slct;

input  [(`abuswidth-1):0]C_PAB;
input  [(`abuswidth-1):0]C_XAB;
input  [(`abuswidth-1):0]C_YAB;
input  [(`abuswidth-1):0]rs_ab;


output [(`abuswidth-1):0]E_PAB;
output [(`abuswidth-1):0]E_XAB;
output [(`abuswidth-1):0]E_YAB;

reg [(`abuswidth-1):0] E_PAB,E_XAB,E_YAB;



always @(cpu_stop or mem_slct or C_PAB or C_XAB or C_YAB or rs_ab)
   begin

      /*
       * cpu_stop is active low!!
       */
      if (cpu_stop == 1'b0)
	      begin
 	         E_PAB <= rs_ab;
 	         E_XAB <= rs_ab;
 	         E_YAB <= rs_ab;
         end
      else
         begin
            E_PAB <= C_PAB;
            E_XAB <= C_XAB;
            E_YAB <= C_YAB;
         end
   end

endmodule


