/* $Id: ram_enable_control.v,v 1.1 1997/10/14 19:56:14 inacio Exp $ */
/*****************************************************************************
 *ram_enable_control
 *
 *selects the control signals and generates the right looking signals for the
 *internal SRAMS
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: inacio $
 *Date: $Date: 1997/10/14 19:56:14 $
 *Revision: $Revision: 1.1 $
 *
 *
 *Log: $Log: ram_enable_control.v,v $
 *Log: Revision 1.1  1997/10/14 19:56:14  inacio
 *Log: Logic to control the output enable and write enable signals to the SRAM
 *Log:
 *****************************************************************************/


module ram_enable_control (cpu_stop, mem_slct, clk, CX_WRITE, CX_READ, 
								CY_READ, CY_WRITE, E_OUTEN, E_WRITE, EP_OUTEN, 
								EP_WRITE, EX_OUTEN, EX_WRITE, EY_OUTEN, EY_WRITE);


input		cpu_stop;
input		[1:0]mem_slct;
input		clk;
input		CX_WRITE;
input		CX_READ;
input		CY_READ;
input		CY_WRITE;
input		E_OUTEN;
input		E_WRITE;
output	EP_OUTEN;
output	EP_WRITE;
output	EX_OUTEN;
output	EX_WRITE;
output	EY_OUTEN;
output	EY_WRITE;


reg	EP_OUTEN;
reg	EP_WRITE;
reg	EX_OUTEN;
reg	EX_WRITE;
reg	EY_OUTEN;
reg	EY_WRITE;




always @(cpu_stop or mem_slct or CX_WRITE or CX_READ or CY_READ or CY_WRITE or E_OUTEN or E_WRITE or clk)
begin

	if (cpu_stop == 1'b1)
		begin
			case (mem_slct)
				2'b00:
					begin
						EX_OUTEN <= E_OUTEN;
						EX_WRITE <= E_WRITE;
						EY_OUTEN <= 1'b1;
						EY_WRITE <= 1'b1;
						EP_OUTEN <= 1'b1;
						EP_WRITE <= 1'b1;
					end
				2'b01:
					begin
						EX_OUTEN <= 1'b1;
						EX_WRITE <= 1'b1;
						EY_OUTEN <= E_OUTEN;
						EY_WRITE <= E_WRITE;
						EP_OUTEN <= 1'b1;
						EP_WRITE <= 1'b1;
					end
				2'b10:
					begin
						EX_OUTEN <= 1'b1;
						EX_WRITE <= 1'b1;
						EY_OUTEN <= 1'b1;
						EY_WRITE <= 1'b1;
						EP_OUTEN <= 1'b1;
						EP_WRITE <= 1'b1;
					end
				2'b11:
					begin
						EX_OUTEN <= 1'b1;
						EX_WRITE <= 1'b1;
						EY_OUTEN <= 1'b1;
						EY_WRITE <= 1'b1;
						EP_OUTEN <= E_OUTEN;
						EP_WRITE <= E_WRITE;
					end
			endcase
		end
	else
		begin
			EX_OUTEN <= ~CX_READ;
			EX_WRITE <= ~CX_WRITE & clk;
			
			EY_OUTEN <= ~CY_READ;
			EY_WRITE <= ~CY_WRITE & clk;
			
			EP_OUTEN <= 0'b0;
			EP_WRITE <= 1'b1;
		end

end




endmodule


