/* $Id: thingy.v,v 1.8 1998/03/03 21:11:16 inacio Exp $ */
/*****************************************************************************
 *thingy.v
 *
 *The transceiver/mux/demux thing necessary to do the bus coupling
 *
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: inacio $
 *Date: $Date: 1998/03/03 21:11:16 $
 *Revision: $Revision: 1.8 $
 *
 *
 *Log: $Log: thingy.v,v $
 *Log: Revision 1.8  1998/03/03 21:11:16  inacio
 *Log: changed 2 of the tri-states into muxes.  This means that the info going into
 *Log: the RAM goes through muxes and the info coming from the RAM goes through
 *Log: tri-states
 *Log:
# Revision 1.7  1998/02/08  00:39:21  ar39
# Fixed the dpath parameters more.
#
 *Log: Revision 1.6  1998/02/07 23:55:44  ar39
 *Log: Cleaned up some dpath parameters <argh>.
 *Log:
 *Log: Revision 1.5  1997/12/14 23:20:19  benk
 *Log: Rebuilt control for thingy.  All logic in thingy_control.
 *Log: Added X_READ, Y_READ to enable signals.
 *Log:
# Revision 1.4  1997/11/09  08:51:31  inacio
# I changed the tabbing in the files to be spaces, now it looks the same in
# less as it does in my editor
#
# Revision 1.3  1997/10/10  20:31:52  inacio
# arranged the input and output lines so they were going the correct
# directions
#
# Revision 1.2  1997/10/08  00:58:03  inacio
# added an enable line to disable all ports
#
# Revision 1.1  1997/10/08  00:21:19  inacio
# new module to talk to the SRAMs
#
 *****************************************************************************/


/*
 * Control signals truth table....
 *
 *
 *
 *  bussel  |
 *  --------+------------
 *    0     |  bidir1
 *    1     |  bidir2
 *
 *  direction      |
 *  ---------------+---------------
 *      0          |  from SRAM
 *      1          |  to SRAM
 *
 *
 *  enable    |
 *  ----------+------------------
 *     0      |     ports enable
 *     1      |     ports disabled
 *
 */
module thingy (bidir1, bidir2, inbus, outbus,
			nbd1_to_sram, nbd2_to_sram, 
			nsram_to_bd1, nsram_to_bd2
			);

 /*
  * module parameters
  */
 parameter bitwidth = 24; // default bus width is 24-bits!
 parameter data_path = 0;


 /*
  *control signals
  */

input nbd1_to_sram;	// to memory
input nbd2_to_sram;
input nsram_to_bd1;	// from memory
input nsram_to_bd2;

 /*
  * Bus definitions
  */

 output [(bitwidth)-1:0]outbus;  // output bus to SRAMS
 input [(bitwidth)-1:0]inbus;  // input bus from SRAMS
  
 inout [(bitwidth)-1:0]bidir1;  // bidirectional busses...
 inout [(bitwidth)-1:0]bidir2;
  

 tribuf #(bitwidth, data_path) bd1_from_sram (nsram_to_bd1, inbus, bidir1);
 tribuf #(bitwidth, data_path) bd2_from_sram (nsram_to_bd2, inbus, bidir2);
 mux2 #(bitwidth, data_path) bus_mux_to_sram (bidir1, bidir2, nbd1_to_sram, 
                                             outbus);

endmodule
