/* $Id: adm_stim.v,v 1.1 1997/11/17 23:04:36 inacio Exp $ */
/*****************************************************************************
 *adm_stim.v
 *
 *module to stimulate the core for the ADM simulations
 *
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: inacio $
 *Date: $Date: 1997/11/17 23:04:36 $
 *Revision: $Revision: 1.1 $
 *
 *
 *Log: $Log: adm_stim.v,v $
 *Log: Revision 1.1  1997/11/17 23:04:36  inacio
 *Log: begininngs the file to generate vectors for the ADM simulations.  I need to
 *Log: add monitor statements.  Right now this is the whole_chip_cascade.v file
 *Log: renamed.
 *Log:
 *****************************************************************************/
              
module thing;


/*
 *fake memories to load up the real Cascade memories
 */
reg	[23:0]temp_P_ram[1023:0];
reg	[23:0]temp_X_ram[1023:0];
reg	[23:0]temp_Y_ram[1023:0];


/*
 * variables to do the writing and bookkeeping
 */
reg	[32:0]clock_count;
reg 	[15:0]loop_count;
reg	[23:0]dummy_word;
reg	[15:0]dummy_addr;
reg	[16:0]dummy_data1;
reg	[23:17]dummy_data2;
reg	dummy_ram_enable;
reg	dummy_ram_write;
reg	[2:0]vil;
reg   old_dummy_estrobe;
reg   dummy_estrobe;


/*
 * Connections to hook up to the chip, uncomment the reg version if you need
 * to make an assignment to it.
 */
reg	Clk;
reg 	reset;
reg	e_enable;
reg	[1:0]mem_select;
/*
reg	[15:0]eab;
reg	[16:0]edb1;
reg	[23:17]edb2;
reg	scanin1;
reg	scanin2;
reg	scanout1;
reg	scanout2;
reg	estrobe;
reg	e_access;
reg	e_outen;
reg	e_write;
reg	test;
*/



/*
 * Connections to hook up to the chip, uncomment the reg version if you need
 * to make an assignment to it.  Comment out the wire version if you need to
 * make an assignment to it
 */
/*
wire	Clk;
wire 	reset;
wire	e_enable;
wire	[1:0]mem_select;
*/
wire	[15:0]eab;
wire	[16:0]edb1;
wire	[23:17]edb2;
wire	scanin1;
wire	scanin2;
wire	scanout1;
wire	scanout2;
wire	estrobe;
wire	e_access;
wire	e_outen;
wire	e_write;
wire	test;
wire  pwr1, pwr2, pwr3, pwr4, pwr5, pwr6, pwr7;
wire  gnd1, gnd2, gnd3, gnd4, gnd5, gnd6, gnd7;


/*****************************************************************************
 *instance the chip
 *
 *****************************************************************************/
pads fullchip (Clk, reset, mem_select, estrobe, e_access, e_enable, eab,
    			edb1, edb2, e_outen, e_write, test, scanin1, scanin2, scanout1,
    			scanout2, pwr1, pwr2, pwr3, pwr4, pwr5, pwr6, pwr7, gnd1, gnd2,
    			gnd3, gnd4, gnd5, gnd6, gnd7);
            

/*****************************************************************************
 *build the assign statements to drive the busses
 *
 *****************************************************************************/
assign
	eab = (e_enable) ? dummy_addr : 16'bz,
	edb1 = (e_enable) ? dummy_data1 : 17'bz,
	edb2 = (e_enable) ? dummy_data2 : 7'bz,
	e_write = (e_enable) ? dummy_ram_write : 1'bz,
	e_outen = (e_enable) ? dummy_ram_enable : 1'bz;
	

/*****************************************************************************
 *
 *clean up the estrobe signal
 *
 *****************************************************************************/
always
   begin
      @(estrobe)
         begin
            if (estrobe == 1'bz || estrobe == 1'bx)
               begin
                  dummy_estrobe <= old_dummy_estrobe;
               end
            else
               begin
                  dummy_estrobe <= estrobe;
                  old_dummy_estrobe <= estrobe;
               end
         end
   end
   


/*****************************************************************************
 *reset the chip and get it ready to load the memories
 *
 *****************************************************************************/
initial
	begin


		/*
		 *
		 * Probe the chip and build wave forms to view
		 *
		 */
		$shm_open("wavesCR.shm");
      $shm_probe("AS");
                  
		/*
		 * reset the clock counter
		 */
		clock_count <= 0;

		/*
		 * read from files into the fake memories
		 */
		$readmemh("pmem_file", temp_P_ram, 16'h0000);
		$readmemh("xmem_file", temp_X_ram, 16'h0000);
		$readmemh("ymem_file", temp_Y_ram, 16'h0000);
		


		/*
		 * signals that I want to have already set when I get ready
		 * to run after reset....  Initializes the E bus to load the internal
		 * RAMs
		 */
      dummy_estrobe <= 0;
      old_dummy_estrobe <= 0;
		e_enable <= 1;
		mem_select <= 2'b11;
		dummy_addr <= 16'b0;
		dummy_data1 <= 17'b0;
		dummy_data2 <= 7'b0;
		dummy_ram_write <= 1'b1;
		dummy_ram_enable <= 1'b1;


		/*
		 * reset the chip
		 */		
		reset <= 1;
		#100 reset <= 0;


		/*
		 * write the values into the memory
		 *
		 */
		wait (reset == 1'b0);
   		for (loop_count = 0; loop_count < 1024; loop_count = loop_count + 1)
				begin
					$display ("Loop count is: %x\n",loop_count);
   				for (vil = 0; vil < 3'b100; vil = vil + 3'b001)
   					begin
   						$display ("vil is %x\n",vil);
         				mem_select <= vil;
   						case (vil)
   							2'b000 : 
   								begin
   									dummy_word <= temp_X_ram[loop_count];
   									dummy_ram_write <= 1'b0;
   								end
   							2'b001 : 
   								begin
   									dummy_word <= temp_Y_ram[loop_count];
   									dummy_ram_write <= 1'b0;
   								end
   							2'b010 : 
   								begin
   									dummy_word <= 256'b0;
   									dummy_ram_write <= 1'b1;
   								end
   							2'b011 : 
   								begin
   									dummy_word <= temp_P_ram[loop_count];
   									dummy_ram_write <= 1'b0;
   								end
   						endcase
         				dummy_addr <= loop_count;
         				dummy_data1 <= dummy_word[16:0];
         				dummy_data2 <= dummy_word[23:17];
                     //
                     // bad edge on estrobe
         				//@(posedge estrobe);
         				// wait (estrobe == 1'b1);
         				@(posedge dummy_estrobe);
      				end
   			end


		/*
		 * Leave test mode
		 */
		@(posedge estrobe);
		dummy_ram_write <= 1'b1;
		e_enable <= 0;

		#60000 $finish;

		
      $shm_close;     /* close the dump file */

		
	end


/*****************************************************************************
 *build the clock signal
 *
 *****************************************************************************/
always
	begin
		#25 Clk <= 1;
		#25 Clk <= 0;
		clock_count = clock_count + 1;
		$display ("Clock is now: %x\n",clock_count);
	end
			


endmodule	



/*****************************************************************************
 *****************************************************************************/
