/*	File:	cr_test.v	  					*/

/*	module name: top						*/

/*	Description: TEST FILE						*/

/*	Tested Module name:	core_ram.v				*/


/*=======================================================================*/

/*************************************************************************/
/*									*/
/*	module:	top							*/
/*									*/
/*************************************************************************/

module top;



/*============		internal nets	=================================*/

   reg		Clk;
   reg		reset;
   reg		e_enable;
   reg [1:0]	mem_select;
   reg		e_outen;
   reg		e_write;
   wire [15:0]	eab;
   wire [16:0]	edb1;
   wire [23:17]	edb2;
   wire [23:0]	edb;
   reg		scanin1;
   reg		scanin2;
   wire		scanout1;
   wire		scanout2;
   wire		estrobe;
   wire		e_access;
   reg		test;

   integer	dumpfile, dumpfile_and_stdout;
   integer	j, mem_dump;

/*=========================================================================*/



/*===========	Module Instantiations	 ===================================*/
`ifdef CORE_ONLY
   core_ram_struc coreR(
	.reset (reset),
	.Clk (Clk)
	);

`endif // ifdef COPRE_ONLY
   
`ifdef PADS
pads fullchip (Clk, reset, mem_select, estrobe, e_access, e_enable, eab,
	       edb, e_outen, e_write, test, scanin1, scanin2, scanout1,
	       scanout2, pwr0, pwr1, pwr2, pwr3, pwr4, pwr5, pwr6, pwr7, 
	       pwr8, pwr9, gnd0, gnd1, gnd2, gnd3, gnd4, gnd5, gnd6, gnd7, 
	       gnd8, gnd9);
`endif // ifdef PADS
   
`ifdef POST_LAYOUT
pads fullchip (Clk, reset, mem_select, estrobe, e_access, e_enable, eab,
	       edb, e_outen, e_write, test, scanin1, scanin2, scanout1,
	       scanout2, pwr0, pwr1, pwr2, pwr3, pwr4, pwr5, pwr6, pwr7, 
	       pwr8, pwr9, gnd0, gnd1, gnd2, gnd3, gnd4, gnd5, gnd6, gnd7, 
	       gnd8, gnd9);
`endif // ifdef POST_LAYOUT
   

/*========================================================================*/


/*======================================================================*/
/*	initialize							*/
/*======================================================================*/

initial
   begin

`ifdef SHM      
      /* generates a dump file for Cwaves to use */
      $shm_open("waves.shm");
      /* currently no dumps for CORE_ONLY */
`ifdef PADS
      $shm_probe(fullchip,"A",fullchip.processor,"A"
		 );
`endif // ifdef SHM
`ifdef POST_LAYOUT
      $shm_probe(top,"A", fullchip,"A",fullchip.pads_core,"A",
      
		 fullchip.pads_core.processor_dsp_ALU, "AS");
`endif // ifdef POST_LAYOUT
`endif // ifdef SHM
            
      /* open text file for dumping results */
      /* always open dump file, but only write data if CYCLE_DUMP */
      dumpfile = $fopen("verilogSim.dump");
      /* also send results to standard output (Palnitkar p. 180) */
      //		dumpfile_and_stdout = $fopen("verilogSim.dump") | 1;
      //dumpfile_and_stdout =  1;


      /* Start simulation with reset signal high */
      Clk = 0;
      reset = 0;	
      e_enable = 0;
      mem_select = 2'b10;
      e_outen = 1;
      e_write = 1;
      scanin1 = 0;
      scanin2 = 0;
      test = 0;	

      /* give inputs a chance to settle */
      #45 ;
      
      /* load data into RAM modules 		*/
      /* start at RAMData[00]             */
`ifdef CORE_ONLY
      $readmemh( "pmem_file", coreR.PRAM.RAMData, 8'h00);
      $readmemh( "xmem_file", coreR.XRAM.RAMData, 12'h00);
      $readmemh( "ymem_file", coreR.YRAM.RAMData, 12'h00);
`endif // ifdef CORE_ONLY
      
`ifdef PADS
      $readmemh( "pmem_file", 
		 fullchip.processor.memory.pram_cell.mem, 8'h00);
      $readmemh( "xmem_file", 
		 fullchip.processor.memory.xram_cell.mem, 8'h00);
      $readmemh( "ymem_file", 
		 fullchip.processor.memory.yram_cell.mem, 8'h00);
`endif // ifdef PADS
`ifdef POST_LAYOUT
      $readmemh( "pmem_file", 
		 fullchip.pads_core.processor_memory_pram_cell.ram.inst1.mem, 
		 8'h00);
      $readmemh( "xmem_file",
		 fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem, 
		 8'h00);
      $readmemh( "ymem_file", 
		 fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem, 
		 8'h00);
`endif // ifdef POST_LAYOUT
      
      /* Start reset cycle */
      #5 reset = 1;

      /* End of reset cycle */
      #50 reset = 0;


      /*
       *dump the input and output ram areas before simulation
       */
      $display ("Dumping pre-simulation memory...");
      mem_dump = $fopen ("verilog_before_input_mem");
      for (j=256; j < 512; j = j + 4)
`ifdef CORE_ONLY
	 $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    coreR.XRAM.RAMData[j],coreR.XRAM.RAMData[j+1],
		    coreR.XRAM.RAMData[j+2],coreR.XRAM.RAMData[j+3]);
`endif // ifdef CORE_ONLY
      
`ifdef PADS
      $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		 fullchip.processor.memory.xram_cell.mem[j],
		 fullchip.processor.memory.xram_cell.mem[j+1],
		 fullchip.processor.memory.xram_cell.mem[j+2],
		 fullchip.processor.memory.xram_cell.mem[j+3]);
`endif // ifdef PADS

`ifdef POST_LAYOUT
      $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		 fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j],
		 fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+1],
		 fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+2],
		 fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+3]);
`endif // ifdef POST_LAYOUT
      $fclose (mem_dump);
      
      mem_dump = $fopen ("verilog_before_output_mem");
      for (j=256; j < 512; j = j +4)
`ifdef CORE_ONLY
	 $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    coreR.YRAM.RAMData[j],coreR.YRAM.RAMData[j+1],
		    coreR.YRAM.RAMData[j+2],coreR.YRAM.RAMData[j+3]);
`endif // ifdef CORE_ONLY
      
`ifdef PADS	 
	 $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    fullchip.processor.memory.yram_cell.mem[j],
		    fullchip.processor.memory.yram_cell.mem[j+1],
		    fullchip.processor.memory.yram_cell.mem[j+2],
		    fullchip.processor.memory.yram_cell.mem[j+3]);
`endif // ifdef PADS

`ifdef POST_LAYOUT
      $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		 fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j],
		 fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+1],
		 fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+2],
		 fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+3]);
`endif // ifdef POST_LAYOUT
      $fclose (mem_dump);
      
      
      /*
       * Simulte the program
       * should need 1802 cycles, but the Verilog model needs 2062
       */
      $display ("Beginning simultaion...");
      
//      #1000;
      #103100;		/* run through the simulation 2062 cycles */
      
      
      /*
       *dump the input and output ram areas after simulation
       */
      $display ("Dumping post-simulation memory...");
      mem_dump = $fopen ("verilog_after_input_mem");
      for (j=256; j < 512; j = j + 4)
`ifdef CORE_ONLY
	 $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    coreR.XRAM.RAMData[j],coreR.XRAM.RAMData[j+1],
		    coreR.XRAM.RAMData[j+2],coreR.XRAM.RAMData[j+3]);
`endif // ifdef CORE_ONLY
      
`ifdef PADS
	 $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    fullchip.processor.memory.xram_cell.mem[j],
		    fullchip.processor.memory.xram_cell.mem[j+1],
		    fullchip.processor.memory.xram_cell.mem[j+2],
		    fullchip.processor.memory.xram_cell.mem[j+3]);
`endif // ifdef PADS

`ifdef POST_LAYOUT
      	 $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j],
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+1],
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+2],
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+3]);
`endif // ifdef POST_LAYOUT
      $fclose (mem_dump);
      
      mem_dump = $fopen ("verilog_after_output_mem");
      for (j=256; j < 512; j = j + 4)
`ifdef CORE_ONLY
	 $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    coreR.YRAM.RAMData[j],coreR.YRAM.RAMData[j+1],
		    coreR.YRAM.RAMData[j+2],coreR.YRAM.RAMData[j+3]);
`endif // ifdef CORE_ONLY
      
`ifdef PADS
	 $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    fullchip.processor.memory.yram_cell.mem[j],
		    fullchip.processor.memory.yram_cell.mem[j+1],
		    fullchip.processor.memory.yram_cell.mem[j+2],
		    fullchip.processor.memory.yram_cell.mem[j+3]);
`endif // ifdef PADS

`ifdef POST_LAYOUT
      	 $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j],
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+1],
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+2],
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+3]);
`endif // ifdef POST_LAYOUT
      $fclose (mem_dump);
      
      $fclose(dumpfile);		

`ifdef SHM      
      $shm_close;		 /* close the dump file */
`endif // ifdef SHM
      
      $finish; /* end simulation */
      
   end



/*======================================================================*/
/*	clock	25 units High, 25 units Low				*/
/* 		on each cycle, dump register contents			*/
/*======================================================================*/

always
   begin
      #25 Clk = 1;
      #25 Clk = 0;

      /* dump contents of architected registers */
      /* to send only to a file, use dumpfile instead of */
      /* dumpfile_and_stdout */
`ifdef CYCLE_DUMP
`ifdef PADS
      $fdisplay(dumpfile,
		"\nCore signals: \n\t PAB = %h \t XAB = %h \t YAB = %h \n",
		fullchip.processor.C_PAB, 
		fullchip.processor.C_XAB, 
		fullchip.processor.C_YAB, 
		"\t PDB = %h \t XDB = %h \t YDB = %h \n",
		fullchip.processor.C_PDB, 
		fullchip.processor.C_XDB, 
		fullchip.processor.C_YDB, 
		
		"PCU Registers: \n",
		"\t PC = %h \n",
		fullchip.processor.dsp.PCU.PC, 	
		
		"ALU Registers: \n",
		"\t x = %h  \ty = %h\n",
		{fullchip.processor.dsp.ALU.x1, 
		 fullchip.processor.dsp.ALU.x0},
		{fullchip.processor.dsp.ALU.y1, 
		 fullchip.processor.dsp.ALU.y0},
		"\t A = %h \t B = %h \n",
		{fullchip.processor.dsp.ALU.a2, 
		 fullchip.processor.dsp.ALU.a1, 
		 fullchip.processor.dsp.ALU.a0},
		{fullchip.processor.dsp.ALU.b2, 
		 fullchip.processor.dsp.ALU.b1, 
		 fullchip.processor.dsp.ALU.b0},
		"next",$time,
		"\n\n"
		);
`endif // ifdef CYCLE_DUMP
`ifdef POST_LAYOUT
            $fdisplay(dumpfile,
		"\nCore signals: \n\t PAB = %h \t XAB = %h \t YAB = %h \n",
		fullchip.pads_core.processor_C_PAB, 
		fullchip.pads_core.processor_C_XAB, 
		fullchip.pads_core.processor_C_YAB, 
		"\t PDB = %h \t XDB = %h \t YDB = %h \n",
		fullchip.pads_core.processor_C_PDB, 
		fullchip.pads_core.processor_C_XDB, 
		fullchip.pads_core.processor_C_YDB, 
		
		"PCU Registers: \n",
		"\t PC = %h \n",
		fullchip.pads_core.processor_dsp_PCU_PC, 	
		
		"ALU Registers: \n",
		"\t x = %h  \ty = %h\n",
		{fullchip.pads_core.processor_dsp_ALU.x1, 
		 fullchip.pads_core.processor_dsp_ALU.x0},
		{fullchip.pads_core.processor_dsp_ALU.y1, 
		 fullchip.pads_core.processor_dsp_ALU.y0},
		"\t A = %h \t B = %h \n",
		{fullchip.pads_core.processor_dsp_ALU.a2, 
		 fullchip.pads_core.processor_dsp_ALU.a1, 
		 fullchip.pads_core.processor_dsp_ALU.a0},
		{fullchip.pads_core.processor_dsp_ALU.b2, 
		 fullchip.pads_core.processor_dsp_ALU.b1, 
		 fullchip.pads_core.processor_dsp_ALU.b0},
		"next",$time,
		"\n\n"
		);
`endif // ifdef POST_LAYOUT
`endif // ifdef CYCLE_DUMP
      
   end
   
endmodule	/* end module top */
