/*	File:	test_pads_cascade.v	  					*/

/*	module name: top						*/

/*	Description: TEST FILE for simulating lms for post-cascade pads	*/


/*=======================================================================*/

/*************************************************************************/
/*									*/
/*	module:	top							*/
/*									*/
/*************************************************************************/

module top;



/*============		internal nets	=================================*/

   reg		Clk;
   reg		reset;
   reg		e_enable;
   reg [1:0]	mem_select;
   reg		e_outen;
   reg		e_write;
   wire [15:0]	eab;
   wire [16:0]	edb1;
   wire [23:17]	edb2;
   reg		scanin1;
   reg		scanin2;
   wire		scanout1;
   wire		scanout2;
   wire		estrobe;
   wire		e_access;
   reg		test;

   integer	dumpfile, dumpfile_and_stdout;
   integer	j, mem_dump;

/*=========================================================================*/



/*===========	Module Instantiations	 ===================================*/

pads fullchip (Clk, reset, mem_select, estrobe, e_access, e_enable, eab,
	       edb1, edb2, e_outen, e_write, test, scanin1, scanin2, scanout1,
	       scanout2, pwr1, pwr2, pwr3, pwr4, pwr5, pwr6, pwr7, gnd1, gnd2,
	       gnd3, gnd4, gnd5, gnd6, gnd7, pwr8, pwr9, pwr10, pwr11, pwr12,
	       pwr13, pwr14, gnd8, gnd9, gnd10, gnd11, gnd12, gnd13, gnd14);

/*========================================================================*/

   
/*======================================================================*/
/*	initialize							*/
/*======================================================================*/

initial
   begin
	
      /* generates a dump file for Cwaves to use */
//      $shm_open("waves.shm");		
      /* keep track of all signals in the hierarcy */
//      $shm_probe(top,"A", fullchip,"A",fullchip.pads_core,"A");


      $monitor($time, "Memory changed: %h", 
	       fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[0]
	       );
            
      /* open text file for dumping results */
      dumpfile = $fopen("verilogSim.dump");
      /* also send results to standard output (Palnitkar p. 180) */
      //		dumpfile_and_stdout = $fopen("verilogSim.dump") | 1;
      dumpfile_and_stdout = 1;    // only dump to stdout
      

      /* Start simulation with reset signal low */
      Clk = 0;
      reset = 0;	
      e_enable = 0;
      mem_select = 2'b00;
      e_outen = 1;
      e_write = 1;
      scanin1 = 0;
      scanin2 = 0;
      test = 0;

      /* give inputs a chance to settle */
      #45 ;
           
      /* load data into RAM modules 		*/
      /* start at RAMData[00]             */
      $readmemh( "pmem_file", 
		 fullchip.pads_core.processor_memory_pram_cell.ram.inst1.mem, 
		 12'h00);
      $readmemh( "xmem_file",
		 fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem, 
		 12'h00);
      $readmemh( "ymem_file", 
		 fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem, 
		 12'h00);
      
      /* start reset cycle */
      #5 reset = 1;
      
      /* end of reset cycle */
      #50 reset = 0;
      
      /*
       *dump out the input vector...
       */
      mem_dump = $fopen ("verilog_input_dump");
      for (j=0; j<=1024; j = j + 4)
	 $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n", j,
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j],
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+1],
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+2],
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+3]);
      
      for (j=0; j<=1024; j = j + 4)
	 $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n", j, 
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j],
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+1],
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+2],
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+3]);
      $fclose (mem_dump);

      /*
       * Run simulation
       */

//      #10000;             
      #3199000;   /* 63980 cycles */

      /*
       *dump out the output vector...
       */
      mem_dump = $fopen ("verilog_output_dump");
      for (j=0; j<=1024; j = j + 4)
	 $fdisplay (mem_dump, "x:$%x=\t$%x\t$%x\t$%x\t$%x\n",j,
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j],
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+1],
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+2],
		    fullchip.pads_core.processor_memory_xram_cell.ram.inst1.mem[j+3]);
      for (j=0; j<=1024; j = j + 4)
	 $fdisplay (mem_dump, "y:$%x=\t$%x\t$%x\t$%x\t$%x\n",j, 
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j],
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+1],
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+2],
		    fullchip.pads_core.processor_memory_yram_cell.ram.inst1.mem[j+3]);
      $fclose (mem_dump);
      
      $finish;		/* end simulation */
      
      $fclose(dumpfile);		
      
//      $shm_close;		 /* close the dump file */
   end



/*======================================================================*/
/*	clock	25 units High, 25 units Low   20 MHz			*/
/* 		on each cycle, dump register contents			*/
/*======================================================================*/

always
   begin
      #20 ;
      
      /* dump contents of architected registers */
      /* to send only to a file, use dumpfile instead of */
      /* dumpfile_and_stdout */
      $fdisplay(dumpfile,
                "\nCore signals: \n\t PAB = %h \t XAB = %h \t YAB = %h \n",
                fullchip.pads_core.processor_C_PAB, 
                fullchip.pads_core.processor_C_XAB, 
                fullchip.pads_core.processor_C_YAB, 
                "\t PDB = %h \t XDB = %h \t YDB = %h \n",
                fullchip.pads_core.processor_C_PDB, 
                fullchip.pads_core.processor_C_XDB, 
                fullchip.pads_core.processor_C_YDB, 
                
                "PCU Registers: \n",
                "\t PC = %h \n",
                fullchip.pads_core.processor_dsp_PCU_PC,        
                
                "ALU Registers: \n",
                "\t x = %h  \ty = %h\n",
                {fullchip.pads_core.processor_dsp_ALU.x1, 
                 fullchip.pads_core.processor_dsp_ALU.x0},
                {fullchip.pads_core.processor_dsp_ALU.y1, 
                 fullchip.pads_core.processor_dsp_ALU.y0},
                "\t A = %h \t B = %h \n",
                {fullchip.pads_core.processor_dsp_ALU.a2, 
                 fullchip.pads_core.processor_dsp_ALU.a1, 
                 fullchip.pads_core.processor_dsp_ALU.a0},
                {fullchip.pads_core.processor_dsp_ALU.b2, 
                 fullchip.pads_core.processor_dsp_ALU.b1, 
                 fullchip.pads_core.processor_dsp_ALU.b0},
                "next",$time,
                "\n\n"
                );
      #5 Clk = 1;
      #25 Clk = 0;
      

/*      
      $fdisplay(dumpfile,
		"\nCore signals: \n\t PAB = %h \t XAB = %h \t YAB = %h \n",
		fullchip.processor.C_PAB, 
		fullchip.processor.C_XAB, 
		fullchip.processor.C_YAB, 

		"\nCore signals: \n\t PDB = %h \t XDB = %h \t YDB = %h \n",
		fullchip.processor.C_PDB, 
		fullchip.processor.C_XDB, 
		fullchip.processor.C_YDB, 
		
		"PCU Registers: \n",
		"\t PC = %h \n",
		fullchip.processor.dsp.PCU.PC, 	
		
		"ALU Registers: \n",
		"\t x = %h  \ty = %h\n",
		{fullchip.processor.dsp.ALU.x1, 
		 fullchip.processor.dsp.ALU.x0},
		{fullchip.processor.dsp.ALU.y1, 
		 fullchip.processor.dsp.ALU.y0},
		"\t A = %h \t B = %h \n",
		{fullchip.processor.dsp.ALU.a2, 
		 fullchip.processor.dsp.ALU.a1, 
		 fullchip.processor.dsp.ALU.a0},
		{fullchip.processor.dsp.ALU.b2, 
		 fullchip.processor.dsp.ALU.b1, 
		 fullchip.processor.dsp.ALU.b0},
 */
      /*
       * 
			"AGU Registers; \n",
			"\t r7 = %h \t n7 = %h \t m7 = %h \n",
			coreR.CORE.agu.R_H.regout4, coreR.CORE.agu.N_H.regout4, coreR.CORE.agu.M_H.regout4, 
			"\t r6 = %h \t n6 = %h \t m6 = %h \n",
			coreR.CORE.agu.R_H.regout3, coreR.CORE.agu.N_H.regout3, coreR.CORE.agu.M_H.regout3, 
			"\t r5 = %h \t n5 = %h \t m5 = %h \n",
			coreR.CORE.agu.R_H.regout2, coreR.CORE.agu.N_H.regout2, coreR.CORE.agu.M_H.regout2, 
			"\t r4 = %h \t n4 = %h \t m4 = %h \n",    
			coreR.CORE.agu.N_H.regout1, coreR.CORE.agu.N_H.regout1, coreR.CORE.agu.M_H.regout1,
			"\t r3 = %h \t n3 = %h \t m3 = %h \n",
			coreR.CORE.agu.R_L.regout4, coreR.CORE.agu.N_L.regout4, coreR.CORE.agu.M_L.regout4, 
			"\t r2 = %h \t n2 = %h \t m2 = %h \n",
			coreR.CORE.agu.R_L.regout3, coreR.CORE.agu.N_L.regout3, coreR.CORE.agu.M_L.regout3, 
			"\t r1 = %h \t n1 = %h \t m1 = %h \n",
			coreR.CORE.agu.R_L.regout2, coreR.CORE.agu.N_L.regout2, coreR.CORE.agu.M_L.regout2,
			"\t r0 = %h \t n0 = %h \t m0 = %h \n",
			coreR.CORE.agu.R_L.regout1, coreR.CORE.agu.N_L.regout1, coreR.CORE.agu.M_L.regout1, 
       */
      /*
		"next",$time,
		"\n\n"
		);
       */
      
   end
	
endmodule	/* end module top */
