/* $Id: rs.v,v 1.2 1997/12/03 00:17:49 inacio Exp $ */
/*****************************************************************************
 *
 *
 *
 *
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: inacio $
 *Date: $Date: 1997/12/03 00:17:49 $
 *Revision: $Revision: 1.2 $
 *
 *
 *Log: $Log: rs.v,v $
 *Log: Revision 1.2  1997/12/03 00:17:49  inacio
 *Log: changed the tests to make a little more sense for data writes and to better
 *Log: simulate the output of the core
 *Log:
# Revision 1.1  1997/12/01  21:06:27  inacio
# verilog file to test the ram switch
#
 *****************************************************************************/

`define	databus 23:0
`define	addrbus 15:0
`define  CLOCK_CYCLE 50


module rs_test ();


/*
 * some registers and variables for use in the testing....
 *
 */
integer clock_counter;
integer fake_data;
integer loop_counter;
integer dummy_mem_counter;


reg	[`databus]dummy_cpu_x_db;
reg	[`databus]dummy_cpu_y_db;
reg	[`databus]dummy_cpu_p_db;

reg	[`addrbus]dummy_external_ab;
reg	[`databus]dummy_external_db;

reg	[`addrbus]dummy_ram_x_ab;
reg	[`databus]dummy_ram_x_in_db;
reg	[`databus]dummy_ram_x_out_db;
reg	[`addrbus]dummy_ram_y_ab;
reg	[`databus]dummy_ram_y_in_db;
reg	[`databus]dummy_ram_y_out_db;
reg	[`addrbus]dummy_ram_p_ab;
reg	[`databus]dummy_ram_p_in_db;
reg	[`databus]dummy_ram_p_out_db;

reg	dummy_external_outen;
reg	dummy_external_write;



/*
 * Large blob of wires to connect to the ramswitch (mutually exclusive with
 * the registers down below)
 *
 */
/*
wire reset;
wire Clkin;
wire EENABLE;
wire [1:0]MEM_SLCT;
wire [`addrbus]C_PAB;
wire [`addrbus]C_XAB;
wire [`addrbus]C_YAB;
wire CX_WRITE;
wire CY_WRITE;
wire CX_READ;
wire CY_READ;
 */
wire E_OUTEN;
wire E_WRITE;
wire Clkout;
wire ESTROBE;
wire EACCESS;
wire test;
wire [`databus]C_PDB;
wire [`databus]C_XDB;
wire [`databus]C_YDB;
wire [`addrbus]E_PAB;
wire [`databus]E_PDB_IN;
wire [`databus]E_PDB_OUT;
wire EP_OUTEN;
wire EP_WRITE;
wire [`addrbus]E_XAB;
wire [`databus]E_XDB_IN;
wire [`databus]E_XDB_OUT;
wire EX_OUTEN;
wire EX_WRITE;
wire [`addrbus]E_YAB;
wire [`databus]E_YDB_IN;
wire [`databus]E_YDB_OUT;
wire EY_OUTEN;
wire EY_WRITE;
wire [`addrbus]EAB;
wire [`databus]EDB;
wire edb_dir_bar;
wire edb_en;
wire eab_dir;
wire eab_en;



/*
 * Large blob of regs to connect to the ramswitch (mutual exclusive with the
 * wires above)
 *
 */
reg reset;
reg Clkin;
reg [1:0]MEM_SLCT;
reg EENABLE;
reg [`addrbus]C_PAB;
reg [`addrbus]C_XAB;
reg [`addrbus]C_YAB;
reg CX_WRITE;
reg CY_WRITE;
reg CX_READ;
reg CY_READ;
/*
reg E_OUTEN;
reg E_WRITE;
reg Clkout;
reg [1:0]MEM_SLCT;
reg ESTROBE;
reg EACCESS;
reg test;
reg [`databus]C_PDB;
reg [`databus]C_XDB;
reg [`databus]C_YDB;
reg [`addrbus]E_PAB;
reg [`databus]E_PDB_IN;
reg [`databus]E_PDB_OUT;
reg EP_OUTEN;
reg EP_WRITE;
reg [`addrbus]E_XAB;
reg [`databus]E_XDB_IN;
reg [`databus]E_XDB_OUT;
reg EX_OUTEN;
reg EX_WRITE;
reg [`addrbus]E_YAB;
reg [`databus]E_YDB_IN;
reg [`databus]E_YDB_OUT;
reg EY_OUTEN;
reg EY_WRITE;
reg [`addrbus]EAB;
reg [`databus]EDB;
reg edb_dir_bar;
reg edb_en;
reg eab_dir;
reg eab_en;
*/



ram_switch rs (reset, MEM_SLCT, ESTROBE, EACCESS, EENABLE, Clkin, Clkout,
						test, C_PAB, C_PDB, C_XAB, C_XDB, CX_READ, CX_WRITE, C_YAB,
						C_YDB, CY_READ, CY_WRITE, E_PAB, E_PDB_IN, E_PDB_OUT, 
						EP_OUTEN, EP_WRITE, E_XAB, E_XDB_IN, E_XDB_OUT, EX_OUTEN,
						EX_WRITE, E_YAB, E_YDB_IN, E_YDB_OUT, EY_OUTEN, EY_WRITE,
						EAB, EDB, E_OUTEN, E_WRITE, edb_dir_bar, edb_en, eab_dir,
						eab_en);

/*
 * The RAMs
 *
 * ramoe #(bitsize, words, addressbits, columnsize) (address, data_out, enable, write, input)
 *
 */
ramoe #(24, 65535, 16, 4) pmem (E_PAB, E_PDB_OUT, EP_OUTEN, EP_WRITE, E_PDB_IN);
ramoe #(24, 65535, 16, 4) xmem (E_XAB, E_XDB_OUT, EX_OUTEN, EX_WRITE, E_XDB_IN);
ramoe #(24, 65535, 16, 4) ymem (E_YAB, E_YDB_OUT, EY_OUTEN, EY_WRITE, E_YDB_IN);


/*
 *
 *Making assignments to the inout ports
 *
 */
assign
	EDB = (EENABLE) ? dummy_external_db : 256'bz,
	EAB = (EENABLE) ? dummy_external_ab : 256'bz,
	C_XDB = (CX_WRITE) ? dummy_cpu_x_db : 256'bz,
	C_YDB = (CY_WRITE) ? dummy_cpu_y_db : 256'bz,
	E_WRITE = (EENABLE) ? dummy_external_write : 256'bz,
	E_OUTEN = (EENABLE) ? dummy_external_outen : 256'bz;


/*
 *Run the simulation
 *
 */
initial
	begin

		/*
		 *
		 * Probe the chip and build wave forms to view
		 *
		 */
		$shm_open("wavesCR.shm");
      $shm_probe("AS");
                  



		/*
		 *
		 * initialize some internal counters and stuff
		 *
		 */
		clock_counter = 0;
		fake_data = 32;
		reset <= 1;

		/*
		 *
		 * initialize some critical signals
		 *
		 */
		MEM_SLCT <= 2'b11;
		EENABLE <= 1'b0;

		/*
		 *
		 * the world inside a clean happy (know value) place
		 *
		 */
		dummy_cpu_x_db <= 256'b0;
		dummy_cpu_y_db <= 256'b0;
		dummy_cpu_p_db <= 256'b0;
		C_XAB <= 256'b0;
		C_PAB <= 256'b0;
		C_YAB <= 256'b0;

		
		#(`CLOCK_CYCLE*3) reset <= 0;


		/*
		 * let's see if the external port works....
		 */
		EENABLE <= 1'b1;
		wait (EACCESS == 1'b1);
      $display ("\n\nLoading RAM from the external port!\n\n");
		for (loop_counter = 0; loop_counter < 128; loop_counter = loop_counter + 1)
		   begin
			   for (dummy_mem_counter = 0; dummy_mem_counter < 5; dummy_mem_counter = dummy_mem_counter + 1)
   			   begin
					   if (dummy_mem_counter != 2)
						   begin
   						   dummy_external_write = 1'b1;
   							MEM_SLCT = dummy_mem_counter;
            				dummy_external_ab = loop_counter;
            				dummy_external_db = fake_data;
         					dummy_external_outen = 1'b1;
         					#(`CLOCK_CYCLE/5 + 1) dummy_external_write = 1'b0;
            				#(`CLOCK_CYCLE - (`CLOCK_CYCLE/4) - 1) dummy_external_write <= 1'b1;
            				@(posedge ESTROBE);
         			   end
         	   end
        			fake_data = fake_data + 1;
   		end
   	
      /*
       * disable the external port, start running from the core
       */
      EENABLE <= 1'b0;
      @(posedge ESTROBE);



		/*
		 * Let's write from the core to the ram
		 *
		 *but the real core doesn't support writing to P memory, so the RAM
		 *switch doesn't either right now
		 *
		 */
      $display ("\n\nWriting from the core to the memory...\n\n");
		fake_data = 32;
   	CX_READ <= 1'b0;
   	CY_READ <= 1'b0;
   	CX_WRITE <= 1'b1;
   	CY_WRITE <= 1'b1;
		for (loop_counter = 0; loop_counter < 64; loop_counter = loop_counter + 1)
			begin
				C_PAB = loop_counter;
				C_XAB = (loop_counter + 0);
				C_YAB = (loop_counter + 0);
				dummy_cpu_x_db = fake_data;
				dummy_cpu_y_db = fake_data;
				dummy_cpu_p_db = fake_data;
				@(posedge ESTROBE);
				fake_data = fake_data + 1;
			end
			
		 
		/*
		 *
		 * now there should be something in the RAM, let's try to read it
		 * back out
		 *
		 */
		$display ("\n\nReading from the core to the memory...\n\n");
      CX_WRITE = 1'b0;
      CY_WRITE = 1'b0;
      CX_READ = 1'b1;
      CY_READ = 1'b1;
      for (loop_counter =0; loop_counter < 128; loop_counter = loop_counter + 1)
         begin
            C_PAB = loop_counter;
            C_XAB = (loop_counter + 0);
            C_YAB = (loop_counter + 0);
            @(posedge ESTROBE);
         end
         		 
		 

		// #(`CLOCK_CYCLE*100) $finish;
		#(`CLOCK_CYCLE*2) $finish;

		
      $shm_close;     /* close the dump file */


	end


	
	
/*
 *
 * Build the clock...
 *
 */
always
	begin
		#(`CLOCK_CYCLE/2) Clkin <= 1;
		#(`CLOCK_CYCLE/2) Clkin <= 0;
		clock_counter = clock_counter + 1;
		$display ("Clock cycle is now: %x\n", clock_counter);
	end




endmodule

