/*	File:	XYregImm_c.v	  					*/

/*	module name: XYregImm						*/

/*	Description: XYregImm */

/*	 register for X, Y registers with input mux for immediate data	*/
/*	Instantiates Cascade modules	*/


/*======================================================================*/





   `include "./verilog/globals.v"



/************************************************************************/
/*									*/
/*	module:								*/
/*									*/
/************************************************************************/

module XYregImm_scan (
	Clk, reset,
	DB,
	imm,
	pdb2,
	in_ctl,
	write,
	scan_in, test, scan_out,
	q
	);


/*============	I/O direction	========================================*/

   input	     Clk, reset;
   input [`databus]  DB;		/* from XDB or YDB */
   input [`databus]  pdb2;		/* immediate data from pdb2 */
   input [7:0]	     imm;		/* short immediate data from pdb3 */
   input [1:0]	     in_ctl;
   input	     write;
   input	     scan_in, test;
   output	     scan_out;
   output [`databus] q;

/*=======================================================================*/


/*============		parameters	=================================*/

   parameter	     dbgroup = "xyreg";

/*============		I/O type	=================================*/

   wire [`databus]   q;
   wire		     scan_out;
   
/*================ internal signals =====================================*/

   wire [`databus]	     mux_out;
   wire [15:0]	     imm_wide;


/*======================================================================*/
/*	processes							*/
/*======================================================================*/

/* mux to select inputs, sign extend immediate data */

assign imm_wide = {16{imm[7]}};


   DW01_mux_any #(96, 2, 24) MUX ({pdb2, pdb2, {imm, 16'b0}, DB }, in_ctl, mux_out);





   
/* enable register.  Reset, enable are active low */

   DW03_reg_s_pl #(24, 0) reg2 (mux_out, Clk, reset, ~write, q);   





   
assign scan_out = q[23];
   
endmodule	/* end module */
