/* $Id: addr_overflow_detect.v,v 1.6 1997/11/19 06:30:08 inacio Exp $ */
/*****************************************************************************
 *addr_overflow_detect
 *
 *detects if the addresses on c_xab, c_pab, or c_yab have gone over the
 *SRAM size
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: inacio $
 *Date: $Date: 1997/11/19 06:30:08 $
 *Revision: $Revision: 1.6 $
 *
 *
 *Log: $Log: addr_overflow_detect.v,v $
 *Log: Revision 1.6  1997/11/19 06:30:08  inacio
 *Log: changed the rest of the num_detect_bits to either data_num_detect_bits or
 *Log: addr_num_detect_bits
 *Log:
# Revision 1.5  1997/11/18  23:39:26  inacio
# added parameters to break up the data and address bus overflow detects.
# They have seperate parameters now, this way instruction memory isn't assumed
# to be twice the size of the data memory
#
# Revision 1.4  1997/11/09  08:51:45  inacio
# I changed the tabbing in the files to be spaces, now it looks the same in
# less as it does in my editor
#
# Revision 1.3  1997/10/29  22:49:40  inacio
# changed the size of the detection to the right size, including the new data
# memory sizes
#
# Revision 1.2  1997/10/15  20:05:59  inacio
# added else statements to make it stop infering latches...
#
# Revision 1.1  1997/10/15  19:57:09  inacio
# added this file to the synthesize directory
#
# Revision 1.3  1997/10/13  21:34:15  inacio
# some small changes for synopsys and make the PAB part correct
#
# Revision 1.2  1997/10/13  20:38:41  inacio
# The is the logic to detect when an address is out of range to any of the
# internal memories
#
# Revision 1.1  1997/10/10  20:30:10  inacio
# some logic to detect if we overflow the internal SRAM sizes
#
 *****************************************************************************/

module addr_overflow_detect (C_XAB, C_PAB, C_YAB, c_xab_ov, c_pab_ov, c_yab_ov);

 parameter data_num_detect_bits = 5;
 parameter addr_num_detect_bits = 5;
 parameter addr_bitwidth = 16;

 input [(addr_bitwidth)-1:0]C_XAB;
 input [(addr_bitwidth)-1:0]C_PAB;
 input [(addr_bitwidth)-1:0]C_YAB;

 output c_xab_ov;
 output c_yab_ov;
 output c_pab_ov;

 reg c_xab_ov;
 reg c_yab_ov;
 reg c_pab_ov;


always @(C_XAB or C_YAB or C_PAB)
begin
 if ( C_XAB[((addr_bitwidth)-1):((addr_bitwidth)-data_num_detect_bits)] != {{data_num_detect_bits {1'b0}}} )
  c_xab_ov <= 1;
 else
  c_xab_ov <= 0;
  
 if ( C_YAB[((addr_bitwidth)-1):((addr_bitwidth)-data_num_detect_bits)] != {{data_num_detect_bits {1'b0}}} )
  c_yab_ov <= 1;
 else
  c_yab_ov <= 0;
  

 if ( C_PAB[((addr_bitwidth)-1):((addr_bitwidth)-addr_num_detect_bits)] != {{addr_num_detect_bits {1'b0}}} )
  c_pab_ov <= 1;
 else
  c_pab_ov <= 0;
  
end


endmodule
