/*	File:	agu_control.v	  					*/
/*	module name: agu_control						*/
/*	Description:	The control logic for the AGU's pipeline stage 2. */
/*	Author: Nitzan Weinberg, Jonathan Tong				*/


/*==============================================================================*/




   `include "./verilog/globals.v"
   `include "./verilog/ctl_def.v"


/********************************************************************************/
/*										*/
/*	module:	agu_control							*/
/*										*/
/********************************************************************************/

module agu_control (
	Clk,
	reset,
	pdb2,
	immediate,
	
	// scan chain controls
	scan_in,
	test,
	scan_out,

	xab_select,	yab_select,
	PAB_2_select,
	xwrite, xread,
	ywrite, yread,
	PABwrite_2,
	gdb_out_select,
	gdb_write,
	addr_alu_H_N_or_1,
	addr_alu_H_add_sub,
	addr_alu_L_N_or_1,
	addr_alu_L_add_sub,
	Rin1_H_select,
	Rraddr1_H_select,
	Rwaddr1_H_select,
	Rwrite1_H_2,
	Rin1_L_select,
	Rraddr1_L_select,
	Rwaddr1_L_select,
	Rwrite1_L_2,
	Nraddr1_H_select,
	Nraddr1_L_select,
	Mraddr1_H_select,
	Mraddr1_L_select,
	Rraddr2_L_select, Rraddr2_H_select,
	Nraddr2_L_select, Nraddr2_H_select,
	Mraddr2_L_select, Mraddr2_H_select,
	Rin2_H_select,
	Rwaddr2_H_select,
	Rwrite2_H_3,
	Rin2_L_select,
	Rwaddr2_L_select,
	Rwrite2_L_3,
	Nin2_H_select,
	Nwaddr2_H_select,
	Nwrite2_H_3,
	Nin2_L_select,
	Nwaddr2_L_select,
	Nwrite2_L_3,
	Min2_H_select,
	Mwaddr2_H_select,
	Mwrite2_H_3,
	Min2_L_select,
	Mwaddr2_L_select,
	Mwrite2_L_3,
	addr_modifier_out,
	wrap_once_L_ena,
	wrap_once_H_ena,

	// guard latches controls
   	addr_aluL_ena,
	addr_aluH_ena
	

	);


/*==============================================================================*/
/*============	I/O direction	================================================*/
/*==============================================================================*/

input Clk;
input reset;
input [`databus] pdb2;
input immediate;
input [7:0] addr_modifier_out;


input scan_in;
input test;
output scan_out;

output [2:0] xab_select;
output [2:0] yab_select;
output [1:0] PAB_2_select;
output xwrite;
output xread;
output ywrite;
output yread;
output PABwrite_2;
output [2:0] gdb_out_select;
output gdb_write;
output addr_alu_H_N_or_1;
output addr_alu_H_add_sub;
output addr_alu_L_N_or_1;
output addr_alu_L_add_sub;
output [1:0] Rin1_H_select;
output Rraddr1_H_select;
output [1:0] Rwaddr1_H_select;
output Rwrite1_H_2;
output [1:0] Rin1_L_select;
output Rraddr1_L_select;
output [1:0] Rwaddr1_L_select;
output Rwrite1_L_2;
output Nraddr1_H_select;
output Nraddr1_L_select;
output Mraddr1_H_select;
output Mraddr1_L_select;
output [1:0] Rraddr2_L_select;
output [1:0] Rraddr2_H_select;
output [1:0] Nraddr2_L_select;
output [1:0] Nraddr2_H_select;
output [1:0] Mraddr2_L_select;
output [1:0] Mraddr2_H_select;

output Rin2_H_select;
output Rwaddr2_H_select;
output Rwrite2_H_3;
output Rin2_L_select;
output Rwaddr2_L_select;
output Rwrite2_L_3;
output Nin2_H_select;
output Nwaddr2_H_select;
output Nwrite2_H_3;
output Nin2_L_select;
output Nwaddr2_L_select;
output Nwrite2_L_3;
output Min2_H_select;
output Mwaddr2_H_select;
output Mwrite2_H_3;
output Min2_L_select;
output Mwaddr2_L_select;
output Mwrite2_L_3;


// guard latches controls
output addr_aluL_ena;
output addr_aluH_ena;
output wrap_once_L_ena;
output wrap_once_H_ena;

/*--------------------------------------------------------------------------------------------------*/
/* address alu control signals and output															*/
/*--------------------------------------------------------------------------------------------------*/

`define take_Nn 0				/* a control signal for addr_alu_?_N_or_1 choosing Nn */
`define take_1  1				/* a control signal for addr_alu_?_N_or_1 choosing 1 */

`define add     0				/* a control signal for addr_alu_?_add_sub choosing to add */
`define sub     1				/* a control signal for addr_alu_?_add_sub choosing to add */

/*==============================================================================*/
/*============		I/O type	================================================*/
/*==============================================================================*/

wire reset;				/* core reset signal */

/*----------------------------------------------------------------------------------*/
/* fetched instruction coming from the PDB, and passed on along the pipeline stages */
/*----------------------------------------------------------------------------------*/

wire  [`databus] pdb2;	/* for pipeline stage 2 */


/*-------------------------------------------------------*/
/* X, Y and P memory address buses - internal to the AGU */
/*-------------------------------------------------------*/

`define xab_take_Rout1_L           3'h0		/* Rout1_L is chosen as the data input into xab */
`define xab_take_Rout1_H           3'h1		/* Rout1_H is chosen as the data input into xab */
`define xab_take_addr_alu_L_out    3'h2		/* addr_alu_L_out is chosen as the data input into xab */
`define xab_take_addr_alu_H_out    3'h3		/* addr_alu_H_out is chosen as the data input into xab */
`define xab_take_pdb2_13_8         3'h4		/* {10'h000, pdb2[13:8]} is chosen as the data input into xab */

reg [2:0] xab_select;		/* select the input into xab  -  coming ONLY from stage 2 */
				/* (xab_select = 0) => (xab = Rout1_L) */
				/* (xab_select = 1) => (xab = Rout1_H) */
				/* (xab_select = 2) => (xab = addr_alu_L_out) */
				/* (xab_select = 3) => (xab = addr_alu_H_out) */
			        /* (xab_select = 4) => (xab = {10'h000, pdb2[13:8]}) */


`define yab_take_Rout1_L           3'h0		/* Rout1_L is chosen as the data input into yab */
`define yab_take_Rout1_H           3'h1		/* Rout1_H is chosen as the data input into yab */
`define yab_take_addr_alu_L_out    3'h2		/* addr_alu_L_out is chosen as the data input into yab */
`define yab_take_addr_alu_H_out    3'h3		/* addr_alu_H_out is chosen as the data input into yab */
`define yab_take_pdb2_13_8         3'h4		/* {10'h000, pdb2[13:8]} is chosen as the data input into yab */

reg [2:0] yab_select;		/* select the input into yab  -  coming ONLY from stage 2 */
				/* (yab_select = 0) => (yab = Rout1_L) */
				/* (yab_select = 1) => (yab = Rout1_H) */
				/* (yab_select = 2) => (yab = addr_alu_L_out) */
				/* (yab_select = 3) => (yab = addr_alu_H_out) */
				/* (yab_select = 4) => (yab = {10'h000, pdb2[13:8]}) */


`define PAB_2_take_Rout1_L           2'h0		/* Rout1_L is chosen as the data input into PAB_2 */
`define PAB_2_take_Rout1_H           2'h1		/* Rout1_H is chosen as the data input into PAB_2 */
`define PAB_2_take_addr_alu_L_out    2'h2		/* addr_alu_L_out is chosen as the data input into PAB_2 */
`define PAB_2_take_addr_alu_H_out    2'h3		/* addr_alu_H_out is chosen as the data input into PAB_2 */

reg [1:0] PAB_2_select;		/* select the input into PAB_2  -  coming ONLY from stage 2 */
				/* (PAB_2_select = 0) => (PAB_2_select = Rout1_L) */
				/* (PAB_2_select = 1) => (PAB_2_select = Rout1_H) */
				/* (PAB_2_select = 2) => (PAB_2_select = addr_alu_L_out) */
				/* (PAB_2_select = 3) => (PAB_2_select = addr_alu_H_out) */




/*---------------------------------------------------------------------*/
/* The GDB output data bus and drive-control from the agu logic unit   */
/*---------------------------------------------------------------------*/

`define gdb_out_take_Rin1_L     3'h0		/* Rin1_L  is chosen as the data input into gdb_out */
`define gdb_out_take_Rin1_H     3'h1		/* Rin1_H  is chosen as the data input into gdb_out */
`define gdb_out_take_Rout2_L    3'h2		/* Rout2_L is chosen as the data input into gdb_out */
`define gdb_out_take_Rout2_H    3'h3		/* Rout2_H is chosen as the data input into gdb_out */
`define gdb_out_take_Nout2_L    3'h4		/* Nout2_L is chosen as the data input into gdb_out */
`define gdb_out_take_Nout2_H    3'h5		/* Nout2_H is chosen as the data input into gdb_out */
`define gdb_out_take_Mout2_L    3'h6		/* Mout2_L is chosen as the data input into gdb_out */
`define gdb_out_take_Mout2_H    3'h7		/* Mout2_H is chosen as the data input into gdb_out */

reg [2:0] gdb_out_select;	/* select the input into gdb_out  -  coming ONLY from stage 2 */
				/* (gdb_out_select = 0) => (gdb_out = Rout2_L) */
				/* (gdb_out_select = 1) => (gdb_out = Rout2_H) */
				/* (gdb_out_select = 2) => (gdb_out = Nout2_L) */
				/* (gdb_out_select = 3) => (gdb_out = Nout2_H) */
				/* (gdb_out_select = 4) => (gdb_out = Mout2_L) */
				/* (gdb_out_select = 5) => (gdb_out = Mout2_H) */
				/* (gdb_out_select = 6) => (gdb_out = Rin3_L) */
				/* (gdb_out_select = 7) => (gdb_out = Rin3_H) */
				

reg gdb_write;			/* GDB_out write control - from stage 2 */



/*------*/
/* High */
/*------*/

reg addr_alu_H_N_or_1;				/* (addr_alu_H_N_or_1 = 0) => (choose Nn)		*/
						/* (addr_alu_H_N_or_1 = 1) => (choose 1)		*/

reg addr_alu_H_add_sub;		/* (addr_alu_H_add_sub = 0) => (add Rn with either Nn or 1, depends on linear_N_or_1)	*/
				/* (addr_alu_H_add_sub = 1) => (subtract Rn with either Nn or 1 depends on linear_N_or_1))*/


/*------*/
/* Low  */
/*------*/

reg addr_alu_L_N_or_1;				/* (addr_alu_L_N_or_1 = 0) => (choose Nn)		*/
						/* (addr_alu_L_N_or_1 = 1) => (choose 1)		*/

reg addr_alu_L_add_sub;		/* (addr_alu_L_add_sub = 0) => (add Rn with either Nn or 1, depends on linear_N_or_1)	*/
				/* (addr_alu_L_add_sub = 1) => (subtract Rn with either Nn or 1 depends on linear_N_or_1))*/



/*--------------------------------------------------------------*/
/* R register-files		Control and buses 						*/
/*--------------------------------------------------------------*/

/*-------------------*/
/* 		High R		 */
/*-------------------*/

`define Rin1_H_take_addr_alu_H_out   2'h0	/* addr_alu_H_out is chosen as the data input into Rin1_H */
`define Rin1_H_take_GDB              2'h1	/* GDB[`addrbus] is chosen as the data input into Rin1_H */
`define Rin1_H_take_addr_alu_L_out   2'h2

reg [1:0] Rin1_H_select;				/* select the input into Rin1_H  -  coming from stage 2 */
						/* (Rin1_H_select = `Rin1_H_take_addr_alu_H_out) => (Rin1_H = addr_alu_H_out) */
						/* (Rin1_H_select = 1) => (Rin1_H = GDB[`addrbus]) */


`define Rraddr1_H_take_pdb2_9_8    1'h0		/* pdb2[9:8] is chosen as the address into Rraddr1_H */
`define Rraddr1_H_take_pdb2_14_13  1'h1		/* pdb2[14:13] is chosen as the address into Rraddr1_H */

reg Rraddr1_H_select;				/* selects the input into Rraddr1_H. */
						/* (Rraddr1_H_select = 0) => (Rraddr1_H = pdb2[9:8]) */
						/* (Rraddr1_H_select = 1) => (Rraddr1_H = pdb2[14:13]) */


`define Rwaddr1_H_take_pdb2_9_8    2'h0		/* pdb2[9:8] is chosen as the address into Rwaddr1_H */
`define Rwaddr1_H_take_pdb2_14_13  2'h1		/* pdb2[14:13] is chosen as the address into Rwaddr1_H */
`define Rwaddr1_H_take_pdb2_17_16  2'h2		/* pdb2[17:16] is chosen as the address into Rwaddr1_H */
`define Rwaddr1_H_take_pdb2_1_0  2'h3		/* pdb2[1:0] is chosen as the address into Rwaddr1_H */

reg [1:0] Rwaddr1_H_select;			/* selects the input into Rwaddr1_H. */
						/* (Rwaddr1_H_select = 0) => (Rwaddr1_H = pdb2[9:8]) */
						/* (Rwaddr1_H_select = 1) => (Rwaddr1_H = pdb2[14:13]) */
						/* (Rwaddr1_H_select = 2) => (Rwaddr1_H = pdb2[17:16]) */
						/* (Rwaddr1_H_select = 3) => (Rwaddr1_H = pdb2[1:0]) */




`define Rin2_H_take_GDB    	1'b0		/* GDB[`addrbus] is chosen as the data input into Rin2_H */
`define Rin2_H_take_imm_data    1'b1		/* {8'h00, pdb2[15:8]} is chosen as the data input into Rin2_H */

reg Rin2_H_select;			/* select the input into Rin2_H  -  coming from stage 3 */
						/* (Rin2_H_select = 0) => (Rin2_H = NORM_add_sub_out) */
						/* (Rin2_H_select = 1) => (Rin2_H = bit_test_out) */
						/* (Rin2_H_select = 2) => (Rin2_H = GDB[`addrbus]) */
						/* (Rin2_H_select = 3) => (Rin2_H = {8'h00, pdb2[15:8]}) short immediate data */



`define Rwaddr2_H_take_pdb2_9_8    1'h0		/* pdb2[9:8] is chosen as the address into Rwaddr2_H */
`define Rwaddr2_H_take_pdb2_17_16  1'h1		/* pdb2[17:16] is chosen as the address into Rwaddr2_H */

reg Rwaddr2_H_select;				/* selects the input into Rwaddr2_H. */
						/* (Rwaddr2_H_select = 0) => (Rwaddr2_H = pdb2[9:8]) */
						/* (Rwaddr2_H_select = 1) => (Rwaddr2_H = pdb2[17:16]) */





/*-------------------*/
/* 		Low R		 */
/*-------------------*/

`define Rin1_L_take_addr_alu_L_out   2'h0	/* addr_alu_L_out is chosen as the data input into Rin1_L */
`define Rin1_L_take_GDB              2'h1	/* GDB[`addrbus] is chosen as the data input into Rin1_L */
`define Rin1_L_take_addr_alu_H_out   2'h2

reg [1:0] Rin1_L_select;			/* select the input into Rin1_L  -  coming from stage 2 */
						/* (Rin1_L_select = `Rin1_L_take_addr_alu_L_out) => (Rin1_L = addr_alu_L_out) */
						/* (Rin1_L_select = 1) => (Rin1_L = GDB[`addrbus]) */



`define Rraddr1_L_take_pdb2_9_8    0		/* pdb2[9:8] is chosen as the address into Rraddr1_L */
`define Rraddr1_L_take_pdb2_14_13  1		/* pdb2[14:13] is chosen as the address into Rraddr1_L */

reg Rraddr1_L_select;				/* selects the input into Rraddr1_L. */
						/* (Rraddr1_L_select = 0) => (Rraddr1_L = pdb2[9:8]) */
						/* (Rraddr1_L_select = 0) => (Rraddr1_L = pdb2[14:13]) */


`define Rwaddr1_L_take_pdb2_9_8    2'h0		/* pdb2[9:8] is chosen as the address into Rwaddr1_L */
`define Rwaddr1_L_take_pdb2_14_13  2'h1		/* pdb2[14:13] is chosen as the address into Rwaddr1_L */
`define Rwaddr1_L_take_pdb2_17_16  2'h2		/* pdb2[17:16] is chosen as the address into Rwaddr1_L */
`define Rwaddr1_L_take_pdb2_1_0    2'h3		/* pdb2[1:0] is chosen as the address into Rwaddr1_L */

reg [1:0] Rwaddr1_L_select;			/* selects the input into Rwaddr1_L. */
						/* (Rwaddr1_L_select = 0) => (Rwaddr1_L = pdb2[9:8]) */
						/* (Rwaddr1_L_select = 1) => (Rwaddr1_L = pdb2[14:13]) */
						/* (Rwaddr1_L_select = 2) => (Rwaddr1_L = pdb2[17:16]) */
						/* (Rwaddr1_L_select = 3) => (Rwaddr1_L = pdb2[1:0]) */




`define Rin2_L_take_GDB    	1'b0		/* GDB[`addrbus] is chosen as the data input into Rin2_L */
`define Rin2_L_take_imm_data    1'b1		/* {8'h00, pdb2[15:8]} is chosen as the data input into Rin2_L */

reg Rin2_L_select;			/* select the input into Rin2_L  -  coming from stage 3 */
						/* (Rin2_L_select = 0) => (Rin2_L = NORM_add_sub_out) */
						/* (Rin2_L_select = 1) => (Rin2_L = bit_test_out) */
						/* (Rin2_L_select = 2) => (Rin2_L = GDB[`addrbus]) */
						/* (Rin2_L_select = 3) => (Rin2_L = {8'h00, pdb2[15:8]}) short immediate data */


`define Rwaddr2_L_take_pdb2_9_8    1'h0		/* pdb2[9:8] is chosen as the address into Rwaddr2_L */
`define Rwaddr2_L_take_pdb2_17_16  1'h1		/* pdb2[17:16] is chosen as the address into Rwaddr2_L */

reg Rwaddr2_L_select;				/* selects the input into Rwaddr2_L. */
						/* (Rwaddr2_L_select = 0) => (Rwaddr2_L = pdb2[9:8]) */
						/* (Rwaddr2_L_select = 1) => (Rwaddr2_L = pdb2[17:16]) */





/*--------------------------------------------------------------*/
/* N register-files		Control and buses 		*/
/*--------------------------------------------------------------*/

/*----------------------*/
/* 		High N 			*/
/*----------------------*/

`define Nraddr1_H_take_pdb2_9_8    1'h0		/* pdb2[9:8] is chosen as the address into Nraddr1_H */
`define Nraddr1_H_take_pdb2_14_13  1'h1		/* pdb2[14:13] is chosen as the address into Nraddr1_H */

reg Nraddr1_H_select;				/* selects the input into Nraddr1_H. */
						/* (Nraddr1_H_select = 0) => (Nraddr1_H = pdb2[9:8]) */
						/* (Nraddr1_H_select = 1) => (Nraddr1_H = pdb2[14:13]) */




`define Nin2_H_take_GDB    		1'b0		/* GDB[`addrbus] is chosen as the data input into Nin2_H */
`define Nin2_H_take_imm_data    1'b1		/* {8'h00, pdb2[15:8]} is chosen as the data input into Nin2_H */

reg Nin2_H_select;			/* select the input into Nin2_H  -  coming from stage 3 */
						/* (Nin2_H_select = 0) => (Nin2_H = bit_test_out) */
						/* (Nin2_H_select = 1) => (Nin2_H = GDB[`addrbus]) */
						/* (Nin2_H_select = 2) => (Nin2_H = {8'h00, pdb2[15:8]}) short immediate data */



`define Nwaddr2_H_take_pdb2_9_8    1'h0		/* pdb2[9:8] is chosen as the address into Nwaddr2_H */
`define Nwaddr2_H_take_pdb2_17_16  1'h1		/* pdb2[17:16] is chosen as the address into Nwaddr2_H */

reg Nwaddr2_H_select;				/* selects the input into Nwaddr2_H. */
						/* (Nwaddr2_H_select = 0) => (Nwaddr2_H = pdb2[9:8]) */
						/* (Nwaddr2_H_select = 1) => (Nwaddr2_H = pdb2[17:16]) */





/*----------------------*/
/* 		Low N 	*/
/*----------------------*/

`define Nraddr1_L_take_pdb2_9_8    1'b0		/* pdb2[9:8] is chosen as the address into Nraddr1_L */
`define Nraddr1_L_take_pdb2_14_13  1'b1		/* pdb2[14:13] is chosen as the address into Nraddr1_L */

reg Nraddr1_L_select;				/* selects the input into Nraddr1_L. */
						/* (Nraddr1_L_select = 0) => (Nraddr1_L = pdb2[9:8]) */
						/* (Nraddr1_L_select = 1) => (Nraddr1_L = pdb2[14:13]) */




`define Nin2_L_take_GDB    		1'b0	/* GDB[`addrbus] is chosen as the data input into Nin2_L */
`define Nin2_L_take_imm_data    1'b1		/* {8'h00, pdb2[15:8]} is chosen as the data input into Nin2_L */

reg Nin2_L_select;			/* select the input into Nin2_L  -  coming from stage 3 */
						/* (Nin2_L_select = 0) => (Nin2_L = bit_test_out) */
						/* (Nin2_L_select = 1) => (Nin2_L = GDB[`addrbus]) */
						/* (Nin2_L_select = 2) => (Nin2_L = {8'h00, pdb2[15:8]}) short immediate data */


`define Nwaddr2_L_take_pdb2_9_8    1'h0		/* pdb2[9:8] is chosen as the address into Nwaddr2_L */
`define Nwaddr2_L_take_pdb2_17_16  1'h1		/* pdb2[17:16] is chosen as the address into Nwaddr2_L */

reg Nwaddr2_L_select;				/* selects the input into Nwaddr2_L. */
						/* (Nwaddr2_L_select = 0) => (Nwaddr2_L = pdb2[9:8]) */
						/* (Nwaddr2_L_select = 1) => (Nwaddr2_L = pdb2[17:16]) */



/*--------------------------------------------------------------*/
/* M register-files		Control and buses						*/
/*--------------------------------------------------------------*/

/*----------------------*/
/* 		High M 			*/
/*----------------------*/

`define Min2_H_take_GDB    	1'b0		/* GDB[`addrbus] is chosen as the data input into Min2_H */
`define Min2_H_take_imm_data    1'b1		/* {8'h00, pdb2[15:8]} is chosen as the data input into Min2_H */

reg  Min2_H_select;			/* select the input into Min2_H  -  coming from stage 3 */
						/* (Min2_H_select = 0) => (Min2_H = bit_test_out) */
						/* (Min2_H_select = 1) => (Min2_H = GDB[`addrbus]) */
						/* (Min2_H_select = 2) => (Min2_H = {8'h00, pdb2[15:8]}) short immediate data */


`define Mwaddr2_H_take_pdb2_9_8    1'h0		/* pdb2[9:8] is chosen as the address into Mwaddr2_H */
`define Mwaddr2_H_take_pdb2_1_0    1'h1		/* pdb2[1:0] is chosen as the address into Mwaddr2_H */

reg Mwaddr2_H_select;				/* selects the input into Mwaddr2_H. */
						/* (Mwaddr2_H_select = 0) => (Mwaddr2_H = pdb2[9:8]) */
						/* (Mwaddr2_H_select = 1) => (Mwaddr2_H = pdb2[1:0]) */




`define Mraddr1_H_take_pdb2_9_8    1'h0		/* pdb2[9:8] is chosen as the address into Mraddr1_H */
`define Mraddr1_H_take_pdb2_14_13  1'h1		/* pdb2[14:13] is chosen as the address into Mraddr1_H */

reg Mraddr1_H_select;				/* selects the input into Mraddr1_H. */
						/* (Mraddr1_H_select = 0) => (Mraddr1_H = pdb2[9:8]) */
						/* (Mraddr1_H_select = 1) => (Mraddr1_H = pdb2[14:13]) */





/*----------------------*/
/* 		Low M 	*/
/*----------------------*/

`define Min2_L_take_GDB    	1'b0		/* GDB[`addrbus] is chosen as the data input into Min2_L */
`define Min2_L_take_imm_data    1'b1		/* {8'h00, pdb2[15:8]} is chosen as the data input into Min2_L */

reg Min2_L_select;			/* select the input into Min2_L  -  coming from stage 3 */
						/* (Min2_L_select = 0) => (Min2_L = bit_test_out) */
						/* (Min2_L_select = 1) => (Min2_L = GDB[`addrbus]) */
						/* (Min2_L_select = 2) => (Min2_L = {8'h00, pdb2[15:8]}) short immediate data */


`define Mwaddr2_L_take_pdb2_9_8    1'h0		/* pdb2[9:8] is chosen as the address into Mwaddr2_L */
`define Mwaddr2_L_take_pdb2_1_0    1'h1		/* pdb2[17:16] is chosen as the address into Mwaddr2_L */

reg Mwaddr2_L_select;				/* selects the input into Mwaddr2_L. */
						/* (Mwaddr2_L_select = 0) => (Mwaddr2_L = pdb2[9:8]) */
						/* (Mwaddr2_L_select = 1) => (Mwaddr2_L = pdb2[1:0]) */





`define Mraddr1_L_take_pdb2_9_8    1'h0		/* pdb2[9:8] is chosen as the address into Mraddr1_L */
`define Mraddr1_L_take_pdb2_14_13  1'h1		/* pdb2[14:13] is chosen as the address into Mraddr1_L */

reg Mraddr1_L_select;				/* selects the input into Mraddr1_L. */
						/* (Mraddr1_L_select = 0) => (Mraddr1_L = pdb2[9:8]) */
						/* (Mraddr1_L_select = 1) => (Mraddr1_L = pdb2[14:13]) */



reg [1:0] Rraddr2_L_select;   
reg [1:0] Rraddr2_H_select;
reg [1:0] Nraddr2_L_select;
reg [1:0] Nraddr2_H_select;
reg [1:0] Mraddr2_L_select;
reg [1:0] Mraddr2_H_select;

`define take_pdb3_9_8 	2'b00
`define take_pdb3_14_13 2'b01
`define take_pdb3_17_16 2'b10
`define take_pdb3_1_0 	2'b10


// guard latches controls
reg addr_aluL_ena;
reg addr_aluH_ena;
reg wrap_once_L_ena;
reg wrap_once_H_ena;

/*==============================================================================*/
/*============		Internal Nets	========================================*/
/*==============================================================================*/

reg xwrite_t;
reg xread_t;
reg ywrite_t;
reg yread_t;
reg PABwrite_2_t;
reg [2:0] gdb_out_select_d;
reg gdb_write_d;

reg [1:0] Rraddr2_L_select_d;
reg [1:0] Rraddr2_H_select_d;
reg [1:0] Nraddr2_L_select_d;
reg [1:0] Nraddr2_H_select_d;
reg [1:0] Mraddr2_L_select_d;
reg [1:0] Mraddr2_H_select_d;

reg Rin2_H_select_d;
reg Rwaddr2_H_select_d;
reg Rwrite2_H_3_d;
reg Rin2_L_select_d;
reg Rwaddr2_L_select_d;
reg Rwrite2_L_3_d;
reg Nin2_H_select_d;
reg Nwaddr2_H_select_d;
reg Nwrite2_H_3_d;
reg Nin2_L_select_d;
reg Nwaddr2_L_select_d;
reg Nwrite2_L_3_d;
reg Min2_H_select_d;
reg Mwaddr2_H_select_d;
reg Mwrite2_H_3_d;
reg Min2_L_select_d;
reg Mwaddr2_L_select_d;
reg Mwrite2_L_3_d;


// signals that control register enables

reg Rwrite1_L_2_t;
reg Rwrite1_H_2_t;

reg Rwrite2_L_3_t;
reg Nwrite2_L_3_t;
reg Mwrite2_L_3_t;
reg Rwrite2_H_3_t;
reg Nwrite2_H_3_t;
reg Mwrite2_H_3_t;
wire xwrite;
wire xread;
wire ywrite;
wire yread;
wire PABwrite_2; 

wire Rwrite1_L_2;
wire Nwrite1_L_2;
wire Mwrite1_L_2;
wire Rwrite1_H_2;
wire Nwrite1_H_2;
wire Mwrite1_H_2;

wire Rwrite2_L_3;
wire Nwrite2_L_3;
wire Mwrite2_L_3;
wire Rwrite2_H_3;
wire Nwrite2_H_3;
wire Mwrite2_H_3;

// for adding guard latches to adders in addr ALUS
// use as an index to read M registers
reg [1:0]  indexL, indexH;

/*==============================================================================*/
task mem_control_signals;
begin
			case ({pdb2[19], pdb2[15]})/* memory access */
				  2'b00	:	xwrite_t = `true;
				  2'b01	:	xread_t  = `true;
				  2'b10	:	ywrite_t = `true;
				  2'b11	:	yread_t  = `true;
			endcase	
end
endtask		

/*==============================================================================*/
/*	processes								*/
/*==============================================================================*/

/*==============================================================================*/
/*==============================================================================*/
/*										*/
/*			Stage 2	(Decode)					*/				
/*==============================================================================*/
/*==============================================================================*/


/*------------------------------------------------------------------------------*/
/*										*/
/*	Decode and carry out the instruction.					*/
/*										*/
/*------------------------------------------------------------------------------*/

always @(reset or pdb2 or immediate )
begin

    
	/* Initially don't allow any writes to any of the register files */														   
	Rwrite1_L_2_t = `false;
	Rwrite1_H_2_t = `false;
				
	/* initially don't read or write any of the memories (false) */
		
	xwrite_t = `false;
	xread_t  = `false;
		
	ywrite_t = `false;
	yread_t  = `false;
		
	PABwrite_2_t = `false;

	/* initially don't write to the GDB bus */

	gdb_write_d = `false;


	/* don't care */

	xab_select = 3'b111;  /* 6-to-1 mux so '111' doesn't correspond ..*/
	yab_select = 3'b111;  /* .. to any useful  inputs  */
	PAB_2_select = 2'b0;
	gdb_out_select_d = 4'b0;
	addr_alu_H_N_or_1 = 1'b0;
	addr_alu_H_add_sub = 1'b0;
	addr_alu_L_N_or_1 = 1'b0;
	addr_alu_L_add_sub = 1'b0;
	Rin1_H_select = 2'b11;   /* 3-to-1 mux so '11' doesn't correspond */
	Rraddr1_H_select = 1'b0; /* to any useful inputs */
	Rwaddr1_H_select = 2'b0;
	Rin1_L_select = 2'b11;
	Rraddr1_L_select = 1'b0;
	Rwaddr1_L_select = 2'b0;
	Nraddr1_H_select = 1'b0;
	Nraddr1_L_select = 1'b0;
	Mraddr1_H_select = 1'b0;
	Mraddr1_L_select = 1'b0;
		
	Rraddr2_L_select_d = 2'b0;   
	Rraddr2_H_select_d = 2'b0;
	Nraddr2_L_select_d = 2'b0;
	Nraddr2_H_select_d = 2'b0;
	Mraddr2_L_select_d = 2'b0;
	Mraddr2_H_select_d = 2'b0;

	Rin2_H_select_d = 1'b0;
	Rwaddr2_H_select_d = 1'b0;
	Rwrite2_H_3_d = 1'b0;
	Rin2_L_select_d = 1'b0;
	Rwaddr2_L_select_d = 1'b0;
	Rwrite2_L_3_d = 1'b0;
	Nin2_H_select_d = 1'b0;
	Nwaddr2_H_select_d = 1'b0;
	Nwrite2_H_3_d = 1'b0;
	Nin2_L_select_d = 1'b0;
	Nwaddr2_L_select_d = 1'b0;
	Nwrite2_L_3_d = 1'b0;
	Min2_H_select_d = 1'b0;
	Mwaddr2_H_select_d = 1'b0;
	Mwrite2_H_3_d = 1'b0;
	Min2_L_select_d = 1'b0;
	Mwaddr2_L_select_d = 1'b0;
	Mwrite2_L_3_d = 1'b0;


   
	/*--------------------------------------------------------------------------*/
	/*								            */
	/* 		Decode the incoming instruction	          		    */
	/*									    */
	/*--------------------------------------------------------------------------*/
	if (reset)
	  begin
	     $display($time, " AGU_control: reset is high\n");
	  end 		
	else if ( pdb2[`movefield] == `no_parallel_data_move )
	  begin		/*--------------------------------------*/
			/* No parallel data move is required	*/
			/* ALU instructuions only		*/
			/* No Action required for AGU        	*/
			/*--------------------------------------*/
	  end
	else if (immediate)
	  begin
			/*--------------------------------------*/
			/*  pdb2 is an immediate data           */
			/*  No decoding needed                  */
			/*--------------------------------------*/
	     $display($time, " AGU_control: immediate signal is high!\n");
	     
	  end

	else if ( {pdb2[23:14], pdb2[7:0]} == 18'b0000_1010_11_1000_0000 )
	  begin	
			/*--------------------------------------*/
			/*	JMP Class II			*/
			/*					*/
			/* Jump with an effective address	*/
			/*--------------------------------------*/

			/*------------------------------------------*/
			/*	Effective Address Calculation	    */
			/*------------------------------------------*/
			/* calculate the effective address for the jump and UPDATE the address register that is being used */
				
		case (pdb2[13:11])
		  3'b000	:/* (Rn)-Nn */
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;	/* M read  address */
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;	/* N read  address */
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;	/* R read  address */	
				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;	/* R write address */
				Rwrite1_H_2_t = `true;			/* enable write */		
				PAB_2_select = `PAB_2_take_Rout1_H;		/* memory address */
				PABwrite_2_t  = `true;
				addr_alu_H_N_or_1 = `take_Nn;		/* choose Nn	*/
				addr_alu_H_add_sub = `sub;		/* subtract	*/											Rin1_H_select = `Rin1_H_take_addr_alu_H_out;	/* select output from addr_alu */
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;	/* M read  address */
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;	/* N read  address */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;	/* R read  address */
				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;	/* R write address */
				Rwrite1_L_2_t = `true;		/* enable write */
				PAB_2_select = `PAB_2_take_Rout1_L;	/* memory address */
				PABwrite_2_t  = `true;

				addr_alu_L_N_or_1 = `take_Nn;		/* choose Nn	*/
				addr_alu_L_add_sub = `sub;		/* subtract	*/
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;	/* select output from addr_alu */
			  end		/* (Rn)-Nn */

		  3'b001	:	/* (Rn)+Nn */
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;	/* M read  address */
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;	/* N read  address */
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;	/* R read  address */
				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;	/* R write address */
				Rwrite1_H_2_t = `true;			/* enable write */

				PAB_2_select = `PAB_2_take_Rout1_H;	/* memory address */
				PABwrite_2_t  = `true;

				addr_alu_H_N_or_1 = `take_Nn;		/* choose Nn	*/
				addr_alu_H_add_sub = `add;	        /* add		*/
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;			/* select output from addr_alu */
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;	/* M read  address */
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;	/* N read  address */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;	/* R read  address */
				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;	/* R write address */
				Rwrite1_L_2_t = `true;			/* enable write */
				PAB_2_select = `PAB_2_take_Rout1_L;	/* memory address */
				PABwrite_2_t  = `true;

				addr_alu_L_N_or_1 = `take_Nn;			/* choose Nn	*/
				addr_alu_L_add_sub = `add;			/* add		*/
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;	/* select output from addr_alu */
			  end		/* (Rn)+Nn */

		  3'b010	:	/* (Rn)- */
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;/* M read  address */
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;/* R read  address */
				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;	/* R write address */
				Rwrite1_H_2_t = `true;		/* enable write */
				PAB_2_select = `PAB_2_take_Rout1_H;	/* memory address */
				PABwrite_2_t  = `true;
				addr_alu_H_N_or_1 = `take_1;	/* choose 1	*/
				addr_alu_H_add_sub = `sub;		/* subtract	*/
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;			/* select output from addr_alu */
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;			/* M read  address */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;			/* R read  address */

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;			/* R write address */
				Rwrite1_L_2_t = `true;			/* enable write */
				PAB_2_select = `PAB_2_take_Rout1_L;		/* memory address */
				PABwrite_2_t  = `true;

				addr_alu_L_N_or_1 = `take_1;			/* choose 1	*/
				addr_alu_L_add_sub = `sub;			/* subtract		*/
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;			/* select output from addr_alu */
			  end		/* (Rn)- */

		  3'b011	:	/* (Rn)+ */
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;			/* M read  address */
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;			/* R read  address */
				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;			/* R write address */
				Rwrite1_H_2_t = `true;	/* enable write */
				PAB_2_select = `PAB_2_take_Rout1_H;	/* memory address */
				PABwrite_2_t  = `true;
				addr_alu_H_N_or_1 = `take_1;		/* choose 1	*/
				addr_alu_H_add_sub = `add;		/* add		*/
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;			/* select output from addr_alu */
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;			/* M read  address */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;			/* R read  address */

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;			/* R write address */
				Rwrite1_L_2_t = `true;	/* enable write */
				PAB_2_select = `PAB_2_take_Rout1_L;	/* memory address */
				PABwrite_2_t  = `true;
				addr_alu_L_N_or_1 = `take_1;	/* choose 1	*/
				addr_alu_L_add_sub = `add;	/* add*/
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;			/* select output from addr_alu */
			  end		/* (Rn)+ */

		  3'b100	:	/* (Rn) */
			if (pdb2[10])
			  begin	/* High */
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;			/* R read  address */
				PAB_2_select = `PAB_2_take_Rout1_H;	/* memory address */
				PABwrite_2_t  = `true;
			  end
			else
			  begin	/* Low */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;			/* R read  address */
				PAB_2_select = `PAB_2_take_Rout1_L;						/* memory address */
				PABwrite_2_t  = `true;
			  end		/* (Rn) */

		  3'b101	:	/* (Rn+Nn) */
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;			/* M read  address */
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;			/* N read  address */
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;			/* R read  address */
				addr_alu_H_N_or_1 = `take_Nn;	  /* choose Nn	*/
				addr_alu_H_add_sub = `add;		/* add	*/
				PAB_2_select = `PAB_2_take_addr_alu_H_out;	/* memory address */
				PABwrite_2_t  = `true;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;			/* M read  address */
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;			/* N read  address */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;			/* R read  address */

				addr_alu_L_N_or_1 = `take_Nn;		/* choose Nn	*/
				addr_alu_L_add_sub = `add;		/* add		*/

				PAB_2_select = `PAB_2_take_addr_alu_L_out;	/* memory address */
				PABwrite_2_t  = `true;
			  end		/* (Rn+Nn) */

		  3'b110	:	/* Absolute_D address */
				/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
				/* Jump using a second program word as an absolute_d address*/ 
				/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
				begin
					
				end

		  3'b111	:	/* -(Rn) */
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;			/* M read  address */
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;			/* R read  address */

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;			/* R write address */
				Rwrite1_H_2_t = `true;			/* enable write */

				PAB_2_select = `PAB_2_take_addr_alu_H_out;		/* memory address */
				PABwrite_2_t  = `true;
				addr_alu_H_N_or_1 = `take_1;		/* choose 1	*/
				addr_alu_H_add_sub = `sub;		/* subtract	*/
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;			/* select output from addr_alu */
			  end
			else
		          begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;			/* M read  address */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;			/* R read  address */

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;			/* R write address */
				Rwrite1_L_2_t = `true;		/* enable write */

				PAB_2_select = `PAB_2_take_addr_alu_L_out;		/* memory address */
				PABwrite_2_t  = `true;
				addr_alu_L_N_or_1 = `take_1;	/* choose 1	*/
				addr_alu_L_add_sub = `sub;	/* subtract	*/
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;			/* select output from addr_alu */
			  end		/* -(Rn) */

		  endcase	/* pdb2[13:11] */
				

	  end		/* JMP Class II */


	else if ( {pdb2[23:14], pdb2[7:0]} == 18'b00000110_11_00100000 )
	  begin	/*--------------------------------------*/
		/*		REP Class IV		*/
		/*					*/
		/* Repeat Next Instruction		*/
		/*--------------------------------------*/

		/* The number of repetitions is in one if the AGU registers. To be put on the GDB */
				
		case (pdb2[13:11])
		  3'b010:  begin	/*---*/
					/* R */
					/*---*/
			gdb_write_d = `true;		/* put on the GDB */									
			if (pdb2[10])			/* choose the register */
			  begin	/* High */
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;		/* R read  address */
				gdb_out_select_d = `gdb_out_take_Rout2_H;
			  end
			else
			  begin	/* Low */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;		/* R read  address */
				gdb_out_select_d = `gdb_out_take_Rout2_L;	
			  end		/* (Rn) */
		   end		/* R */
								
		  3'b011:  begin	/*---*/
					/* N */
					/*---*/
			gdb_write_d = `true;		/* put on the GDB */
									
			if (pdb2[10])			/* choose the register */
			  begin	/* High */
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
				gdb_out_select_d = `gdb_out_take_Nout2_H;
			  end
			else
			  begin	/* Low */
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;
				gdb_out_select_d = `gdb_out_take_Nout2_L;
			  end
		      end		/* N */
								
		  3'b100:  begin	/*---*/
					/* M */
					/*---*/
					gdb_write_d = `true;		/* put on the GDB */
									
			if (pdb2[10])		/* choose the register */
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				gdb_out_select_d = `gdb_out_take_Mout2_H;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				gdb_out_select_d = `gdb_out_take_Mout2_L;
			  end
			end		/* M */
								
		  default	:	;	/* No Action */
		endcase

		end		/* REP Class IV */


	else if ( {pdb2[23:14], pdb2[7], pdb2[5:0]} == 17'b00000110_01_0_100000 )
	  begin	/*--------------------------------------*/
		/*	REP Class I			*/
		/*					*/
		/* Repeat Next Instruction		*/
		/*--------------------------------------*/
					
		/*------------------------------------------------------------------------------*/
		/* 			Effective address calculation   	        	*/
		/*										*/
		/* Read from X or Y memory. The read datum is the number of repetitions.	*/
		/*------------------------------------------------------------------------------*/

		case (pdb2[13:11])
		  	3'b000	:  
			  begin	/* (Rn)-Nn */
				if (pdb2[6])	/* memory access */							yread_t = `true;
				else
					xread_t = `true;

				if (pdb2[10])
				  begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;	/* M read  address */
					Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;	/* N read  address */
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;	/* R read  address */
					Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;	/* R write address */
					Rwrite1_H_2_t = `true;		/* enable write */

					if (pdb2[6])			/* memory address */
						yab_select = `yab_take_Rout1_H;
					else
						xab_select = `xab_take_Rout1_H;

					addr_alu_H_N_or_1 = `take_Nn;		/* choose Nn	*/
					addr_alu_H_add_sub = `sub;		/* subtract	*/
					Rin1_H_select = `Rin1_H_take_addr_alu_H_out;			/* select output from addr_alu */
				  end
				else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;			/* M read  address */
					Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;			/* N read  address */
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;			/* R read  address */

					Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;			/* R write address */
					Rwrite1_L_2_t = `true;		/* enable write */
					if (pdb2[6])		/* memory address */
						yab_select = `yab_take_Rout1_L;
					else
						xab_select = `xab_take_Rout1_L;

					addr_alu_L_N_or_1 = `take_Nn;		/* choose Nn	*/
					addr_alu_L_add_sub = `sub;		/* subtract	*/
					Rin1_L_select = `Rin1_L_take_addr_alu_L_out;	/* select output from addr_alu */
				  end
			  end		/* (Rn)-Nn */

			3'b001	:	
			  begin	/* (Rn)+Nn */
				if (pdb2[6])			/* memory access */
					yread_t = `true;
				else
					xread_t = `true;

				if (pdb2[10])
			   	  begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;			/* M read  address */
					Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;			/* N read  address */
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;			/* R read  address */

					Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;			/* R write address */
					Rwrite1_H_2_t = `true;	/* enable write */

					if (pdb2[6])		/* memory address */
						yab_select = `yab_take_Rout1_H;
					else
						xab_select = `xab_take_Rout1_H;

					addr_alu_H_N_or_1 = `take_Nn;		/* choose Nn	*/
					addr_alu_H_add_sub = `add;		/* add	*/
					Rin1_H_select = `Rin1_H_take_addr_alu_H_out;			/* select output from addr_alu */
				  end
				else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;			/* M read  address */
					Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;			/* N read  address */
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;			/* R read  address */

					Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;			/* R write address */
					Rwrite1_L_2_t = `true;		/* enable write */

					if (pdb2[6])		/* memory address */
						yab_select = `yab_take_Rout1_L;
					else
						xab_select = `xab_take_Rout1_L;

					addr_alu_L_N_or_1 = `take_Nn;	/* choose Nn	*/
					addr_alu_L_add_sub = `add;		/* add	*/
					Rin1_L_select = `Rin1_L_take_addr_alu_L_out;			/* select output from addr_alu */
				  end
				end		/* (Rn)+Nn */

			3'b010	:	
			  begin	/* (Rn)- */
				if (pdb2[6])		/* memory access */
					yread_t = `true;
				else
					xread_t = `true;

				if (pdb2[10])
				  begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;	/* M read  address */
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;	/* R read  address */

					Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;	/* R write address */
					Rwrite1_H_2_t = `true;	/* enable write */

					if (pdb2[6])	/* memory address */
						yab_select = `yab_take_Rout1_H;
					else
						xab_select = `xab_take_Rout1_H;

					addr_alu_H_N_or_1 = `take_1;	/* choose 1	*/
					addr_alu_H_add_sub = `sub;		/* subtract	*/
					Rin1_H_select = `Rin1_H_take_addr_alu_H_out;			/* select output from addr_alu */
				  end
				else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;			/* M read  address */
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;			/* R read  address */

					Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;			/* R write address */
					Rwrite1_L_2_t = `true;	/* enable write */

					if (pdb2[6])		/* memory address */
						yab_select = `yab_take_Rout1_L;
					else
						xab_select = `xab_take_Rout1_L;

					addr_alu_L_N_or_1 = `take_1;	/* choose 1	*/
					addr_alu_L_add_sub = `sub;	/* subtract	*/
					Rin1_L_select = `Rin1_L_take_addr_alu_L_out;			/* select output from addr_alu */
				  end
				end		/* (Rn)- */

			3'b011	:	
			  begin	/* (Rn)+ */
				if (pdb2[6])			/* memory access */
					yread_t = `true;
				else
					xread_t = `true;

				if (pdb2[10])
				  begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;			/* M read  address */
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;			/* R read  address */

					Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;			/* R write address */
					Rwrite1_H_2_t = `true;		/* enable write */

					if (pdb2[6])		/* memory address */
						yab_select = `yab_take_Rout1_H;
					else
						xab_select = `xab_take_Rout1_H;

					addr_alu_H_N_or_1 = `take_1;	/* choose 1	*/
					addr_alu_H_add_sub = `add;	/* add	*/
					Rin1_H_select = `Rin1_H_take_addr_alu_H_out;			/* select output from addr_alu */
				  end
				else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;			/* M read  address */
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;			/* R read  address */

					Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;			/* R write address */
					Rwrite1_L_2_t = `true;	/* enable write */

					if (pdb2[6])		/* memory address */
						yab_select = `yab_take_Rout1_L;
					else
						xab_select = `xab_take_Rout1_L;

					addr_alu_L_N_or_1 = `take_1;		/* choose 1	*/
					addr_alu_L_add_sub = `add;	/* add	*/
					Rin1_L_select = `Rin1_L_take_addr_alu_L_out;			/* select output from addr_alu */
				  end
				end		/* (Rn)+ */

			3'b100	:	
			  begin	/* (Rn) */
				if (pdb2[6])			/* memory access */
					yread_t = `true;
				else
					xread_t = `true;

				if (pdb2[10])
				  begin	/* High */
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;			/* R read  address */

					if (pdb2[6])	/* memory address */
						yab_select = `yab_take_Rout1_H;
					else
						xab_select = `xab_take_Rout1_H;
				  end
				else
				  begin	/* Low */
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;			/* R read  address */

					if (pdb2[6])		/* memory address */
						yab_select = `yab_take_Rout1_L;
					else
						xab_select = `xab_take_Rout1_L;
				  end
				end		/* (Rn) */

			  3'b101	:
			    begin	/* (Rn+Nn) */
				if (pdb2[6])		/* memory access */
					yread_t = `true;
				else
					xread_t = `true;

				if (pdb2[10])
				  begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;			/* M read  address */
					Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;			/* N read  address */
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;			/* R read  address */

					if (pdb2[6])		/* memory address */
						yab_select = `yab_take_addr_alu_H_out;
					else
						xab_select = `xab_take_addr_alu_H_out;

						addr_alu_H_N_or_1 = `take_Nn;		/* choose Nn	*/
						addr_alu_H_add_sub = `add;		/* add	*/
				  end
				else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;			/* M read  address */
					Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;			/* N read  address */
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;			/* R read  address */

					if (pdb2[6])	/* memory address */
						yab_select = `yab_take_addr_alu_L_out;
					else
						xab_select = `xab_take_addr_alu_L_out;

					addr_alu_L_N_or_1 = `take_Nn;		/* choose Nn	*/
					addr_alu_L_add_sub = `add;		/* add	*/
				  end
				end		/* (Rn+Nn) */

			3'b110	:	;		/* No action */

			3'b111	:	
			  begin	/* -(Rn) */
				if (pdb2[6])		/* memory access */
					yread_t = `true;
				else
					xread_t = `true;

				if (pdb2[10])
				  begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;			/* M read  address */
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;			/* R read  address */

					Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;			/* R write address */
					Rwrite1_H_2_t = `true;	/* enable write */

					if (pdb2[6])	/* memory address */
						yab_select = `yab_take_addr_alu_H_out;
					else
						xab_select = `xab_take_addr_alu_H_out;

					addr_alu_H_N_or_1 = `take_1;		/* choose 1 */
					addr_alu_H_add_sub = `sub;		/* subtract */
					Rin1_H_select = `Rin1_H_take_addr_alu_H_out;			/* select output from addr_alu */
				  end
				else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;			/* M read  address */
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;			/* R read  address */

					Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;			/* R write address */
					Rwrite1_L_2_t = `true;		/* enable write */

					if (pdb2[6])	/* memory address */
						yab_select = `yab_take_addr_alu_L_out;
					else
						xab_select = `xab_take_addr_alu_L_out;

					addr_alu_L_N_or_1 = `take_1;		/* choose 1	*/
					addr_alu_L_add_sub = `sub;		/* subtract	*/
					Rin1_L_select = `Rin1_L_take_addr_alu_L_out;			/* select output from addr_alu */
			   	  end
				end		/* -(Rn) */

			endcase	/* pdb2[13:11] */
				
		  end		/* REP Class I */




	else if ( {pdb2[23:14], pdb2[7], pdb2[5:0]} == 17'b00000110_00_0_100000 )
	  begin	/*--------------------------------------*/
		/*	REP Class II			*/
		/*					*/
		/* Repeat Next Instruction		*/
		/*--------------------------------------*/
								
		/* Short absolute_d address taken from the instruction word */

		/* memory address */
		if (pdb2[6])
		  begin
			yab_select = `yab_take_pdb2_13_8;
			yread_t = `true;
		  end
		else
		  begin
			xab_select = `xab_take_pdb2_13_8;
			xread_t = `true;
		  end

	  end		/* REP Class II */


	else if ( {pdb2[23:16], pdb2[7:4]} == 12'b00000110_1010 )
	  begin	/*--------------------------------------*/
		/*		REP Class III		*/
		/*					*/
		/* No Action				*/
		/*--------------------------------------*/
					
	  end		/* REP Class III */



	else if ( {pdb2[23:16], pdb2[7:4]} == 12'b0000_0110_1000 )
	  begin	/*--------------------------------------*/
		/*	DO Class III			*/
		/*					*/
		/* Start Hardware Loop			*/
		/*--------------------------------------*/
		/* An absolute_d address is ALWAYS supplied in the following program word. */
		/* This is the end-of-the-loop expression */

				
	  end		/* DO Class III */
	
	else if ( {pdb2[23:14], pdb2[7:0]} == 18'b0000_0110_11_0000_0000 )
	  begin		/*--------------------------------------*/
			/*		DO Class IV		*/
			/*					*/
			/* Start Hardware Loop			*/
			/*--------------------------------------*/

			/* currently supports all AGU registers, N, M , and R */
				
		if (pdb2[13:10] == 4'b0110 )    /* source is N0-N3   */	
		  begin
			Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;
			gdb_out_select_d = `gdb_out_take_Nout2_L;
			gdb_write_d = `true;
		  end
				
		else if (pdb2[13:10] == 4'b0111)  /* source is N4-N7  */
		  begin
			Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
			gdb_out_select_d = `gdb_out_take_Nout2_H;
			gdb_write_d = `true;
		  end
				
		else if (pdb2[13:10] == 4'b0100 ) /* source is R0-R3 */
		  begin
			Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;
			gdb_out_select_d = `gdb_out_take_Rout2_L;
			gdb_write_d = `true;
		  end

		else if (pdb2[13:10] == 4'b0101 ) /* source is R4-R7 */
		  begin
			Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;
			gdb_out_select_d = `gdb_out_take_Rout2_H;
			gdb_write_d = `true;
		  end

		else if (pdb2[13:10] == 4'b1000 ) /* source is M0-M3 */
		  begin
			Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
			gdb_out_select_d = `gdb_out_take_Mout2_L;
			gdb_write_d = `true;
		  end

		else if (pdb2[13:10] == 4'b1001 ) /* source is M4-M7 */
		  begin
			Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
			gdb_out_select_d = `gdb_out_take_Mout2_H;
			gdb_write_d = `true;
		  end

		/* An absolute_d address is ALWAYS supplied in the following program word. */
		/* This is the end-of-the-loop expression */
	
			
				
		end		/* DO Class IV */


	else if ( {pdb2[23:14], pdb2[7:6]} == 12'b00001010_11_00 )
	  begin	/*--------------------------------------*/
		/*		JCLR / JSET Class III	*/
		/*					*/
		/*Jump if Bit Clear/Set - Class III	*/
		/*--------------------------------------*/

		
	  end		/* JCLR/JSET Class III */



	else if ( {pdb2[23:14], pdb2[7]} == 11'b00001010_01_1 )

	  begin	/*--------------------------------------*/
		/*		JCLR / JSET Class I	*/
		/*					*/
		/*	Jump if Bit Clear/Set - Class I	*/
		/*--------------------------------------*/


	  end		/* JCLR/JSET Class I */


	else if ( {pdb2[23:14], pdb2[7]} == 11'b00001010_00_1 )
	  begin	/*--------------------------------------*/
		/*	JCLR / JSET Class II		*/
		/*					*/
		/* Jump if Bit Clear/Set - Class II	*/
		/*--------------------------------------*/

	  end		/* JCLR/JSET Class II */


	else if ( ({pdb2[23:14], pdb2[7]} == 11'b00001010_01_0) ||
		({pdb2[23:14], pdb2[7], pdb2[5]} == 12'b00001011_01_0_1) )

	  begin	/*--------------------------------------*/
		/*BCLR / BSET / BTST Class I		*/
		/*					*/
		/*Bit Test and Clear/Set - Class I	*/
		/*--------------------------------------*/

	  end		/* BCLR/BSET/BTST Class I */




	else if ( ({pdb2[23:14], pdb2[7]} == 11'b00001010_00_0) ||
		({pdb2[23:14], pdb2[7], pdb2[5]} == 12'b00001011_00_0_1) )

	  begin	/*--------------------------------------*/
		/* BCLR / BSET / BTST Class II		*/
		/*					*/
		/* Bit Test and Clear/Set - Class II	*/
		/*--------------------------------------*/

	  end		/* BCLR/BSET/BTST Class II */


	else if ( ({pdb2[23:14], pdb2[7:6]} == 12'b00001010_11_01) ||
		({pdb2[23:14], pdb2[7:5]} == 13'b00001011_11_011) )
				  
	  begin	/*--------------------------------------*/
		/*	BCLR / BSET / BTST Class III	*/
		/*					*/
		/* Bit Test and Clear/Set - Class III	*/
		/*--------------------------------------*/

	end		/* BCLR/BSET/BTST Class III */



	else if ( pdb2[23:13] == 11'b00100000_010 )
	  begin	/*--------------------------------------*/
		/*	U:			 	*/
		/*			    	   	*/
		/*    Address Register Update  		*/
		/*--------------------------------------*/

		/* choose one of the effective addresses */

	    case (pdb2[12:11])
	      2'b00	:	/* (Rn)-Nn */
		if (pdb2[10])
		  begin		/* High R */
			Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;	/* M read address */
			Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;	/* N read address */
			Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;
			Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
			Rwrite1_H_2_t = `true;	/* enable write */
			addr_alu_H_N_or_1 = `take_Nn;	/* choose Nn	*/
			addr_alu_H_add_sub = `sub;	/* subtract	*/
			Rin1_H_select = `Rin1_H_take_addr_alu_H_out;			/* select output from addr_alu */
		  end
		else
		  begin		/* Low  R */
			Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;	/* M read address */
			Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;	/* N read address */
			Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;
			Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
			Rwrite1_L_2_t = `true;	/* enable write */
			addr_alu_L_N_or_1 = `take_Nn;	/* choose Nn	*/
			addr_alu_L_add_sub = `sub;	/* subtract	*/
			Rin1_L_select = `Rin1_L_take_addr_alu_L_out;			/* select output from addr_alu */
		  end

	  2'b01	:	/* (Rn)+Nn */
		if (pdb2[10])
		  begin		/* High R */
			Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;	/* M read address */
			Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;	/* N read address */
			Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

			Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
			Rwrite1_H_2_t = `true;		/* enable write */

			addr_alu_H_N_or_1 = `take_Nn;	/* choose Nn	*/
			addr_alu_H_add_sub = `add;		/* add	*/
			Rin1_H_select = `Rin1_H_take_addr_alu_H_out;			/* select output from addr_alu */
		  end
		else
		  begin		/* Low  R */
			Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;	/* M read address */
			Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;	/* N read address */
			Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;
			Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
			Rwrite1_L_2_t = `true;	/* enable write */
			addr_alu_L_N_or_1 = `take_Nn;	/* choose Nn	*/
			addr_alu_L_add_sub = `add;	/* add	*/
			Rin1_L_select = `Rin1_L_take_addr_alu_L_out;	/* select output from addr_alu */
		  end

	  2'b10	:	/* (Rn)- */
		if (pdb2[10])
		  begin		/* High R */
			Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;	/* M read address */
			Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;
			Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
			Rwrite1_H_2_t = `true;	/* enable write */
			addr_alu_H_N_or_1 = `take_1;	/* choose 1	*/
			addr_alu_H_add_sub = `sub;	/* subtract	*/
			Rin1_H_select = `Rin1_H_take_addr_alu_H_out;			/* select output from addr_alu */
		  end
		else
		  begin		/* Low  R */
			Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;	/* M read address */
			Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;
			Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
			Rwrite1_L_2_t = `true;	  /* enable write */
			addr_alu_L_N_or_1 = `take_1;	/* choose 1	*/
			addr_alu_L_add_sub = `sub;	/* subtract	*/
			Rin1_L_select = `Rin1_L_take_addr_alu_L_out;			/* select output from addr_alu */
		  end

	  2'b11	:	/* (Rn)+ */
		if (pdb2[10])
		  begin		/* High R */
			Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;	/* M read address */
			Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;
			Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
			Rwrite1_H_2_t = `true;	/* enable write */
			addr_alu_H_N_or_1 = `take_1;	/* choose 1	*/
			addr_alu_H_add_sub = `add;	/* add	*/
			Rin1_H_select = `Rin1_H_take_addr_alu_H_out;			/* select output from addr_alu */
		   end
		else
		  begin		/* Low  R */
			Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;	/* M read address */
			Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;
			Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
			Rwrite1_L_2_t = `true;		/* enable write */
			addr_alu_L_N_or_1 = `take_1;	/* choose 1	*/
			addr_alu_L_add_sub = `add;	/* add	*/
			Rin1_L_select = `Rin1_L_take_addr_alu_L_out;			/* select output from addr_alu */
		   end
									
	endcase		/* pdb2[12:11] */

	end		/* U: */

	else if ( {pdb2[23:16], pdb2[7:5]} == 11'b00000100_101 )
	  begin	/*--------------------------------------*/
		/*	MOVEC Class III       	        */
		/*   Move Control Register	 	*/
		/*--------------------------------------*/
		/*---------------------------------------------------------------*/
		/* ONLY Partially implemented to transfer data between M registers */
		/*------------------------------------------------------------*/
		
		if (pdb2[15])  /* if W =1 */
		  begin
		 	if (pdb2[13:10] == 4'b1000) /*  read from Low M registers */
			  begin
				Mraddr2_L_select_d = `take_pdb3_9_8;				  	
				gdb_write_d = `true;
				gdb_out_select_d = `gdb_out_take_Mout2_L;
				if (pdb2[4:2] == 3'b000) /* write to low bank */
				  begin
					Mwaddr2_L_select_d = `Mwaddr2_L_take_pdb2_1_0;
					Min2_L_select_d = `Min2_L_take_GDB;
					Mwrite2_L_3_d = 1'b1;
				  end
				else if (pdb2[4:2] == 3'b001) /* write to high bank */
				  begin
					Mwaddr2_H_select_d = `Mwaddr2_H_take_pdb2_1_0;
					Min2_H_select_d = `Min2_H_take_GDB;
					Mwrite2_H_3_d = 1'b1;
				  end
					  
			  end
			else if (pdb2[13:10] == 4'b1001) /* read from High M registers */
			  begin
				Mraddr2_H_select_d = `take_pdb3_9_8;
				gdb_write_d = `true;
				gdb_out_select_d = `gdb_out_take_Mout2_H;
				if (pdb2[4:2] == 3'b000) /* write to low bank */
				  begin
					Mwaddr2_L_select_d = `Mwaddr2_L_take_pdb2_1_0;
					Min2_L_select_d = `Min2_L_take_GDB;
					Mwrite2_L_3_d = 1'b1;
				  end
				else if (pdb2[4:2] == 3'b001) /* write to high bank */
				  begin
					Mwaddr2_H_select_d = `Mwaddr2_H_take_pdb2_1_0;
					Min2_H_select_d = `Min2_H_take_GDB;
					Mwrite2_H_3_d = 1'b1;
				  end
			  end
				  
		  end
		else        /* if W = 0 */
		  begin
			if (pdb2[4:2] == 3'b000) /*  Read from low bank */
			  begin
				Mraddr2_L_select_d = `take_pdb3_1_0;				   
				gdb_write_d = `true;
				gdb_out_select_d = `gdb_out_take_Mout2_L;
				if (pdb2[13:10] == 4'b1000)	/* write to low  bank */
				  begin
					Mwaddr2_L_select_d = `Mwaddr2_L_take_pdb2_9_8;
					Min2_L_select_d = `Min2_L_take_GDB;
					Mwrite2_L_3_d = 1'b1;
				  end
				else if(pdb2[13:10] == 4'b1001) /* write to high bank */
				  begin
					Mwaddr2_H_select_d = `Mwaddr2_H_take_pdb2_9_8;
					Min2_H_select_d = `Min2_H_take_GDB;
					Mwrite2_H_3_d = 1'b1;
				  end
		
			  end
			else if (pdb2[4:2] == 3'b001) /* Read from high registers */
			  begin
				Mraddr2_H_select_d = `take_pdb3_1_0;
				gdb_write_d = `true;
				gdb_out_select_d = `gdb_out_take_Mout2_H;
				Mwaddr2_H_select_d = `Mwaddr2_H_take_pdb2_9_8;
				Min2_H_select_d = 2'b01;
				Mwrite2_H_3_d = 1'b1;
				if (pdb2[13:10] == 4'b1000)	/* write to low  bank */
				  begin
					Mwaddr2_L_select_d = `Mwaddr2_L_take_pdb2_9_8;
					Min2_L_select_d = `Min2_L_take_GDB;
					Mwrite2_L_3_d = 1'b1;
				  end
				else if(pdb2[13:10] == 4'b1001) /* write to high bank */
				  begin
					Mwaddr2_H_select_d = `Mwaddr2_H_take_pdb2_9_8;
					Min2_H_select_d = `Min2_H_take_GDB;
					Mwrite2_H_3_d = 1'b1;
				  end
			  end
			
		  end  /* pdb2[15] */

			  
	  end /* MOVEC III */

	else if ( {pdb2[23:16], pdb2[7:5]} == 11'b00000101_101 )
	  begin	/*--------------------------------------*/
		/*	MOVEC Class IV	       	        */
		/*   Move Control Register	 	*/
		/*--------------------------------------*/
		/*---------------------------------------------------------------*/
		/* ONLY Partially implemented to load immediate data into the M registers */
		/*------------------------------------------------------------*/
				
		/* always loads immediate short data from the inst word */
		if (pdb2[4:3] == 2'b00)
		/* Affects the M registers */
		  begin
			/* the address of the M register */
			if (pdb2[2])
				Mwaddr2_H_select_d = `Mwaddr2_H_take_pdb2_1_0;		/* to the High M rf */
			else
				Mwaddr2_L_select_d = `Mwaddr2_L_take_pdb2_1_0;		/* to the Low  M rf */
		/* the immediate data is treated as an unsigned integer */

			if (pdb2[2])
				Min2_H_select_d = `Min2_H_take_imm_data;				/* to the High rf */
			else
				Min2_L_select_d = `Min2_L_take_imm_data;				/* to the Low rf */
				/* enable write to one of M's register files */
				Mwrite2_H_3_d = pdb2[2];
				Mwrite2_L_3_d = (~pdb2[2]);
		  end
	  end		/* MOVEC Class IV */


	else if ( {pdb2[23:16], pdb2[7], pdb2[5]} == 10'b00000101_0_1 )
	  begin	/*----------------------------------*/
		/*	MOVEC Class I and II	    */
		/*				    */
		/* Move Control Register	    g */
		/*----------------------------------*/

		/*--------------------------------------------------------------*/
		/* 			Effective address calculation 		*/
		/*								*/
		/* Read from X or Y memory when the destination is a register.	*/
		/* Write to X or Y memory when the source is a register.		*/
		/*--------------------------------------------------------------*/
		if ( pdb2[15] && (pdb2[13:8] != 6'b110100) )	
			/* write destination register, NOT immediate data */
		  begin
			if (pdb2[4:3] == 2'b00)  /* Affects the M registers */
			  begin    /* the address of the M register */

			 	if (pdb2[2])
					Mwaddr2_H_select_d = `Mwaddr2_H_take_pdb2_1_0;		/* to the High M rf */
				else
					Mwaddr2_L_select_d = `Mwaddr2_L_take_pdb2_1_0;		/* to the Low  M rf */

				/* fetch data */
				if (pdb2[2])
					Min2_H_select_d = `Min2_H_take_GDB;		/* to the High rf */
				else
					Min2_L_select_d = `Min2_L_take_GDB;		/* to the Low rf */

				/* enable write to one of M's register files */

				Mwrite2_H_3_d = pdb2[2];
				Mwrite2_L_3_d = (~pdb2[2]);
			   end
		  end

		if (pdb2[14])
		  begin	/*------------------*/
			/* Class I	    */
			/*------------------*/

			case (pdb2[13:11])
			  3'b000	:	
			    begin	/* (Rn)-Nn */
			        mem_control_signals;
				if (pdb2[10])
				  begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;	/* M read  address */
					Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;	/* N read  address */
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;	/* R read  address */
					Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;	/* R write address */
					Rwrite1_H_2_t = `true;	/* enable write */
					
					if (pdb2[6])		/* memory address */
						yab_select = `yab_take_Rout1_H;
					else
						xab_select = `xab_take_Rout1_H;
					
					addr_alu_H_N_or_1 = `take_Nn;		/* choose Nn	*/
					addr_alu_H_add_sub = `sub;	/* subtract	*/
					Rin1_H_select = `Rin1_H_take_addr_alu_H_out;	/* select output from addr_alu */
			    	  end
				else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;	/* M read  address */
					Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;	/* N read  address */
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;	/* R read  address */
					Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;	/* R write address */
					Rwrite1_L_2_t = `true;	/* enable write */

					if (pdb2[6])  /* memory address */
						yab_select = `yab_take_Rout1_L;
					else
						xab_select = `xab_take_Rout1_L;

					addr_alu_L_N_or_1 = `take_Nn;	/* choose Nn	*/
					addr_alu_L_add_sub = `sub;	/* subtract	*/
					Rin1_L_select = `Rin1_L_take_addr_alu_L_out;	/* select output from addr_alu */
				  end
			  end		/* (Rn)-Nn */

			3'b001	:	
			  begin	/* (Rn)+Nn */
				mem_control_signals;

				if (pdb2[10])
				  begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;	/* M read  address */
					Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;	/* N read  address */
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;	/* R read  address */
					Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;	/* R write address */
					Rwrite1_H_2_t = `true;	/* enable write */

					if (pdb2[6])	/* memory address */
						yab_select = `yab_take_Rout1_H;
					else
						xab_select = `xab_take_Rout1_H;

					addr_alu_H_N_or_1 = `take_Nn;	/* choose Nn	*/
					addr_alu_H_add_sub = `add;	/* add	*/
					Rin1_H_select = `Rin1_H_take_addr_alu_H_out;	/* select output from addr_alu */
				  end
				else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;	/* M read  address */
					Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;	/* N read  address */
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;	/* R read  address */
					Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;	/* R write address */
					Rwrite1_L_2_t = `true;	/* enable write */

					if (pdb2[6])	/* memory address */
						yab_select = `yab_take_Rout1_L;
					else
						xab_select = `xab_take_Rout1_L;

					addr_alu_L_N_or_1 = `take_Nn;	/* choose Nn	*/
					addr_alu_L_add_sub = `add;	/* add	*/
					Rin1_L_select = `Rin1_L_take_addr_alu_L_out;	/* select output from addr_alu */
				  end
			  end		/* (Rn)+Nn */

		  	3'b010	:	
		    	  begin	/* (Rn)- */
				mem_control_signals;
				if (pdb2[10])
				  begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;/* M read  address */
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;/* R read  address */
					Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;/* R write address */
					Rwrite1_H_2_t = `true;		/* enable write */

					if (pdb2[6])	/* memory address */
						yab_select = `yab_take_Rout1_H;
					else
						xab_select = `xab_take_Rout1_H;

					addr_alu_H_N_or_1 = `take_1;	/* choose 1	*/
					addr_alu_H_add_sub = `sub;	/* subtract	*/
					Rin1_H_select = `Rin1_H_take_addr_alu_H_out;	/* select output from addr_alu */
				  end
				else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;/* M read  address */
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;/* R read  address */
					Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;/* R write address */
					Rwrite1_L_2_t = `true;	/* enable write */

					if (pdb2[6])	/* memory address */
						yab_select = `yab_take_Rout1_L;
					else
						xab_select = `xab_take_Rout1_L;

					addr_alu_L_N_or_1 = `take_1;/* choose 1	*/
					addr_alu_L_add_sub = `sub;	/* subtract	*/
					Rin1_L_select = `Rin1_L_take_addr_alu_L_out;	/* select output from addr_alu */
				  end
			  end		/* (Rn)- */

			3'b011	:	
			  begin	/* (Rn)+ */
				mem_control_signals;

				if (pdb2[10])
			          begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;	/* M read  address */
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;	/* R read  address */
					Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8; /* R write address */
					Rwrite1_H_2_t = `true;		/* enable write */

					if (pdb2[6])	/* memory address */
						yab_select = `yab_take_Rout1_H;
					else
						xab_select = `xab_take_Rout1_H;

					addr_alu_H_N_or_1 = `take_1;	/* choose 1	*/
					addr_alu_H_add_sub = `add;	/* add	*/
					Rin1_H_select = `Rin1_H_take_addr_alu_H_out;	/* select output from addr_alu */
				  end
				else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;	/* M read  address */
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;	/* R read  address */
					Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;	/* R write address */
					Rwrite1_L_2_t = `true;		/* enable write */

					if (pdb2[6])		/* memory address */
						yab_select = `yab_take_Rout1_L;
					else
						xab_select = `xab_take_Rout1_L;

					addr_alu_L_N_or_1 = `take_1;	/* choose 1	*/
					addr_alu_L_add_sub = `add;	/* add	*/
					Rin1_L_select = `Rin1_L_take_addr_alu_L_out;	/* select output from addr_alu */
				  end
			  end		/* (Rn)+ */

			3'b100	:	
			   begin	/* (Rn) */
				mem_control_signals;

				if (pdb2[10])
				  begin	/* High */
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;	/* R read  address */

					if (pdb2[6])	/* memory address */
						yab_select = `yab_take_Rout1_H;
					else
						xab_select = `xab_take_Rout1_H;
				  end
				else
					begin	/* Low */
						Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;	/* R read  address */

						if (pdb2[6])	/* memory address */
							yab_select = `yab_take_Rout1_L;
						else
							xab_select = `xab_take_Rout1_L;
					  end
			  end		/* (Rn) */

			3'b101	:	
			  begin	/* (Rn+Nn) */
				mem_control_signals;

				if (pdb2[10])
				  begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;	/* M read  address */
					Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;	/* N read  address */
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;	/* R read  address */

					if (pdb2[6])		/* memory address */
						yab_select = `yab_take_addr_alu_H_out;
					else
						xab_select = `xab_take_addr_alu_H_out;

					addr_alu_H_N_or_1 = `take_Nn;	/* choose Nn	*/
					addr_alu_H_add_sub = `add;		/* add	*/
				  end
				else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;	/* M read  address */
					Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;	/* N read  address */
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;	/* R read  address */

					if (pdb2[6])		/* memory address */
						yab_select = `yab_take_addr_alu_L_out;
					else
						xab_select = `xab_take_addr_alu_L_out;

					addr_alu_L_N_or_1 = `take_Nn;	/* choose Nn	*/
					addr_alu_L_add_sub = `add;	/* add	*/
				  end
			  end		/* (Rn+Nn) */

			3'b110	:	/* Absolute_D address & Immediate_D data */
			  begin
				if (pdb2[10:8] == 3'b100)	/* immediate_d data */
				  begin
					if ( pdb2[15] )		/* write destination */
					  begin
						
						if ( pdb2[4:3] == 2'b00 )
						/*-------------------------*/
						/* Destination: M register */
						/*-------------------------*/

							if (pdb2[2])
							  begin	/* High */
								Mwaddr2_H_select_d = `take_pdb3_17_16;	/* M write address */
								Mwrite2_H_3_d = `true;		/* enable write */	
								Min2_H_select_d = `Min2_H_take_GDB; /* select input from GDB */
							  end
							else
							  begin	/* Low */
								Mwaddr2_L_select_d = `take_pdb3_17_16;	/* M write address */								   
								Mwrite2_L_3_d = `true;		/* enable write */
								Min2_L_select_d = `Min2_L_take_GDB; /* select input from GDB */
							  end		/* M */
					  end		/* write */
				  end		/* immediate_d data */
												
												
				else if (pdb2[10:8] == 3'b000)	/* absolute_d address */
				  begin
					
					mem_control_signals;
				  end
				end/* Absolute_D address & Immediate_D data */
										
			3'b111	:	begin	/* -(Rn) */
			  	mem_control_signals;

				if (pdb2[10])
				  begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;		
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;
					Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
					Rwrite1_H_2_t = `true;		/* enable write */

					if (pdb2[6])	/* memory address */
						yab_select = `yab_take_addr_alu_H_out;
					else
						xab_select = `xab_take_addr_alu_H_out;

					addr_alu_H_N_or_1 = `take_1;
					addr_alu_H_add_sub = `sub;	
					Rin1_H_select = `Rin1_H_take_addr_alu_H_out;		
				  end
				else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;
					Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
					Rwrite1_L_2_t = `true;	

				if (pdb2[6])	/* memory address */
					yab_select = `yab_take_addr_alu_L_out;
				else
					xab_select = `xab_take_addr_alu_L_out;

					addr_alu_L_N_or_1 = `take_1;	
					addr_alu_L_add_sub = `sub;	
					Rin1_L_select = `Rin1_L_take_addr_alu_L_out;	
				  end
			  end		/* -(Rn) */

			endcase	/* pdb2[13:11] */

		  end		/* Class I */


		else		/* if pdb2[14]==0 */

		  begin	/*------------------------------*/
			/* 	Class II		*/
			/*------------------------------*/

			/* memory access */
			mem_control_signals;

			/* memory address */
			if (pdb2[6])
				yab_select = `yab_take_pdb2_13_8;
			else
				xab_select = `xab_take_pdb2_13_8;

		  end	/* Class II */
					
				
		/*---------------------------------------------*/
		/* if a source register is written into memory */
		/*---------------------------------------------*/

		if (~pdb2[15])
		  begin
			if (pdb2[4:3] == 2'b00)
			  begin	/* M registers ^^^^ using port 3 ^^^^ */
				if (pdb2[2])
				  begin		/* High */
					// Mraddr2_H_2 gets pdb2[1:0]
					Mraddr2_H_select_d = `take_pdb3_1_0;
					gdb_out_select_d = `gdb_out_take_Mout2_H;
					gdb_write_d = `true;
				  end
				else
				  begin		/* Low */
					// Mraddr2_L_2 gets pdb2[1:0]
					Mraddr2_L_select_d = `take_pdb3_1_0;
					gdb_out_select_d = `gdb_out_take_Mout2_L;
					gdb_write_d = `true;
				  end
			  end		/* M */
		  end		/* read source */
					
	   end		/* MOVEC Class I and II */


	else if ( {pdb2[23:17], pdb2[14]} == 8'b0000100_0 )
	  begin	/*--------------------------------------------------------------*/
		/*		X:R Class II  or  R:Y  Class II			*/
		/*								*/
		/* X Memory and Register Data Move		Class II	*/
		/* Register and Y Memory Data Move		Class II	*/
		/*								*/
		/* Generates the effective address for a data write.		*/
		/* Source and Destination registers are of the Data ALU ONLY.	*/
		/* Condition codes ARE affected.				*/
		/*--------------------------------------------------------------*/
		
		case (pdb2[13:11])
		  3'b000	:	/* (Rn)-Nn */
		    	if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;
				
				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;	

				if (pdb2[15])	/* memory access */
				  begin
					yab_select = `yab_take_Rout1_H;
					ywrite_t = `true;
				  end
				else
				  begin
					xab_select = `xab_take_Rout1_H;
					xwrite_t = `true;
				  end

				addr_alu_H_N_or_1 = `take_Nn;
				addr_alu_H_add_sub = `sub;
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
		 	else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;

				if (pdb2[15])	/* memory access */
				  begin
					yab_select = `yab_take_Rout1_L;
					ywrite_t = `true;
				  end
				else
				  begin
					xab_select = `xab_take_Rout1_L;
					xwrite_t = `true;
				  end

				addr_alu_L_N_or_1 = `take_Nn;	
				addr_alu_L_add_sub = `sub;	
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end		/* (Rn)-Nn */

		  3'b001	:	/* (Rn)+Nn */
			if (pdb2[10])
			   begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;

				if (pdb2[15])		/* memory access */
				   begin
					yab_select = `yab_take_Rout1_H;	
					ywrite_t = `true;
				  end
				else
				  begin
					xab_select = `xab_take_Rout1_H;
					xwrite_t = `true;
				  end

				addr_alu_H_N_or_1 = `take_Nn;	/* choose Nn	*/
				addr_alu_H_add_sub = `add;	/* add	*/
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;			

				if (pdb2[15])		/* memory access */
				  begin
					yab_select = `yab_take_Rout1_L;
					ywrite_t = `true;
				  end
				else
				  begin
					xab_select = `xab_take_Rout1_L;
					xwrite_t = `true;
				  end

				addr_alu_L_N_or_1 = `take_Nn;
				addr_alu_L_add_sub = `add;	
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end		/* (Rn)+Nn */

		  3'b010	:	/* (Rn)- */
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;

				if (pdb2[15])
				  begin
					yab_select = `yab_take_Rout1_H;
					ywrite_t = `true;
				  end
				else
				  begin
					xab_select = `xab_take_Rout1_H;
					xwrite_t = `true;
				  end

				addr_alu_H_N_or_1 = `take_1;
				addr_alu_H_add_sub = `sub;
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;

				if (pdb2[15])
				  begin
					yab_select = `yab_take_Rout1_L;
					ywrite_t = `true;
				  end
				else
				  begin
					xab_select = `xab_take_Rout1_L;
					xwrite_t = `true;
				  end

				addr_alu_L_N_or_1 = `take_1;	
				addr_alu_L_add_sub = `sub;	
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end		/* (Rn)- */

		  3'b011	:	/* (Rn)+ */
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;

				if (pdb2[15])
				  begin
					yab_select = `yab_take_Rout1_H;
					ywrite_t = `true;
				  end
				else
				  begin
					xab_select = `xab_take_Rout1_H;
					xwrite_t = `true;
				  end

				addr_alu_H_N_or_1 = `take_1;
				addr_alu_H_add_sub = `add;	
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;

				if (pdb2[15])
				  begin
					yab_select = `yab_take_Rout1_L;
					ywrite_t = `true;
				  end
				else
				  begin
					xab_select = `xab_take_Rout1_L;
					xwrite_t = `true;
				  end

				addr_alu_L_N_or_1 = `take_1;
				addr_alu_L_add_sub = `add;
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			   end		/* (Rn)+ */

		  3'b100	:	/* (Rn) */
			if (pdb2[10])
			  begin	/* High */
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				if (pdb2[15])	
				  begin
					yab_select = `yab_take_Rout1_H;
					ywrite_t = `true;
				  end
				else
				  begin
					xab_select = `xab_take_Rout1_H;
					xwrite_t = `true;
				  end
			  end
			else
			  begin	/* Low */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				if (pdb2[15])
				  begin
					yab_select = `yab_take_Rout1_L;
					ywrite_t = `true;
				  end
				else
				  begin
					xab_select = `xab_take_Rout1_L;
					xwrite_t = `true;
				  end
			  
		  	  end	/* (Rn) */

		3'b101	:	/* (Rn+Nn) */
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				if (pdb2[15])
				  begin
					yab_select = `yab_take_addr_alu_H_out;
					ywrite_t = `true;
				  end
				else
				  begin
					xab_select = `xab_take_addr_alu_H_out;
					xwrite_t = `true;
				  end

				addr_alu_H_N_or_1 = `take_Nn;
				addr_alu_H_add_sub = `add;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				if (pdb2[15])
				  begin
					yab_select = `yab_take_addr_alu_L_out;
					ywrite_t = `true;
				  end
				else
				  begin
					xab_select = `xab_take_addr_alu_L_out;
					xwrite_t = `true;
				  end

				addr_alu_L_N_or_1 = `take_Nn;
				addr_alu_L_add_sub = `add;
			  end		/* (Rn+Nn) */

		  3'b110	:	begin	/* No Action */
					end

		  3'b111	:	/* -(Rn) */
			/* The source register is NEVER an AGU register	*/

			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;

				if (pdb2[15])
				  begin
					yab_select = `yab_take_addr_alu_H_out;
					ywrite_t = `true;
				  end
				else
				  begin
					xab_select = `xab_take_addr_alu_H_out;
					xwrite_t = `true;
				  end

				addr_alu_H_N_or_1 = `take_1;
				addr_alu_H_add_sub = `sub;
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;

				if (pdb2[15])
				  begin
					yab_select = `yab_take_addr_alu_L_out;
					ywrite_t = `true;
				  end
				else
				  begin
					xab_select = `xab_take_addr_alu_L_out;
					xwrite_t = `true;
				  end

				addr_alu_L_N_or_1 = `take_1;
				addr_alu_L_add_sub = `sub;
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end		/* -(Rn) */

		  endcase	/* pdb2[13:11] */

	  end		/* X:R Class II  or  R:Y Class II */



	else if ( pdb2[23:18] == 6'b001000 )
	  begin	/*--------------------------------------*/
		/*		R:			*/
		/*					*/
		/* Register to Register Data Move	*/
		/* Condition Codes ARE changed.		*/
		/*					*/
				
		/* Active when the source is either	*/
		/* an R or an N register.		*/
		/*----------------------------------*/
		

		/* Active when the destination is*/
		/* either an R or an N register.*/
		/*----------------------------------*/

		/* A register gets the gdb 16 LS bits */

		if ( pdb2[12:11] == 2'b11 )
			/*-------------------------*/
			/* Destination: N register */
			/*-------------------------*/

			if (pdb2[10])
			  begin		/* High */
				Nwaddr2_H_select_d = `Nwaddr2_H_take_pdb2_9_8;
				Nwrite2_H_3_d = `true;
				Nin2_H_select_d = `Nin2_H_take_GDB;
			  end
			else
			  begin		/* Low */
				Nwaddr2_L_select_d = `Nwaddr2_L_take_pdb2_9_8;
				Nwrite2_L_3_d = `true;
				Nin2_L_select_d = `Nin2_L_take_GDB;
			  end

		else if ( pdb2[12:11] == 2'b10 )
			/*-------------------------*/
			/* Destination: R register */
			/*-------------------------*/

			if (pdb2[10])	/* High */
			  begin
				Rwaddr2_H_select_d = `Rwaddr2_H_take_pdb2_9_8;
				Rwrite2_H_3_d = `true;
				Rin2_H_select_d = `Rin2_H_take_GDB;
			  end
			else			/* Low */
			  begin
				Rwaddr2_L_select_d = `Rwaddr2_L_take_pdb2_9_8;
				Rwrite2_L_3_d = `true;
				Rin2_L_select_d = `Rin2_L_take_GDB;
			  end

		/* 16 LS bits in gdb_out get the register contents and the upper 8 are zeroed */

		/* choose the source */
				
		if ( pdb2[17:16] == 2'b11 )
		  begin	/*------------*/
			/* N register */
			/*------------*/
						
			gdb_write_d = `true;

			if (pdb2[15])
			  begin	/* High */				
				Nraddr2_H_select_d = `take_pdb3_14_13;	
				gdb_out_select_d = `gdb_out_take_Nout2_H;
			  end
			else
			  begin	/* Low */
				Nraddr2_L_select_d = `take_pdb3_14_13;
				gdb_out_select_d = `gdb_out_take_Nout2_L;
			  end
		  end		/* N */
		
		else if ( pdb2[17:16] == 2'b10 )
		  begin	/*------------*/
			/* R register */
			/*------------*/
							
			gdb_write_d = `true;
						
			if (pdb2[15])
			  begin	/* High */
			    	Rraddr2_H_select_d = `take_pdb3_14_13;
				gdb_out_select_d = `gdb_out_take_Rout2_H;
			  end
			else
			  begin	/* Low */
				Rraddr2_L_select_d = `take_pdb3_14_13;
				gdb_out_select_d = `gdb_out_take_Rout2_L;
			   end
		  end		/* R */

	  end		/* R: */


	else if ( {pdb2[23:20], pdb2[18]} == 5'b0100_0 )
	  begin	/*------------------------------*/
		/* 		L:		*/
		/* 				*/
		/* Long Memory Data Move	*/
		/*-------------------------------*/
				
		if (pdb2[14])
		  begin	/*------------------------------*/
			/*	L: Class I		*/
			/*------------------------------*/
						
			/*----------------------------------*/
			/* calculate the effective address. */
			/*----------------------------------*/
						
			/* Both X and Y accesses are using the SAME effective address */
						
			case (pdb2[13:11])
			  3'b000	:
			    begin	/* (Rn)-Nn */
				if (pdb2[15])
				  begin		/* read from memory */
					xread_t = `true;
					yread_t = `true;
				  end
				else
				  begin		/* write to memory */
					xwrite_t = `true;
					ywrite_t = `true;
				  end

				if (pdb2[10])	/* effective address */
				  begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
					Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

					Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
					Rwrite1_H_2_t = `true;
					yab_select = `yab_take_Rout1_H;
					xab_select = `xab_take_Rout1_H;

					addr_alu_H_N_or_1 = `take_Nn;
					addr_alu_H_add_sub = `sub;
					Rin1_H_select = `Rin1_H_take_addr_alu_H_out;		
				  end
				else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;			/* M read  address */
					Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;			/* N read  address */
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;			/* R read  address */

					Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;			/* R write address */
					Rwrite1_L_2_t = `true;

					yab_select = `yab_take_Rout1_L;
					xab_select = `xab_take_Rout1_L;

					addr_alu_L_N_or_1 = `take_Nn;
					addr_alu_L_add_sub = `sub;
					Rin1_L_select = `Rin1_L_take_addr_alu_L_out;			/* select output from addr_alu */
														  end
													  end		/* (Rn)-Nn */

			3'b001	:	
			  begin	/* (Rn)+Nn */
														if (pdb2[15])
				  begin	/* read from memory */
					xread_t = `true;
					yread_t = `true;
				  end
				else
				  begin	/* write to memory */
					xwrite_t = `true;
					ywrite_t = `true;
				  end

				if (pdb2[10])	/* effective address */
				  begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;			/* M read  address */
					Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;			/* N read  address */
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;			/* R read  address */
					Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;			/* R write address */
					Rwrite1_H_2_t = `true;
															yab_select = `yab_take_Rout1_H;
					xab_select = `xab_take_Rout1_H;

					addr_alu_H_N_or_1 = `take_Nn;
					addr_alu_H_add_sub = `add;
					Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
				  end
														else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;			/* M read  address */
					Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;			/* N read  address */
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;			/* R read  address */

					Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;			/* R write address */
					Rwrite1_L_2_t = `true;

					yab_select = `yab_take_Rout1_L;
					xab_select = `xab_take_Rout1_L;

					addr_alu_L_N_or_1 = `take_Nn;
					addr_alu_L_add_sub = `add;
					Rin1_L_select = `Rin1_L_take_addr_alu_L_out;			/* select output from addr_alu */
				  end
			  end		/* (Rn)+Nn */

		  3'b010	:	
		    begin	/* (Rn)- */
			if (pdb2[15])	/* memory accesses */
			  begin	
				xread_t = `true;
				yread_t = `true;
			  end
			else
			  begin	
				xwrite_t = `true;
				ywrite_t = `true;
			  end

			if (pdb2[10])	/* effective address */
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;			/* M read  address */
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;			/* R read  address */

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;			/* R write address */
				Rwrite1_H_2_t = `true;

				yab_select = `yab_take_Rout1_H;
				xab_select = `xab_take_Rout1_H;

				addr_alu_H_N_or_1 = `take_1;
				addr_alu_H_add_sub = `sub;
														Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			   end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;			/* M read  address */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;			/* R read  address */

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;			/* R write address */
				Rwrite1_L_2_t = `true;

				yab_select = `yab_take_Rout1_L;
				xab_select = `xab_take_Rout1_L;

				addr_alu_L_N_or_1 = `take_1;
				addr_alu_L_add_sub = `sub;
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end
		    end		/* (Rn)- */

		3'b011	:	
		  begin	/* (Rn)+ */					
			if (pdb2[15])					
			  begin	
				xread_t = `true;
				yread_t = `true;
			  end
			else
			  begin	
				xwrite_t = `true;
				ywrite_t = `true;
			  end

			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;
				
				yab_select = `yab_take_Rout1_H;
				xab_select = `xab_take_Rout1_H;

				addr_alu_H_N_or_1 = `take_1;
				addr_alu_H_add_sub = `add;
				
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;	

				yab_select = `yab_take_Rout1_L;
				xab_select = `xab_take_Rout1_L;

				addr_alu_L_N_or_1 = `take_1;
				addr_alu_L_add_sub = `add;
				
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end
		    end		/* (Rn)+ */

		  3'b100	:	
		    begin	/* (Rn) */
			if (pdb2[15])
			  begin
				xread_t = `true;
				yread_t = `true;
			  end
			else
			  begin
				xwrite_t = `true;
				ywrite_t = `true;
			  end

			if (pdb2[10])
			  begin	/* High */
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				yab_select = `yab_take_Rout1_H;
				xab_select = `xab_take_Rout1_H;
			  end
			else
			  begin	/* Low */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				yab_select = `yab_take_Rout1_L;
				xab_select = `xab_take_Rout1_L;
			  end
		    end		/* (Rn) */

		  3'b101	:	begin	/* (Rn+Nn) */
			if (pdb2[15])
			  begin	
				xread_t = `true;
				yread_t = `true;
			  end
			else
			  begin	
				xwrite_t = `true;
				ywrite_t = `true;
			  end

			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				yab_select = `yab_take_addr_alu_H_out;
				xab_select = `xab_take_addr_alu_H_out;

				addr_alu_H_N_or_1 = `take_Nn;
				addr_alu_H_add_sub = `add;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				yab_select = `yab_take_addr_alu_L_out;
				xab_select = `xab_take_addr_alu_L_out;

				addr_alu_L_N_or_1 = `take_Nn;	
				addr_alu_L_add_sub = `add;
			  end
		    end		/* (Rn+Nn) */

		  3'b110	:	
			if (pdb2[10:8] == 3'b000)
			  begin
								
				if (pdb2[15])
				  begin
					xread_t = `true;
					yread_t = `true;
				  end
				else
				  begin	
					xwrite_t = `true;
					ywrite_t = `true;
				  end
			  end		/* absolute_d address */
										
		  3'b111	:	
		    begin	/* -(Rn) */
			if (pdb2[15])
			  begin	
			  	xread_t = `true;
				yread_t = `true;
			  end
			else
			  begin	
				xwrite_t = `true;
				ywrite_t = `true;
			  end

			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;

				yab_select = `yab_take_addr_alu_H_out;
				xab_select = `xab_take_addr_alu_H_out;

				addr_alu_H_N_or_1 = `take_1;
				addr_alu_H_add_sub = `sub;
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;	

				yab_select = `yab_take_addr_alu_L_out;
				xab_select = `xab_take_addr_alu_L_out;

				addr_alu_L_N_or_1 = `take_1;
				addr_alu_L_add_sub = `sub;	
				
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end
		    end		/* -(Rn) */

		endcase	/* pdb2[13:11] */

	  end		/* L: Class I */

	else
	  begin	/*--------------------------------------*/
		/*		L: Class II		*/
		/*--------------------------------------*/
						
		/* the address is an absolute_d short address in the instruction word */
						
		xab_select = `xab_take_pdb2_13_8;
		yab_select = `yab_take_pdb2_13_8;

					
		if (pdb2[15])
		  begin		
			xread_t = `true;
			yread_t = `true;
		  end
		else
		  begin		
			xwrite_t = `true;
			ywrite_t = `true;
		  end
							
	  end		/* L: Class II */
						

	end	/* L: */





	else if ( pdb2[23:20] == 4'b0001 )
	  begin	/*--------------------------------------------------------------*/
		/*	X:R Class I  or  R:Y Class I				*/
		/*								*/
		/* X Memory and Register Data Move		Class I		*/
		/* Register and Y Memory Data Move		Class I		*/
		/*								*/
		/* Generates the effective address for a data read or write.	*/
		/* Source and Destination registers are of the Data ALU ONLY.	*/
		/* Condition codes ARE affected.				*/
		/*--------------------------------------------------------------*/
		case (pdb2[13:11])
		  3'b000 :
		    begin	/* (Rn)-Nn */
			mem_control_signals;

			if (pdb2[10])		/* effective address */
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;

				if (pdb2[14])	
					yab_select = `yab_take_Rout1_H;
				else
					xab_select = `xab_take_Rout1_H;

				addr_alu_H_N_or_1 = `take_Nn;
				addr_alu_H_add_sub = `sub;	
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;	

				if (pdb2[14])	/* memory address */
					yab_select = `yab_take_Rout1_L;
				else
					xab_select = `xab_take_Rout1_L;
				addr_alu_L_N_or_1 = `take_Nn;	
				addr_alu_L_add_sub = `sub;
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end
		  end		/* (Rn)-Nn */

		3'b001	:	
		  begin	/* (Rn)+Nn */
			mem_control_signals;

			if (pdb2[10])		/* effective address */
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;

				if (pdb2[14])		/* memory address */
					yab_select = `yab_take_Rout1_H;
				else
					xab_select = `xab_take_Rout1_H;

				addr_alu_H_N_or_1 = `take_Nn;
				addr_alu_H_add_sub = `add;
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;	

				if (pdb2[14])	/* memory address */
					yab_select = `yab_take_Rout1_L;
				else
					xab_select = `xab_take_Rout1_L;

				addr_alu_L_N_or_1 = `take_Nn;
				addr_alu_L_add_sub = `add;
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end
		    end		/* (Rn)+Nn */

		  3'b010	:
		    begin	/* (Rn)- */
			mem_control_signals;

			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;

				if (pdb2[14])
					yab_select = `yab_take_Rout1_H;
				else
					xab_select = `xab_take_Rout1_H;

				addr_alu_H_N_or_1 = `take_1;
				addr_alu_H_add_sub = `sub;	
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			 begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;

				if (pdb2[14])
					yab_select = `yab_take_Rout1_L;
				else
					xab_select = `xab_take_Rout1_L;

				addr_alu_L_N_or_1 = `take_1;
				addr_alu_L_add_sub = `sub;	
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end
		    end		/* (Rn)- */

		  3'b011	:	
		    begin	/* (Rn)+ */
			mem_control_signals;

			if (pdb2[10])	
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;

				if (pdb2[14])
					yab_select = `yab_take_Rout1_H;
				else
					xab_select = `xab_take_Rout1_H;

				addr_alu_H_N_or_1 = `take_1;
				addr_alu_H_add_sub = `add;	
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;	

			if (pdb2[14])	
				yab_select = `yab_take_Rout1_L;
			else
				xab_select = `xab_take_Rout1_L;

			addr_alu_L_N_or_1 = `take_1;	
			addr_alu_L_add_sub = `add;	
			Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
		  end
		end		/* (Rn)+ */

	    3'b100	:	
		begin	/* (Rn) */
			mem_control_signals;
				
			if (pdb2[10])
			  begin	/* High */
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				if (pdb2[14])	
					yab_select = `yab_take_Rout1_H;
				else
					xab_select = `xab_take_Rout1_H;
			  end
			else
			  begin	/* Low */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				if (pdb2[14])
					yab_select = `yab_take_Rout1_L;
				else
					xab_select = `xab_take_Rout1_L;
			  end
		    end		/* (Rn) */

		  3'b101	:	
		    begin	/* (Rn+Nn) */
			mem_control_signals;

			if (pdb2[10])	
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				if (pdb2[14])	
					yab_select = `yab_take_addr_alu_H_out;
				else
					xab_select = `xab_take_addr_alu_H_out;

				addr_alu_H_N_or_1 = `take_Nn;	
				addr_alu_H_add_sub = `add;
			  end
			else
			  begin	/* Low */
			 	Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				if (pdb2[14])
					yab_select = `yab_take_addr_alu_L_out;
				else
					xab_select = `xab_take_addr_alu_L_out;

				addr_alu_L_N_or_1 = `take_Nn;	
				addr_alu_L_add_sub = `add;
			  end
		    end		/* (Rn+Nn) */

		  3'b110	:/* Absolute_D address & Immediate_D data */
		    begin
			if ( {pdb2[15], pdb2[10:8]} == 4'b1_100 )
			  begin	 /* immediate_d data */
				
			  end
			else if (pdb2[10:8] == 3'b000)
			  begin		/* absolute_d address */
				
				if (pdb2[14])
					yread_t = `true;	/* Y: */
				else
					xread_t = `true;	/* X: */
			  end
		    end

		  3'b111	:	
		    begin	/* -(Rn) */
			/* The source register is NEVER an AGU register	*/
								
			mem_control_signals;

			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;	

				if (pdb2[14])
					yab_select = `yab_take_addr_alu_H_out;
				else
					xab_select = `xab_take_addr_alu_H_out;

				addr_alu_H_N_or_1 = `take_1;
				addr_alu_H_add_sub = `sub;
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;			

				if (pdb2[14])
					yab_select = `yab_take_addr_alu_L_out;
				else
					xab_select = `xab_take_addr_alu_L_out;

				addr_alu_L_N_or_1 = `take_1;
				addr_alu_L_add_sub = `sub;
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end
		    end		/* -(Rn) */

	    endcase	/* pdb2[13:11] */

	  end		/* X:R Class I  or  R:Y Class I */


	else if ( pdb2[23:21] == 3'b001 )
	  begin	/*--------------------------------------*/
		/* 		I:			*/
		/*					*/
		/* Immediate Short Data Move 		*/
		/* Condition Codes are NOT changed.	*/
		/*--------------------------------------*/
		/* choose either R or N register type */

		if (pdb2[20:19] == 2'b11)
		  begin	/*-------------*/
			/* N registers */
			/*-------------*/

			/* the address of the N register */

			if (pdb2[18])
				Nwaddr2_H_select_d = `Nwaddr2_H_take_pdb2_17_16;		/* to the High N rf */
			else
				Nwaddr2_L_select_d = `Nwaddr2_L_take_pdb2_17_16;		/* to the Low  N rf */


		/* the immediate data is treated as an unsigned integer */

			if (pdb2[18])
				Nin2_H_select_d= `Nin2_H_take_imm_data;				/* to the High rf */
			else
				Nin2_L_select_d = `Nin2_L_take_imm_data;					/* to the Low rf */

			/* enable write to one of N's register files */

			Nwrite2_H_3_d = pdb2[18];
			Nwrite2_L_3_d = (~pdb2[18]);
		  end		/* N */
					

		else if (pdb2[20:19] == 2'b10)
		  begin	/*-------------*/
			/* R registers */
			/*-------------*/
							
			/* the immediate data is treated as an unsigned integer */

			if (pdb2[18])	/* High */
			  begin
				Rwaddr2_H_select_d = `Rwaddr2_H_take_pdb2_17_16;
				Rwrite2_H_3_d = `true;
				Rin2_H_select_d = `Rin2_H_take_imm_data;
			  end
			else			/* Low */
			  begin
				Rwaddr2_L_select_d = `Rwaddr2_L_take_pdb2_17_16;
				Rwrite2_L_3_d = `true;
				Rin2_L_select_d = `Rin2_L_take_imm_data;
			  end
			end		/* R */
		  end	/* I: */
			

		else if ( pdb2[23:22] == 2'b01 )
		  begin	/*--------------------------------------*/
			/* 	X:	or	Y:		*/
			/*					*/
			/* X or Y Memory Data Move		*/
			/* Condition codes ARE affected		*/
			/*--------------------------------------*/
			if ( pdb2[15]  && (pdb2[13:8] != 6'b110100) )		/* it's not an immediate data mode */

			/*--------------------------*/
			/* Destination: a REGISTER	*/
			/*--------------------------*/

			/* Registers R and N are the destination in the AGU.	*/
			/* Input data arrive from the GDB 16 LS bits 		*/

			begin
				if ( pdb2[21:20] == 2'b10 )
				  begin	/*-------------------------*/
					/* Destination: R register */
					/*-------------------------*/
									
					if (pdb2[18])	/* High */
					  begin
						Rwaddr2_H_select_d = `Rwaddr2_H_take_pdb2_17_16;
						Rwrite2_H_3_d = `true;
						Rin2_H_select_d = `Rin2_H_take_GDB;
					  end
					else			/* Low */
					  begin
						Rwaddr2_L_select_d = `Rwaddr2_L_take_pdb2_17_16;
						Rwrite2_L_3_d = `true;
						Rin2_L_select_d = `Rin2_L_take_GDB;
					  end
				   end		/* R */
							

				else if ( pdb2[21:20] == 2'b11 )
				  begin	/*-------------------------*/
					/* Destination: N register */
					/*-------------------------*/
									
					if (pdb2[18])	/* High */
					  begin
						Nwaddr2_H_select_d = `Nwaddr2_H_take_pdb2_17_16;
						Nwrite2_H_3_d = `true;
						Nin2_H_select_d = `Nin2_H_take_GDB;
					  end
					else			/* Low */
					  begin
						Nwaddr2_L_select_d = `Nwaddr2_L_take_pdb2_17_16;
						Nwrite2_L_3_d = `true;
						Nin2_L_select_d = `Nin2_L_take_GDB;
					  end
				  end		/* N */

			  end		/* pdb2[15]==1	*/

		if ( (~pdb2[15]) & (pdb2[13:11] != 3'b110) )

		begin	/*----------------------------------*/
			/* Source: a register				*/
			/* Destination : X or Y data memory	*/
			/*----------------------------------*/

			/* W = 0 : Put the content of a source register on gdb_out only if	*/
			/* the effective address is not one of:	*/
			/* absolute_d address or immediate_d data.*/

			/*--------------------------*/
			/* Pick the source register */
			/*--------------------------*/

			if ( pdb2[21:20] == 2'b10 )
			  begin	/*----------------------*/
				/* Source is R register */
				/*----------------------*/
									
				gdb_write_d = `true;	/* access gdb */

				if (pdb2[13:11] == 3'b111) /* ea = Rn */
				  begin
				/* if ea and the source are the same, the newly calculated value */
				/* should be taken and not the stored one. */

					if (pdb2[10:8] == pdb2[18:16])			/* the same register */
					  begin
						/* ea and the source are the same register. */
						/* ea is used from the input to the appropriate bank -- port 1 --. */
						gdb_out_select_d = (pdb2[10]) ? `gdb_out_take_Rin1_H : `gdb_out_take_Rin1_L;
					  end
				  end		/* ea = Rn */
									
				else											/* ea != Rn and other addressing modes */
				   begin
					if (pdb2[18])	/* High */
					  begin  // Rraddr2_H_2		read address -- using port 3 --
						gdb_out_select_d = `gdb_out_take_Rout2_H;		/* take the existing value of Rn */
						Rraddr2_H_select_d = `take_pdb3_17_16;   // use pdb[17:16]
					  end
					else			/* Low */
					  begin
					// Rraddr2_L_2 		read address -- using port 3 --
						gdb_out_select_d = `gdb_out_take_Rout2_L;		/* take the existing value of Rn */
						Rraddr2_L_select_d = `take_pdb3_17_16;   // use pdb[17:16]
					  end
				   end
			  end		/* R */


			else if ( pdb2[21:20] == 2'b11 )
			/*----------------------*/
			/* Source is N register */
			/*----------------------*/
				gdb_write_d = `true;
		
			if ( pdb2[18] )		/* High */
		 	 begin
				// Nraddr2_H gets pdb2[17:16]
				gdb_out_select_d = `gdb_out_take_Nout2_H;
				Nraddr2_H_select_d = `take_pdb3_17_16; 	
		  	end
			else				/* Low */
		  	  begin 	// Nraddr2_L gets pdb2[17:16]
				gdb_out_select_d = `gdb_out_take_Nout2_L;
				Nraddr2_L_select_d = `take_pdb3_17_16;   
		  	  end		/* N */

		  end	/* choose a source */



		if (pdb2[14])
		  begin	/*------------------------------*/
			/*				*/
			/* instruction format type 1	*/
			/*				*/
			/*------------------------------*/

			/*----------------------------------------------*/
			/* Effective address calculation 		*/
			/*						*/
			/* Read from X or Y memory when the destination is a register.	*/
			/* Write to X or Y memory when the source is a register.		*/
			/*-----------------------------------------------*/

			case (pdb2[13:11])
			  3'b000	:	begin	/* (Rn)-Nn */
				mem_control_signals;

				if (pdb2[10])
				  begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
					Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

					Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;	
					Rwrite1_H_2_t = `true;

					if (pdb2[19]) /* memory access */
						yab_select = `yab_take_Rout1_H;
					else
						xab_select = `xab_take_Rout1_H;

					addr_alu_H_N_or_1 = `take_Nn;	
					addr_alu_H_add_sub = `sub;
					Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
				  end
				else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;	
					Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;	
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;	

					Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;	
					Rwrite1_L_2_t = `true;	

					if (pdb2[19])
						yab_select = `yab_take_Rout1_L;
					else
						xab_select = `xab_take_Rout1_L;

					addr_alu_L_N_or_1 = `take_Nn;
					addr_alu_L_add_sub = `sub;
					Rin1_L_select = `Rin1_L_take_addr_alu_L_out;			/* select output from addr_alu */
				  end
			    end		/* (Rn)-Nn */

			  3'b001	:	
			    begin	/* (Rn)+Nn */
				mem_control_signals;
				if (pdb2[10])
				  begin	/* High */
					Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
					Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
					Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

					Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;	
					Rwrite1_H_2_t = `true;

					if (pdb2[19])
						yab_select = `yab_take_Rout1_H;
					else
						xab_select = `xab_take_Rout1_H;

					addr_alu_H_N_or_1 = `take_Nn;	
					addr_alu_H_add_sub = `add;	
					Rin1_H_select = `Rin1_H_take_addr_alu_H_out;		
				  end
				else
				  begin	/* Low */
					Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
					Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;
					Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

					Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;	
					Rwrite1_L_2_t = `true;

					if (pdb2[19])
						yab_select = `yab_take_Rout1_L;
					else
						xab_select = `xab_take_Rout1_L;

					addr_alu_L_N_or_1 = `take_Nn;

					addr_alu_L_add_sub = `add;
					Rin1_L_select = `Rin1_L_take_addr_alu_L_out;			/* select output from addr_alu */
				  end
			  end		/* (Rn)+Nn */

		  3'b010	:	
		    begin/* (Rn)- */
			mem_control_signals;

			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;

				if (pdb2[19])
					yab_select = `yab_take_Rout1_H;
				else
					xab_select = `xab_take_Rout1_H;

				addr_alu_H_N_or_1 = `take_1;
				addr_alu_H_add_sub = `sub;
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;	

				if (pdb2[19])	
					yab_select = `yab_take_Rout1_L;
				else
					xab_select = `xab_take_Rout1_L;

				addr_alu_L_N_or_1 = `take_1;
				addr_alu_L_add_sub = `sub;
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end
			end		/* (Rn)- */

		  3'b011	:	
		    begin	/* (Rn)+ */
			mem_control_signals;									

			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;

				if (pdb2[19])
					yab_select = `yab_take_Rout1_H;
				else
					xab_select = `xab_take_Rout1_H;

				addr_alu_H_N_or_1 = `take_1;
				addr_alu_H_add_sub = `add;
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;

				if (pdb2[19])
					yab_select = `yab_take_Rout1_L;
				else
					xab_select = `xab_take_Rout1_L;

				addr_alu_L_N_or_1 = `take_1;
				addr_alu_L_add_sub = `add;
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end
		        end		/* (Rn)+ */

		  3'b100	:	
		    begin	/* (Rn) */
			mem_control_signals;		

			if (pdb2[10])
			  begin	/* High */
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				if (pdb2[19])
					yab_select = `yab_take_Rout1_H;
				else
					xab_select = `xab_take_Rout1_H;
			  end
			else
			  begin	/* Low */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				if (pdb2[19])		
					yab_select = `yab_take_Rout1_L;
				else
					xab_select = `xab_take_Rout1_L;
			  end
			end		/* (Rn) */

		  3'b101	:	
		    begin	/* (Rn+Nn) */
			mem_control_signals;								

			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				if (pdb2[19])
					yab_select = `yab_take_addr_alu_H_out;
				else
					xab_select = `xab_take_addr_alu_H_out;

				addr_alu_H_N_or_1 = `take_Nn;	
				addr_alu_H_add_sub = `add;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				if (pdb2[19])
					yab_select = `yab_take_addr_alu_L_out;
				else
					xab_select = `xab_take_addr_alu_L_out;

				addr_alu_L_N_or_1 = `take_Nn;
				addr_alu_L_add_sub = `add;	
			  end
			end		/* (Rn+Nn) */

		3'b110	:	/* Absolute_D address & Immediate_D data */
		  begin
			if ( {pdb2[15], pdb2[10:8]} == 4'b1_100 )
			/*---------------------*/
			/* immediate_d data mode */
			/*---------------------*/

			/*--------------------------*/
			/* Destination: a REGISTER	*/
			/*--------------------------*/

			/* Registers R and N are the destination in the AGU.*/
			/* Input data arrive from the GDB 16 LS bits 	*/
			  begin
				if ( pdb2[21:20] == 2'b10 )
				/*-------------------------*/							/* Destination: R register */
				/*-------------------------*/								if (pdb2[18])
					  begin	/* High */									Rwaddr2_H_select_d = `Rwaddr2_H_take_pdb2_17_16;	/* R write address */											Rwrite2_H_3_d = `true;
															  end
					else
					  begin	/* Low */
						Rwaddr2_L_select_d = `Rwaddr2_L_take_pdb2_17_16;	/* R write address */
						Rwrite2_L_3_d = `true;
																Rin2_L_select_d = `Rin2_L_take_GDB;				/* select input from GDB */
					  end		/* R */
														else if ( pdb2[21:20] == 2'b11 )							/*--------------------------*/							/* Destination: N register  */
					/*--------------------------*/

					if (pdb2[18])
					  begin	/* High */									Nwaddr2_H_select_d = `Nwaddr2_H_take_pdb2_17_16;	/* N write address */											Nwrite2_H_3_d = `true;
					  end
					else
					  begin	/* Low */
						Nwaddr2_L_select_d = `Nwaddr2_L_take_pdb2_17_16;	/* N write address */
						Nwrite2_L_3_d = `true;
																Nin2_L_select_d = `Nin2_L_take_GDB;
					  end		/* N */						  end		/* immediate_d data	*/

			else if (pdb2[10:8] == 3'b000)
			  begin	/*------------------*/	
				/* absolute_d address */
				/*------------------*/
				
				if (pdb2[19])
					yread_t = `true;
				else
					xread_t = `true;
			  end		/* absolute_d address */
												    end		/* absolute_d & immediate_d data */
							
		  3'b111	:	
		    begin	/* -(Rn) */
			mem_control_signals;								
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;	

				if (pdb2[19])
					yab_select = `yab_take_addr_alu_H_out;
				else
					xab_select = `xab_take_addr_alu_H_out;

				addr_alu_H_N_or_1 = `take_1;
				addr_alu_H_add_sub = `sub;
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;

				if (pdb2[19])
					yab_select = `yab_take_addr_alu_L_out;
				else
					xab_select = `xab_take_addr_alu_L_out;

				addr_alu_L_N_or_1 = `take_1;
				addr_alu_L_add_sub = `sub;
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end
		      end		/* -(Rn) */

		    endcase	/* pdb2[13:11] */

		  end	/* if pdb2[14]==1	instruction format type 1 */
		else		/* if pdb2[14]==0 */
		  begin	/*------------------------------*/
			/*				*/
			/* instruction format type 2	*/
			/*				*/
			/* 6 bit absolute_d short address	*/
			/*------------------------------*/

			mem_control_signals;		

			/* memory address */
			if (pdb2[19])
				yab_select = `yab_take_pdb2_13_8;
			else
				xab_select = `xab_take_pdb2_13_8;

		  end	/* if pdb2[14]==0	instruction format type 2 */

	   end	/* X:  or  Y: */



	else if ( pdb2[23] )
	  begin	/*--------------------------------------*/
		/*	X:Y:				*/
		/*					*/
		/* XY Memory Data Move			*/
		/*--------------------------------------*/

	/* the R registers that are used in ea calculations are coming from both banks, one from each using port 1. */
				
		/*------------------------------*/
		/* X memory effective address	*/
		/*------------------------------*/

		/* X memory access */

		if (pdb2[15])
			xread_t = `true;
		else
			xwrite_t  = `true;

		/* choose addressing mode */
		/* R registers: using port 1 for read and write  */

		case (pdb2[12:11])
		  2'b00	:	/* (Rn) */
		    if (pdb2[10])
		      begin	/* High */
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;
				xab_select = `xab_take_Rout1_H;
		      end
		    else
		      begin	/* Low */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;
				xab_select = `xab_take_Rout1_L;
		      end
								
		  2'b01	:	/* (Rn)+Nn */
		    	if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;	

				xab_select = `xab_take_Rout1_H;

				addr_alu_H_N_or_1 = `take_Nn;	
				addr_alu_H_add_sub = `add;
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;			

				xab_select = `xab_take_Rout1_L;

				addr_alu_L_N_or_1 = `take_Nn;	
				addr_alu_L_add_sub = `add;
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end

		  2'b10	:	/* (Rn)- */
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;

				xab_select = `xab_take_Rout1_H;

				addr_alu_H_N_or_1 = `take_1;
				addr_alu_H_add_sub = `sub;
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;	

				xab_select = `xab_take_Rout1_L;

				addr_alu_L_N_or_1 = `take_1;	
				addr_alu_L_add_sub = `sub;	
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end

		  2'b11	:	/* (Rn)+ */
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_9_8;
				Rwrite1_H_2_t = `true;			

				xab_select = `xab_take_Rout1_H;

				addr_alu_H_N_or_1 = `take_1;	
				addr_alu_H_add_sub = `add;
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_9_8;
				Rwrite1_L_2_t = `true;		

				xab_select = `xab_take_Rout1_L;

				addr_alu_L_N_or_1 = `take_1;	
				addr_alu_L_add_sub = `add;
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end

		endcase		/* X effective address */

		/*------------------------------*/
		/* Y memory effective address	*/
		/*------------------------------*/

		/* Y memory access */

		if (pdb2[22])
			yread_t = `true;
		else
			ywrite_t  = `true;


		/* choose addressing mode */
				
		/* R registers: using port 1 for read and write. Guaranteed not to write to the same bank as x: addressing does. */
				
		/* If X address is using R4-R7, Y will use R0-R3 	*/
		/* If X address is using R0-R3, Y will use R4-R7 	*/
		/* pdb[10] chooses between the two register banks.	*/ 

		case (pdb2[21:20])
		  2'b00	:	/* (Rn) */
			if (pdb2[10])
			  begin	/* Y is from Low because X is from High */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_14_13;
				yab_select = `yab_take_Rout1_L;
			  end
			else
			  begin	/* Y is from High because X is from Low */
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_14_13;			/* R read address */
				yab_select = `yab_take_Rout1_H;
			  end
								
		  2'b01	:	/* (Rn)+Nn */
			if (pdb2[10])
			  begin	/* Y is from Low because X is from High */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_14_13;
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_14_13;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_14_13;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_14_13;
				Rwrite1_L_2_t = `true;

				yab_select = `yab_take_Rout1_L;

				addr_alu_L_N_or_1 = `take_Nn;
				addr_alu_L_add_sub = `add;
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end
			else
			  begin	/* Y is from High because X is from Low */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_14_13;
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_14_13;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_14_13;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_14_13;
				Rwrite1_H_2_t = `true;

				yab_select = `yab_take_Rout1_H;

				addr_alu_H_N_or_1 = `take_Nn;	
				addr_alu_H_add_sub = `add;
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end

		  2'b10	:	/* (Rn)- */
			if (pdb2[10])
			  begin	/* Y is from Low because X is from High */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_14_13;			/* M read  address */
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_14_13;			/* R read  address */

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_14_13;			/* R write address */
				Rwrite1_L_2_t = `true;

				yab_select = `yab_take_Rout1_L;

				addr_alu_L_N_or_1 = `take_1;
				addr_alu_L_add_sub = `sub;
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end
			else
			  begin	/* Y is from High because X is from Low */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_14_13;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_14_13;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_14_13;
				Rwrite1_H_2_t = `true;

				yab_select = `yab_take_Rout1_H;

				addr_alu_H_N_or_1 = `take_1;
				addr_alu_H_add_sub = `sub;
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end

		  2'b11	:	/* (Rn)+ */
			if (pdb2[10])
			  begin	/* Y is from Low because X is from High */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_14_13;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_14_13;

				Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_14_13;
				Rwrite1_L_2_t = `true;

				yab_select = `yab_take_Rout1_L;

				addr_alu_L_N_or_1 = `take_1;
				addr_alu_L_add_sub = `add;
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
			  end
			else
			  begin	/* Y is from High because X is from Low */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_14_13;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_14_13;

				Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_14_13;
				Rwrite1_H_2_t = `true;	

				yab_select = `yab_take_Rout1_H;

				addr_alu_H_N_or_1 = `take_1;	
				addr_alu_H_add_sub = `add;
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			  end

		  endcase		/* Y effective address */

	  end		/* X:Y: */

		
	else if ( {pdb2[23:14], pdb2[7:4]} == 18'b0000_0100_01_0001)
	  begin	/*--------------------------------------*/
		/*          LUA 			*/
		/*					*/
		/* Load Updated Address                 */
		/*--------------------------------------*/

		/*------------------------------------------*/
		/*	Effective Address Calculation			*/
		/*------------------------------------------*/
				
				
		/* pdb[2] determines whether destination register is in low bank or high bank*/
		if (pdb2[2])	/* destination register is in the high bank */
		  begin
			Rwaddr1_H_select = `Rwaddr1_H_take_pdb2_1_0;	/* R write address */
			Rwrite1_H_2_t = `true;
			if (pdb2[10])
				Rin1_H_select = `Rin1_H_take_addr_alu_H_out;
			else
				Rin1_H_select = `Rin1_H_take_addr_alu_L_out;		  	  end	
		else		/* destination register is in the low bank */
		  begin
			Rwaddr1_L_select = `Rwaddr1_L_take_pdb2_1_0;
			Rwrite1_L_2_t = `true;
			if (pdb2[10])
				Rin1_L_select = `Rin1_L_take_addr_alu_H_out;
			else
				Rin1_L_select = `Rin1_L_take_addr_alu_L_out;
		  end

		 case (pdb2[13:11])
		  3'b000	:/* (Rn)-Nn */
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;
								
				addr_alu_H_N_or_1 = `take_Nn;	
				addr_alu_H_add_sub = `sub;
								
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

								
				addr_alu_L_N_or_1 = `take_Nn;	
				addr_alu_L_add_sub = `sub;
								
			  end		/* (Rn)-Nn */

		  3'b001	:	/* (Rn)+Nn */
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Nraddr1_H_select = `Nraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				addr_alu_H_N_or_1 = `take_Nn;	
				addr_alu_H_add_sub = `add;	
									
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Nraddr1_L_select = `Nraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				addr_alu_L_N_or_1 = `take_Nn;	
				addr_alu_L_add_sub = `add;
									
			  end		/* (Rn)+Nn */

		  3'b010	:	/* (Rn)- */
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;
									
								
				addr_alu_H_N_or_1 = `take_1;
				addr_alu_H_add_sub = `sub;
									
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;
									
				addr_alu_L_N_or_1 = `take_1;
				addr_alu_L_add_sub = `sub;
									
			  end		/* (Rn)- */

		  3'b011	:	/* (Rn)+ */
			if (pdb2[10])
			  begin	/* High */
				Mraddr1_H_select = `Mraddr1_H_take_pdb2_9_8;
				Rraddr1_H_select = `Rraddr1_H_take_pdb2_9_8;

				addr_alu_H_N_or_1 = `take_1;		
				addr_alu_H_add_sub = `add;		
									
			  end
			else
			  begin	/* Low */
				Mraddr1_L_select = `Mraddr1_L_take_pdb2_9_8;
				Rraddr1_L_select = `Rraddr1_L_take_pdb2_9_8;

				addr_alu_L_N_or_1 = `take_1;
				addr_alu_L_add_sub = `add;
									
			  end		/* (Rn)+ */

		  endcase					
		$display ($time, " AGU_control: LUA activated ");
	  end		/* LUA */

		
		
	else if (pdb2 == 24'b0)
		$display($time, " AGU_control: NO OPERATION");


	else	/* if no operation is required */
	  begin
		$display($time, " AGU_control: WARNING: instructions not supported!  ");
		$display($time, " AGU_control: pdb2 is: %h\n", pdb2);
	  end
										
	if (Mraddr1_L_select == 1'b0)
		indexL = pdb2[9:8];
	else
		indexL = pdb2[14:13];

	if (Mraddr1_H_select == 1'b0)
		indexH = pdb2[9:8];
	else
		indexH = pdb2[14:13];


	
      end		/* @(pdb2) */


always @(xab_select or yab_select or Rin1_L_select
        or Rin1_H_select or addr_modifier_out or indexL or indexH)
begin
	/* Guard latch enable signals  */
	if (xab_select == 2'b10 || yab_select == 2'b10 || 
  	 Rin1_L_select == 2'b00 || Rin1_H_select == 2'b10 ) 
  	 	addr_aluL_ena = 1'b1;
	else
  	 	addr_aluL_ena = 1'b0;

	if (xab_select == 2'b11 || yab_select == 2'b11 || 
  	 Rin1_L_select == 2'b10 || Rin1_H_select == 2'b00 ) 
  		 addr_aluH_ena = 1'b1;
	else
 	 	 addr_aluH_ena = 1'b0;


	if (addr_modifier_out[indexL] == 1'b1)
		 wrap_once_L_ena = 1'b1;
	else 
		 wrap_once_L_ena = 1'b0;

	if (addr_modifier_out[{1'b1, indexH}] == 1'b1)
		 wrap_once_H_ena = 1'b1;
	else 
		 wrap_once_H_ena = 1'b0;
end

/* ==================================================================== */
/* Positive flip flop instantiations					*/
/* ==================================================================== */

always @(posedge Clk)
begin
	if (reset)
	 begin
	        
		gdb_out_select <= 3'b000;
		gdb_write <= `false;

		Rraddr2_L_select <= 2'b0;
		Rraddr2_H_select <= 2'b0;
		Nraddr2_L_select <= 2'b0;
		Nraddr2_H_select <= 2'b0;
		Mraddr2_L_select <= 2'b0;
		Mraddr2_H_select <= 2'b0;
		
		Rin2_H_select <= 1'b0;
		Rwaddr2_H_select <= 1'b0;
		Rwrite2_H_3_t <= `false;
		Rin2_L_select <= 1'b0;
		Rwaddr2_L_select <= 1'b0;
		Rwrite2_L_3_t <= `false;
		Nin2_H_select <= 1'b0;
		Nwaddr2_H_select <= 1'b0;
		Nwrite2_H_3_t <= `false;
		Nin2_L_select <= 1'b0;
		Nwaddr2_L_select <= 1'b0;
		Nwrite2_L_3_t <= `false;
		Min2_H_select <= 1'b0;
		Mwaddr2_H_select <= 1'b0;
		Mwrite2_H_3_t <= `false;
		Min2_L_select <= 1'b0;
		Mwaddr2_L_select <= 1'b0;
		Mwrite2_L_3_t <= `false;

	   
			   
	end else
	begin
	   
		gdb_out_select <= gdb_out_select_d;
		gdb_write <= gdb_write_d;
		
		Rraddr2_L_select <= Rraddr2_L_select_d;
		Rraddr2_H_select <= Rraddr2_H_select_d;
		Nraddr2_L_select <= Nraddr2_L_select_d;
		Nraddr2_H_select <= Nraddr2_H_select_d;
		Mraddr2_L_select <= Mraddr2_L_select_d;
		Mraddr2_H_select <= Mraddr2_H_select_d;


		Rwrite2_H_3_t <= Rwrite2_H_3_d;
		Rwrite2_L_3_t <= Rwrite2_L_3_d;
		Nwrite2_H_3_t <= Nwrite2_H_3_d;
		Nwrite2_L_3_t <= Nwrite2_L_3_d;
		Mwrite2_H_3_t <= Mwrite2_H_3_d;
		Mwrite2_L_3_t <= Mwrite2_L_3_d;

		Rin2_L_select <= Rin2_L_select_d;
		Rin2_H_select <= Rin2_H_select_d;
		Nin2_L_select <= Nin2_L_select_d;
		Nin2_H_select <= Nin2_H_select_d;
		Min2_H_select <= Min2_H_select_d;
		Min2_L_select <= Min2_L_select_d;


		Mwaddr2_L_select <= Mwaddr2_L_select_d;
		Mwaddr2_H_select <= Mwaddr2_H_select_d;
		Nwaddr2_L_select <= Nwaddr2_L_select_d;
		Nwaddr2_H_select <= Nwaddr2_H_select_d;
		Rwaddr2_L_select <= Rwaddr2_L_select_d;
		Rwaddr2_H_select <= Rwaddr2_H_select_d;

				
	end 
end /* always */

/* add gates to disable register enables when test (scan signal) is high */

assign Rwrite2_H_3 = Rwrite2_H_3_t & !test;
assign Rwrite2_L_3 = Rwrite2_L_3_t & !test;
assign Nwrite2_H_3 = Nwrite2_H_3_t & !test;
assign Nwrite2_L_3 = Nwrite2_L_3_t & !test;
assign Mwrite2_H_3 = Mwrite2_H_3_t & !test;
assign Mwrite2_L_3 = Mwrite2_L_3_t & !test;

assign xwrite = xwrite_t & !test;
assign xread = xread_t & !test;
assign ywrite = ywrite_t & !test;
assign yread = yread_t & !test;
assign PABwrite_2 = PABwrite_2_t & !test;

assign Rwrite1_L_2 = Rwrite1_L_2_t & !test;
assign Rwrite1_H_2 = Rwrite1_H_2_t & !test;

endmodule	/* end module agu_control */
