/*	File:	agu_tristate.v	  					*/

/*	module name: agu_tristate					*/

/*	Description:	generate tristate buffer enable signals */

/*	Author: Jonathan Tong		*/


/*==============================================================================*/





   `include "./verilog/globals.v"
   `include "./verilog/ctl_def.v"


/*==============================================================================*/


/********************************************************************************/
/*																				*/
/*	module:	agu_registers																*/
/*																				*/
/********************************************************************************/
module agu_tristate(	
	gdb_out,
	gdb_write,
	PABwrite_2,
	test,
        GDB_out,				
	pab_tri_en,
	gdb_tri_en 
	);

/*==============================================================================*/
/*============	I/O direction	================================================*/
/*==============================================================================*/


input [`addrbus] gdb_out;
input gdb_write;

input PABwrite_2;

input test;

output [`databus] GDB_out;
output pab_tri_en;
output gdb_tri_en;


/*==============================================================================*/
/*============		I/O type	================================================*/
/*==============================================================================*/





wire [`addrbus] gdb_out;
wire gdb_write;

wire PABwrite_2;

wire test;

reg [`databus] GDB_out;
reg pab_tri_en;
reg gdb_tri_en;



/*==============================================================================*/
/*============		Internal Nets	============================================*/
/*==============================================================================*/



/*==============================================================================*/
/*	processes																	*/
/*==============================================================================*/


/*---------*/
/* GDB bus */
/*---------*/

/* On write permit, let the AGU drive the GDB bus, otherwise let it go (high-Z) */
always @(gdb_write or gdb_out or test)
	begin    
	        GDB_out = {8'h00, gdb_out};
		if (gdb_write == 1'b1 && test == 1'b0)
		        gdb_tri_en = 1'b0;    // tristate enabled 	   
		else
		        gdb_tri_en = 1'b1;   // tristate disabled	       
	end

/*---------*/
/* PAB bus */
/*---------*/

/* On write permit, let the AGU drive the PAB bus, otherwise let it go (high-Z) */
always @(PABwrite_2 or test)
	begin
		if (PABwrite_2 == 1'b1 && test == 1'b0)
			pab_tri_en = 1'b0;   // tristate enabled
		else
		        pab_tri_en = 1'b1;   // tristate disabled
	end

endmodule	/* end module agu_tristate */
