/* $Id: cascade_ram.v,v 1.6 1997/12/14 23:18:34 benk Exp $ */
/*****************************************************************************
 *cascade_ram.v
 *
 *Instatiates the cascade rams
 *
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: benk $
 *Date: $Date: 1997/12/14 23:18:34 $
 *Revision: $Revision: 1.6 $
 *
 *
 *Log: $Log: cascade_ram.v,v $
 *Log: Revision 1.6  1997/12/14 23:18:34  benk
 *Log: Rewired data input and output of ram cells
 *Log:
# Revision 1.5  1997/12/09  22:41:52  inacio
# added some comments and reformatted the file a bit
#
# Revision 1.4  1997/12/03  00:27:53  inacio
# corrected the in/out busses on the ram chips!
#
# Revision 1.3  1997/11/07  22:47:19  inacio
# changed some typos of adress to address
#
# Revision 1.2  1997/10/23  15:14:13  inacio
# Add necessary parameters to the rams for Cascade, changed the size of the
# address lines for the P ram to the right size.  Reformmatted some so you can
# read it.
#
# Revision 1.1  1997/10/14  20:11:30  inacio
# Initial whack at making a ram system for the chip
#
 *****************************************************************************/


module ram_system (xaddress, xdbus_in, xoe, xwr, xdbus_out,
                  yaddress, ydbus_in, yoe, ywr, ydbus_out,
                  paddress, pdbus_in, poe, pwr, pdbus_out);


   parameter		      datasize = 512;
   parameter		      daddresssize = 9;
   parameter		      progsize = 1024;
   parameter		      paddresssize = 10;
   parameter		      word_size = 24;
   parameter		      bits_per_column = 4;
   parameter		      autofloorplan = 1;

   input [(daddresssize-1):0] xaddress;
   input [(word_size-1):0]    xdbus_in;
   input		      xoe;
   input		      xwr;
   output [(word_size-1):0]   xdbus_out;
   input [(daddresssize-1):0] yaddress;
   input [(word_size-1):0]    ydbus_in;
   input		      yoe;
   input		      ywr;
   output [(word_size-1):0]   ydbus_out;
   input [(paddresssize-1):0] paddress;
   input [(word_size-1):0]    pdbus_in;
   input		      poe;
   input		      pwr;
   output [(word_size-1):0]   pdbus_out;



/*
 *ramoe #(bits/word, num_words, bits/address_line, bits/column) ram
 *    (address, data_input, output_enable, write_enable, data_output); 
 */

ramoe #(word_size, datasize, daddresssize, bits_per_column, autofloorplan) 
      xram_cell (xaddress, xdbus_in, xoe, xwr, xdbus_out);
ramoe #(word_size, datasize, daddresssize, bits_per_column, autofloorplan) 
      yram_cell (yaddress, ydbus_in, yoe, ywr, ydbus_out);
ramoe #(word_size, progsize, paddresssize, bits_per_column, autofloorplan) 
      pram_cell (paddress, pdbus_in, poe, pwr, pdbus_out);


endmodule              