/*	File:	limit.v	  						*/

/*	module name: limit						*/

/*	Description: limit */

/*		limits register output to 24 bits 	*/
/*		and shifts data 1 bit, depending on S1 and S0	*/
/*		Uses cascade modules	*/
/*	Note: does not support long data moves, lsb ignored */


/*======================================================================*/




   `include "./verilog/globals.v"


/************************************************************************/
/*									*/
/*	module:								*/
/*									*/
/************************************************************************/

module limit_c (
	in,
	S1, S0,
//	lsb,
	pass,
	out,
	L
	);


/*============	parameters	=========================================*/

   parameter	     dbgroup = "limit";


/*============	I/O direction	=========================================*/

   input [`acc]	     in;
   input	     S1, S0;			 /* status bits */
//   input	     lsb;			 /* 1 if output LSB (in0) */
   input	     pass;			 /* 1 if no limiting */
   output [`databus] out;
   output	     L;				 /* has limiting occured */

/*=======================================================================*/



/*============		I/O type	=================================*/

   wire [`databus]   out;
   wire		     L;

/*========================================================================*/

   wire [`databus]   in_shift;	/* bits of input scaled by S1, S0 */
   wire		     ext_not_equal;	/* result of ext_check */
   wire		     no_limit, no_limit_t;  /* 1 if no limiting */		     
   wire		     sign;	/* sign bit */


/*======================================================================*/
/*	processes							*/
/*======================================================================*/


/* Based on scaling mode bits, select correct portion of X register */
/*  S1  S0  |	in_shift */
/*   0   0  |  	in[47:24] */
/*   0   1  |  	in[48:25] */
/*   1   x  |  	in[46:23] */



   DW01_mux_any #(96, 2, 24) MUX3_SCALE (
	{in[46:23], in[46:23], in[48:25], in[47:24]}, {S1, S0}, in_shift);





extcheck_c EXTCHECK(
	.in(in[55:46]),
	.S1(S1), .S0(S0),
	.not_equal(ext_not_equal)
	);
assign sign = in[55];

/* do not limit if ext is equal or pass is set */

   assign no_limit_t = ~(~ext_not_equal | pass);
   assign no_limit = ~no_limit_t;





   
/* select inputs based on results of no_limit */
/*  no_limit sign |	out */
/*    0       0   |  limit positive */
/*    0       1   |  limit negative */
/*    1       x   |  do not limit   */


  DW01_mux_any #(96, 2, 24) MUX3_LIMIT({in_shift, in_shift, 24'h800000, 24'h7fffff}, {no_limit, sign}, out);   




   
/* Set L status bit */

   assign L = ~no_limit;   



	

endmodule	/* end module limit_c0 */






