/*	File:	pcu_stase2.v	  					*/

/*	module name: pcu_stage2						*/

/*	Description:	RTL description for the control logic of the PCU's pipeline stage 2. */

/*	Author: Jonathan Tong

                based on behavioral model by 
                Nitzan Weinberg			*/


/*==============================================================================*/



   `include "./verilog/globals.v"


module pcu_stage2 (
	reset,
	pdb2,
        PDB,
	REPEAT,
	PC,
	LA,
        LC,
        SP,
        PCwrite_2,
        REPEATset,
        SSLwrite_2,
        SSHwrite_2,
        SPwrite_2,
        LAwrite_2,
        PC_in_2,
        SSH_in_2,
        SSL_in_2,
        SP_in_2
	);


/*==================================================================================*/
/*============						I/O direction						============*/
/*==================================================================================*/

input reset;
input [`databus] pdb2;
input REPEAT;
input [`addrbus] PC;
input [`addrbus] LA;
input [`addrbus] LC;
input [`addrbus] SP;
input [`databus] PDB;

output PCwrite_2;
output REPEATset;
output SSLwrite_2;
output SSHwrite_2;
output SPwrite_2;
output LAwrite_2;
output [`addrbus] PC_in_2;
output [`addrbus] SSH_in_2;
output [`addrbus] SSL_in_2;
output [`addrbus] SP_in_2;




/*==================================================================================*/
/*============							I/O type						============*/
/*==================================================================================*/
reg PCwrite_2;
reg REPEATset;
reg SSLwrite_2;
reg SSHwrite_2;
reg SPwrite_2;
reg LAwrite_2;
reg [`addrbus] PC_in_2;
reg [`addrbus] SSH_in_2;
reg [`addrbus] SSL_in_2;
reg [`addrbus] SP_in_2;

/*--------------------------------------------------------------------------------------*/
/*																						*/
/* 		Logic Section																	*/
/*																						*/
/*--------------------------------------------------------------------------------------*/

always @(	pdb2 or 
			reset or
			REPEAT or
			PC or
			LA or
			LC or
			SP or
		PDB
			)
	begin

		/*--------------------------*/
		/* initialize controls		*/
		/*--------------------------*/

		PCwrite_2 = `false;

		PC_in_2 = 16'hx;
		
		SP_in_2 = 16'hx;

		SSL_in_2 = 16'hx;

		SSH_in_2 = 16'hx;
		
		REPEATset = `false;

		SSLwrite_2 = `false;

		SSHwrite_2 = `false;

		SPwrite_2 = `false;
		
		LAwrite_2 = `false;
		
		
		
		/*---------------------------------*/
		/* Decode the incoming instruction */
		/*---------------------------------*/
		
	
      /*
       * reset code added by cmi
       */
      if (reset == 1'b1)
         begin
            PC_in_2 = 0;
            PCwrite_2 = `false;
         end
      else
         begin
         
   		if ( {pdb2[23:14], pdb2[7:0]} == 18'b0000_1010_11_1000_0000 )
   			begin	/*--------------------------------------*/
   				/*	JMP Class II			*/
   				/*					*/
   				/* Jump with an effective address		*/
   				/*--------------------------------------*/
   					
   				/* The effective address is generated by the AGU */

   					
   				if (pdb2[13:8] == 6'b110000)
   					begin
   						//PC_in_2 = PC + 1;
   					   $display ($time, " PCU: JMP Class II activated.  pdb = %h", pdb2);
   					   
   						PC_in_2 = PDB[15:0];					/* read the absolute address from program memory */
   						PCwrite_2 = `true;

   					end
   				
   			end		/* JMP Class II */




   		else if ( {pdb2[23:14], pdb2[7:0]} == 18'b00000110_11_00100000 )
   			begin	/*--------------------------------------*/
   					/*				REP Class IV			*/
   					/*										*/
   					/* Repeat Next Instruction				*/
   					/*--------------------------------------*/

   				/* number of repetitions in coming from the AGU, data ALU or the PCU itself */
   				
   				REPEATset = `true;	/* set the repeat signal */

   				PC_in_2 = PC + 1;
   				PCwrite_2 = `true;

   			end		/* REP Class IV */




   		else if ( {pdb2[23:15], pdb2[7], pdb2[5:0]} == 16'b00000110_0_0_100000 )
   			begin	/*--------------------------------------*/
   					/*			REP Class I or II			*/
   					/*										*/
   					/* Repeat Next Instruction				*/
   					/*--------------------------------------*/
   					
   				/* The effective address is generated by the AGU */

   				REPEATset = `true;	/* set the repeat signal */

   				PC_in_2 = PC + 1;
   				PCwrite_2 = `true;
   				
   			end		/* REP Class I or II */




   		else if ( {pdb2[23:16], pdb2[7:2]} == 14'b00000000_101110 )
   			begin	/*--------------------------------------*/
   					/*					ANDI				*/
   					/*										*/
   					/* No Action							*/
   					/*--------------------------------------*/

   			end	/* ANDI */




   		else if ( {pdb2[23:16], pdb2[7:4]} == 12'b00000110_1010 )
   			begin	/*--------------------------------------*/
   					/*			REP Class III				*/
   					/*										*/
   					/* Repeat Next Instruction				*/
   					/*--------------------------------------*/
   					
   				/* Immediate data in the program instruction */

   				REPEATset = `true;	/* set the repeat signal */

   				PC_in_2 = PC + 1;
   				PCwrite_2 = `true;
   				
   			end		/* REP Class III */




   		else if ( pdb2[23:12] == 12'b0000_1100_0000 )
   			begin	/*--------------------------------------*/
   					/*				JMP Class I				*/
   					/*										*/
   					/* Jump with a short immediate address	*/
   					/*--------------------------------------*/
   				
   				/* an absolute address is encoded in the program word */
   				
   				PC_in_2 = {4'h0, pdb2[11:0]};		/* zero extended */
   				PCwrite_2 = `true;
   				
   			end		/* JMP Class I */
   			



   		else if ( {pdb2[23:16], pdb2[7:4]} == 12'b0000_0110_1000 || {pdb2[23:14], pdb2[7:0]} == 18'b0000_0110_11_0000_0000 )
   			begin	/*--------------------------------------*/
   					/*			DO Class III				*/
   					/*			DO Class IV				*/
   					/* Start Hardware Loop					*/
   					/*--------------------------------------*/
   				
   				/* push LA and LC into the system stack */
   				
   				SSH_in_2 = LA;			/* save the current LA into SSH */
   				SSHwrite_2 = `true;
   				
   				SSL_in_2 = LC;			/* save the current LC into SSL */
   				SSLwrite_2 = `true;
   				
   				SP_in_2 = SP[5:0] + 1'h1;	/* adjust the stack pointer, only 6 LSBs  */
   				SPwrite_2 = `true;
   				
   				LAwrite_2 = `true;		/* latch the new address loop */

   				/* An absolute address is ALWAYS supplied in the following program word. */
   				/* This is the end-of-the-loop expression */
   				
   		
   				
   				/* advance the PC */
   				
   				PC_in_2 = PC + 1'b1;
   				PCwrite_2 = `true;
   				
   			end		/* DO Class III */
   			



   		else if ( ({pdb2[23:14], pdb2[7]}   == 11'b00001010_01_1) ||
   				  ({pdb2[23:14], pdb2[7]}   == 11'b00001010_00_1) ||
   				  ({pdb2[23:14], pdb2[7:6]} == 12'b00001010_11_00) )
   			begin	/*----------------------------------------------*/
   					/*		JCLR / JSET Class I, II	or III			*/
   					/*												*/
   					/*	Jump if Bit Clear/Set - Class I, II or III	*/
   					/*----------------------------------------------*/


   				/* bring the next program word which is an absolute jump address */
   				
   				PC_in_2 = PC + 1'b1;
   				PCwrite_2 = `true;


   			end		/* JCLR/JSET Class I, II or III */




   		else if ( ({pdb2[23:14], pdb2[7]} == 11'b00001010_01_0) ||
   			 	  ({pdb2[23:14], pdb2[7], pdb2[5]} == 12'b00001011_01_0_1) )
   			 
   			begin	/*--------------------------------------*/
   					/*		BCLR / BSET / BTST Class I		*/
   					/*										*/
   					/*	Bit Test and Clear/Set - Class I	*/
   					/*--------------------------------------*/
   				
   				/* bring the next program word */
   				
   				PC_in_2 = PC + 1'b1;
   				PCwrite_2 = `true;

   			end		/* BCLR/BSET/BTST Class I */




   		else if ( {pdb2[23:16], pdb2[7:5]} == 11'b00000101_101 )
   			begin	/*----------------------------------*/
   					/*		MOVEC Class IV				*/
   					/*									*/
   					/* Move Control Register	 		*/
   					/*----------------------------------*/

   				/* bring the next program word */
   				
   				PC_in_2 = PC + 1'b1;
   				PCwrite_2 = `true;

   			end		/* MOVEC  Class IV */




   		else if ( {pdb2[23:16], pdb2[7], pdb2[5]} == 10'b00000101_0_1 )
   			begin	/*----------------------------------*/
   					/*		MOVEC Class I or II			*/
   					/*									*/
   					/* Move Control Register	 		*/
   					/*----------------------------------*/


   				/* bring the next program word */
   				
   				PC_in_2 = PC + 1'b1;
   				PCwrite_2 = `true;

   			end		/* MOVEC Class I and II */




   		else if ( {pdb2[23:20], pdb2[18]} == 5'b0100_0 )
   			begin	/*----------------------------------*/
   					/* 				L:					*/
   					/*----------------------------------*/

   	
   				/* bring the next program word */
   				
   				PC_in_2 = PC + 1'b1;
   				PCwrite_2 = `true;
   				
   			end		/* L: */




   		else if ( pdb2[23:20] == 4'b0001 )
   			begin	/*--------------------------------------------------------------*/
   					/*				X:R Class I  or  R:Y Class I					*/
   					/*																*/
   					/* X Memory and Register Data Move		Class I					*/
   					/* Register and Y Memory Data Move		Class I					*/
   					/*																*/
   					/*--------------------------------------------------------------*/

   				if ({pdb2[15], pdb2[13:11]} == 4'b1_110)

   	
   				/* bring the next program word */
   				
   				PC_in_2 = PC + 1'b1;
   				PCwrite_2 = `true;

   			end		/* X:R Class I  or  R:Y Class I */




   		else if ( pdb2[23:22] == 2'b01 )
   			begin	/*----------------------------------*/
   					/* 			X:	or	Y:				*/
   					/*									*/
   					/* X or Y Memory Data Move			*/
   					/* Condition codes ARE affected		*/
   					/*----------------------------------*/

   	
   				/* bring the next program word */
   				
   				PC_in_2 = PC + 1'b1;
   				PCwrite_2 = `true;

   			end	/* X: or Y: */



   		else
   			begin	/*--------------------------------------*/
   					/*				default					*/
   					/*										*/
   					/* increment the program address by one */
   					/*--------------------------------------*/
   					
   				PC_in_2 = (REPEAT) ? PC : (PC + 1'b1);		/* while REP is in progress don't advance the PC */
   				PCwrite_2 = `true;
   			end

      end /* if reset */
			
	end		/* @(pdb2) */


endmodule


