/*	File:	selector_5_to_1_16bit.v	  					*/

/*	module name: selector_5_to_1_16bit					*/

/*	Description:	A generic selector between 5 inputs of 16 bit each, towards a single output of 16 bit. */

/*	Author: Nitzan Weinberg			*/


/*==============================================================================*/


/*==============================================================================*/


/********************************************************************************/
/*																				*/
/*	module:	selector_5_to_1_16bit												*/
/*																				*/
/********************************************************************************/

module selector_5_to_1_16bit (
	in0,
	in1,
	in2,
	in3,
	in4,
	control,
	out
	);


/*==============================================================================*/
/*============	I/O direction	================================================*/
/*==============================================================================*/

input [15:0] in0;
input [15:0] in1;
input [15:0] in2;
input [15:0] in3;
input [15:0] in4;
input [2:0] control;

output [15:0] out;



/*==============================================================================*/
/*============		I/O type	================================================*/
/*==============================================================================*/

wire [15:0] in0;
wire [15:0] in1;
wire [15:0] in2;
wire [15:0] in3;
wire [15:0] in4;
wire [2:0] control;		/* (control = 0) => (out = in0)		*/
						/* (control = 1) => (out = in1)		*/
						/* (control = 2) => (out = in2)		*/
						/* (control = 3) => (out = in3)		*/
						/* (control = 4) => (out = in4)		*/

wire [15:0] out;


/*==============================================================================*/
/*============		Internal Nets	============================================*/
/*==============================================================================*/

wire [15:0] temp;		/* holds the output from m4 */


/*==============================================================================*/
/*============		Instantiated Modules	====================================*/
/*==============================================================================*/

/* chooses among the 4 inputs in3-in0 */

mux4
	#(16,0) m4 (
	.IN0 (in0),
	.IN1 (in1),
	.IN2 (in2),
	.IN3 (in3),
	.S0 (control[0]),
	.S1 (control[1]),
	.Y (temp)
	);
	

/* chooses between the output from m4 and the fifth input (in4) */
	
mux2
	#(16,0) m2 (
	.IN0 (temp),
	.IN1 (in4),
	.S0 (control[2]),
	.Y (out)
	);
	


endmodule	/* end module selector_5_to_1_16bit */








