/*	File:	shift1.v	  		*/

/*	module name: shift1			*/

/*	Description: 1 bit shifter module for ALU 		*/

/*		    Arithmetically shift one bit left or right	*/
/*		    Uses Cascade modules			*/

/*	Unused zero output when {right, left} = 2'b11 	*/


/*=====================================================================*/



   `include "./verilog/globals.v"


/************************************************************************/
/*						                        */
/*	module:			             				*/
/*									*/
/************************************************************************/

module shift1_c (
		 left, right,
		 in,    /* in2, ext:msb:lsb */
		 out, mac_acc
		 );


/*============	I/O direction	================================================*/

input		left, right;
input [`acc]	in;
output [`acc]	out, mac_acc;

/*==============================================================================*/


/*============		I/O type	=================================*/

wire [`acc] out;		

/*========================================================================*/


/*======================================================================*/
/*	processes							*/
/*======================================================================*/

   /*   left  right   out    mac_acc 
    *     0     0     in     in
    *     0     1     in>>1  in
    *     1     0     in<<1  0
    *     1     1     0      0
    */

   DW01_mux_any #(224, 2, 56) SHIFT1_MUX (
	{56'b0, {in[55], in[55:1]}, {in[54:0], 1'b0}, in}, {right, left}, out);






   DW01_mux_any #(112, 1, 56) MAC_ACC_MUX (
	{56'b0, in}, left, mac_acc);




   
endmodule	/* end module */
